import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {AppHelper} from "@app/app.helper";
import {PageNotFoundPresentational} from "@app/components/presentationals/page-not-found/page-not-found.presentational";
import {IconAppRoutable} from "@app/components/routables/icon-app/icon-app.routable";
import {ApplicationRolePermissionEnum} from "@app/shared/enums/application-role-permission.enum";
import {AppRoutesEnum} from "@app/shared/enums/routes.enum";
import {ApplicationRoleGuardService} from "@app/shared/guards/application-role-guard.service";
import {DlcmRoutes} from "@app/shared/models/dlcm-route.model";
import {ApplicationDevGuardService} from "@shared/guards/application-dev-guard.service";
import {UrlQueryParamHelper} from "@shared/helpers/url-query-param.helper";

const routes: DlcmRoutes = [
  {
    path: AppRoutesEnum.homePage,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/home/home.module").then(m => m.HomeModule),
    data: {
      permission: ApplicationRolePermissionEnum.noPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.deposit,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/deposit/deposit.module").then(m => m.DepositModule),
    data: {
      breadcrumb: AppHelper.deposit,
      permission: ApplicationRolePermissionEnum.userPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.preservationSpace,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/preservation-space/preservation-space.module").then(m => m.PreservationSpaceModule),
    data: {
      breadcrumb: AppHelper.preservationSpace,
      permission: ApplicationRolePermissionEnum.userPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.admin,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/admin/admin.module").then(m => m.AdminModule),
    data: {
      breadcrumb: AppHelper.admin,
      permission: ApplicationRolePermissionEnum.adminPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.preservation,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/preservation/preservation.module").then(m => m.PreservationModule),
    data: {
      breadcrumb: AppHelper.preservationPlanning,
      permission: ApplicationRolePermissionEnum.adminPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.order,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/order/order.module").then(m => m.OrderModule),
    data: {
      breadcrumb: AppHelper.order,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.icons,
    component: IconAppRoutable,
    canActivate: [ApplicationDevGuardService],
  },
  {
    path: AppRoutesEnum.root,
    redirectTo: "/" + AppRoutesEnum.homePage + UrlQueryParamHelper.getQueryParamOAuth2(),
    pathMatch: "full",
  },
  {
    path: AppRoutesEnum.index,
    redirectTo: "/" + AppRoutesEnum.homePage + UrlQueryParamHelper.getQueryParamOAuth2(),
    pathMatch: "full",
  },
  {
    path: "**",
    component: PageNotFoundPresentational,
  },
];

@NgModule({
  imports: [RouterModule.forRoot(routes, {
    useHash: true,
  })],
  exports: [RouterModule],
})
export class AppRoutingModule {
}
