import {
  ChangeDetectionStrategy,
  Component,
} from "@angular/core";
import {MatDialogRef} from "@angular/material/dialog";
import {AppCartAction} from "@app/stores/cart/app-cart.action";
import {AppCartState} from "@app/stores/cart/app-cart.state";
import {AppCartArchiveState} from "@app/stores/cart/archive/app-cart-archive.state";
import {SearchScopeEnum} from "@home/enums/search-scope.enum";
import {
  Actions,
  ofActionCompleted,
  Store,
} from "@ngxs/store";
import {SharedAbstractContainer} from "@shared/components/containers/shared-abstract/shared-abstract.container";
import {ArchiveMetadata} from "@shared/models/business/archive-metadata.model";
import {MetadataUtil} from "@shared/utils/metadata.util";
import {Observable} from "rxjs";
import {tap} from "rxjs/operators";
import {
  isTrue,
  MemoizedUtil,
  ResourceState,
} from "solidify-frontend";

@Component({
  selector: "dlcm-cart-dialog",
  templateUrl: "./cart.dialog.html",
  styleUrls: ["./cart.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class CartDialog extends SharedAbstractContainer {
  isLoadingCartObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, AppCartState as any);
  isLoadingArchiveObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, AppCartArchiveState);
  listArchiveObs: Observable<ArchiveMetadata[]> = ResourceState.list(this._store, AppCartArchiveState);

  get searchScopeEnum(): typeof SearchScopeEnum {
    return SearchScopeEnum;
  }

  get metadataUtil(): typeof MetadataUtil {
    return MetadataUtil;
  }

  constructor(private readonly _dialogRef: MatDialogRef<CartDialog>,
              private readonly _store: Store,
              private readonly _actions$: Actions) {
    super();
  }

  submit(): void {
    this._store.dispatch(new AppCartAction.Submit());
    this.subscribe(this._actions$.pipe(
      ofActionCompleted(AppCartAction.SubmitSuccess),
      tap((result) => {
        if (isTrue(result.result.successful)) {
          this._dialogRef.close();
        }
      }),
    ));
  }

  remove(resId: string): void {
    this._store.dispatch(new AppCartAction.RemoveToCart(resId));
  }
}
