import {
  ChangeDetectionStrategy,
  Component,
} from "@angular/core";
import {SharedAbstractContainer} from "@shared/components/containers/shared-abstract/shared-abstract.container";
import {
  ClipboardUtil,
  NotificationService,
  OAuth2Service,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-token-dialog",
  templateUrl: "./token.dialog.html",
  styleUrls: ["./token.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class TokenDialog extends SharedAbstractContainer {
  get token(): string {
    return this.oauth2Service.getAccessToken();
  }

  constructor(public oauth2Service: OAuth2Service,
              public notificationService: NotificationService) {
    super();
  }

  copy(): void {
    if (ClipboardUtil.copyStringToClipboard(this.token)) {
      this.notificationService.showInformation(MARK_AS_TRANSLATABLE("app.notification.tokenCopiedToClipboardSuccess"), true);
    } else {
      this.notificationService.showWarning(MARK_AS_TRANSLATABLE("app.notification.tokenCopiedToClipboardFail"));
    }
  }
}
