import {PersonExtended} from "@admin/models/person-extended.model";
import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
  ViewChild,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {AppPersonAction} from "@app/stores/person/app-person.action";
import {
  Person,
  User,
} from "@models";
import {
  Select,
  Store,
} from "@ngxs/store";
import {SharedAbstractContainer} from "@shared/components/containers/shared-abstract/shared-abstract.container";
import {SharedPersonFormPresentational} from "@shared/components/presentationals/shared-person-form/shared-person-form.presentational";
import {LocalStateModel} from "@shared/models/local-state.model";
import {Observable} from "rxjs";
import {
  ModelFormControlEvent,
  StoreUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-user-dialog",
  templateUrl: "./user.dialog.html",
  styleUrls: ["./user.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class UserDialog extends SharedAbstractContainer implements OnInit {
  @Select((state: LocalStateModel) => state.application.application_person.current) currentPersonObs: Observable<Person>;
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state.application.application_person)) isLoadingPersonObs: Observable<boolean>;

  @ViewChild("formPresentational")
  readonly formPresentational: SharedPersonFormPresentational;

  constructor(protected store: Store,
              protected dialogRef: MatDialogRef<UserDialog>,
              @Inject(MAT_DIALOG_DATA) public user: User) {
    super();
  }

  ngOnInit(): void {
    // Not necessary to refresh current user and cause ExpressionChangedAfterItHasBeenCheckedError when open profile info in deposit-edit.routable (cf DLCM-795)
    // this.store.dispatch(new AppPersonAction.GetById(this.user.person.resId));
  }

  savePersonInfo($event: ModelFormControlEvent<PersonExtended>): void {
    this.store.dispatch(new AppPersonAction.Update($event));
    // this.dialogRef.close();
  }
}
