import {
  ChangeDetectionStrategy,
  Component,
  Input,
  Output,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {TokenDialog} from "@app/components/dialogs/token/token.dialog";
import {UserDialog} from "@app/components/dialogs/user/user.dialog";
import {MenuToolbar} from "@app/components/presentationals/main-toolbar/abstract-main-toolbar/abstract-main-toolbar.presentational";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {User} from "@models";
import {ThemeEnum} from "@shared/enums/theme.enum";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  isNullOrUndefined,
  ObservableUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-avatar",
  templateUrl: "./avatar.presentational.html",
  styleUrls: ["./avatar.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AvatarPresentational extends SharedAbstractPresentational {
  private _user: User;

  @Input()
  mode: "horizontal" | "vertical" = "vertical";

  private get _theme(): ThemeEnum | undefined {
    return this._themeChangeBS.getValue();
  }

  private set _theme(value: ThemeEnum | undefined) {
    this._themeChangeBS.next(value);
  }

  get theme(): ThemeEnum | undefined {
    return this._theme;
  }

  @Input()
  set theme(theme: ThemeEnum | undefined) {
    this._theme = theme;
  }

  @Input()
  set user(value: User) {
    this._user = value;
    this.computeInitial();
  }

  @Input()
  listMenuAdmin: MenuToolbar[];

  get user(): User {
    return this._user;
  }

  private readonly DEFAULT_INITIAL: string = "U";

  initial: string = this.DEFAULT_INITIAL;

  // Logout
  private readonly _logoutBS: BehaviorSubject<void | undefined> = new BehaviorSubject<void | undefined>(undefined);
  @Output("logoutChange")
  readonly logoutObs: Observable<void | undefined> = ObservableUtil.asObservable(this._logoutBS);

  private readonly _userGuideOpenBS: BehaviorSubject<void | undefined> = new BehaviorSubject<void | undefined>(undefined);
  @Output("userGuideOpen")
  readonly userGuideOpenObs: Observable<void | undefined> = ObservableUtil.asObservable(this._userGuideOpenBS);

  private readonly _themeChangeBS: BehaviorSubject<ThemeEnum | undefined> = new BehaviorSubject<ThemeEnum | undefined>(undefined);
  @Output("themeChange")
  readonly themeChangeObs: Observable<ThemeEnum | undefined> = ObservableUtil.asObservable(this._themeChangeBS);

  constructor(private dialog: MatDialog) {
    super();
  }

  logout(): void {
    this._logoutBS.next();
  }

  displayToken(): void {
    this.dialog.open(TokenDialog);
  }

  computeInitial(): void {
    if (isNullOrUndefined(this._user)) {
      this.initial = this.DEFAULT_INITIAL;
    } else {
      this.initial = this._user.firstName.substring(0, 1) + this._user.lastName.substring(0, 1);
    }
  }

  profileInfo(): void {
    this.dialog.open(UserDialog, {
      data: this._user,
      width: "90%",
    });
  }

  openUserGuide(): void {
    this._userGuideOpenBS.next();
  }

  toggleDarkMode(): void {

  }
}
