import {
  ChangeDetectionStrategy,
  Component,
  Input,
  Output,
} from "@angular/core";
import {User} from "@models";
import {SharedAbstractPresentational} from "@shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ApplicationRoleEnum} from "@shared/enums/application-role.enum";
import {LanguagesEnum} from "@shared/enums/languages.enum";
import {ThemeEnum} from "@shared/enums/theme.enum";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {ObservableUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-main-toolbar",
  templateUrl: "./main-toolbar.presentational.html",
  styleUrls: ["./main-toolbar.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class MainToolbarPresentational extends SharedAbstractPresentational {
  @Input()
  logged: boolean = false;

  @Input()
  currentPath: string;

  @Input()
  currentLanguage: LanguagesEnum;

  @Input()
  userRoles: ApplicationRoleEnum[];

  @Input()
  user: User;

  @Input()
  numberArchiveInCart: number;

  @Input()
  numberMyOrderReadyNew: number;

  @Input()
  numberPendingRequestNotificationInbox: number;

  @Input()
  theme: ThemeEnum;

  @Input()
  logo: string;

  @Input()
  displayCart: boolean;

  @Input()
  isHomePage: boolean;

  private readonly _themeBS: BehaviorSubject<ThemeEnum | undefined> = new BehaviorSubject<ThemeEnum | undefined>(undefined);
  @Output("themeChange")
  readonly themeObs: Observable<ThemeEnum | undefined> = ObservableUtil.asObservable(this._themeBS);

  // Navigate
  private readonly _navigateBS: BehaviorSubject<string | undefined> = new BehaviorSubject<string | undefined>(undefined);
  @Output("navigateChange")
  readonly navigateObs: Observable<string | undefined> = ObservableUtil.asObservable(this._navigateBS);

  // Language
  private readonly _languageBS: BehaviorSubject<LanguagesEnum | undefined> = new BehaviorSubject<LanguagesEnum | undefined>(undefined);
  @Output("languageChange")
  readonly languageObs: Observable<LanguagesEnum | undefined> = ObservableUtil.asObservable(this._languageBS);

  // Logout
  private readonly _logoutBS: BehaviorSubject<void | undefined> = new BehaviorSubject<void | undefined>(undefined);
  @Output("logoutChange")
  readonly logoutObs: Observable<void | undefined> = ObservableUtil.asObservable(this._logoutBS);

  // Cart
  private readonly _cartBS: BehaviorSubject<void | undefined> = new BehaviorSubject<void | undefined>(undefined);
  @Output("cartChange")
  readonly cartObs: Observable<void | undefined> = ObservableUtil.asObservable(this._cartBS);

  // UserGuide
  private readonly _userGuideOpenBS: BehaviorSubject<void | undefined> = new BehaviorSubject<void | undefined>(undefined);
  @Output("userGuideOpen")
  readonly userGuideOpenObs: Observable<void | undefined> = ObservableUtil.asObservable(this._userGuideOpenBS);

  //Search
  private readonly _searchBS: BehaviorSubject<string | undefined> = new BehaviorSubject<string | undefined>(undefined);
  @Output("searchChange")
  readonly searchObs: Observable<string | undefined> = ObservableUtil.asObservable(this._searchBS);

  changeTheme(theme: ThemeEnum): void {
    if (this._themeBS.value !== theme) {
      this._themeBS.next(theme);
    }
  }

  changeLanguage(language: LanguagesEnum): void {
    if (this._languageBS.value !== language) {
      this._languageBS.next(language);
    }
  }

  changeNavigate(navigate: string): void {
    this._navigateBS.next(navigate);
  }

  changeCart(): void {
    this._cartBS.next();
  }

  logout(): void {
    this._logoutBS.next();
  }

  openUserGuide(): void {
    this._userGuideOpenBS.next();
  }

  search(value: string): void {
    this._searchBS.next(value);
  }
}
