import {NgModule} from "@angular/core";
import {
  RouterModule,
  Routes,
} from "@angular/router";
import {AdminHomeRoutable} from "@app/features/admin/components/routables/admin-home/admin-home.routable";
import {
  AdminRoutesEnum,
  AppRoutesEnum,
} from "@app/shared/enums/routes.enum";
import {ApplicationRoleGuardService} from "@app/shared/guards/application-role-guard.service";
import {ApplicationRolePermissionEnum} from "@shared/enums/application-role-permission.enum";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

const routes: Routes = [
  {
    path: AppRoutesEnum.root,
    component: AdminHomeRoutable,
    data: {},
  },
  {
    path: AdminRoutesEnum.license,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./license/license.module").then(m => m.AdminLicenseModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.license.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.institution,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./institution/institution.module").then(m => m.AdminInstitutionModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.institution.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.researchDomain,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./research-domain/research-domain.module").then(m => m.AdminResearchDomainModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.researchDomain.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.fundingAgencies,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./funding-agencies/funding-agencies.module").then(m => m.AdminFundingAgenciesModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.funding-agencies.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.oaiSet,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./oai-set/oai-set.module").then(m => m.AdminOaiSetModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.oai-set.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.oauth2client,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./oauth2-client/oauth2-client.module").then(m => m.AdminOauth2ClientModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.oauth2Client.root"),
      permission: ApplicationRolePermissionEnum.rootPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.organizationalUnit,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./orgunit/orgunit.module").then(m => m.AdminOrgunitModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.organizationalUnit.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.submissionPolicy,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./submission-policy/submission-policy.module").then(m => m.AdminSubmissionPolicyModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.submissionPolicy.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.person,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./person/person.module").then(m => m.AdminPersonModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.person.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.preservationPolicy,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./preservation-policy/preservation-policy.module").then(m => m.AdminPreservationPolicyModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.preservationPolicy.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.disseminationPolicy,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./dissemination-policy/dissemination-policy.module").then(m => m.AdminDisseminationPolicyModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.disseminationPolicy.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.role,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./role/role.module").then(m => m.AdminRoleModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.role.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.notification,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./notification/notification.module").then(m => m.AdminNotificationModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.notification.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.user,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./user/user.module").then(m => m.AdminUserModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.user.root"),
      permission: ApplicationRolePermissionEnum.rootPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.language,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./language/language.module").then(m => m.AdminLanguageModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.language.root"),
      permission: ApplicationRolePermissionEnum.rootPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.metadataType,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./metadata-type/metadata-type.module").then(m => m.AdminMetadataTypeModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.admin.metadataType.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class AdminRoutingModule {
}
