import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Input,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {FundingAgencyExtended} from "@deposit/models/funding-agency-extended.model";
import {OrganizationalUnit} from "@models";
import {SharedAbstractFormPresentational} from "@shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {LocalModelAttributeEnum} from "@shared/enums/model-attribute.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {RegexpUtil} from "@shared/utils/regexp.util";
import _ from "lodash";
import {
  isNullOrUndefined,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-funding-agencies-form",
  templateUrl: "./admin-funding-agencies-form.presentational.html",
  styleUrls: ["../../../../../../shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminFundingAgenciesFormPresentational extends SharedAbstractFormPresentational<FundingAgencyExtended> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  @Input()
  selectedOrgUnits: OrganizationalUnit[];

  @Input()
  listOrgUnits: OrganizationalUnit[];

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              private readonly _fb: FormBuilder) {
    super(_changeDetectorRef, _elementRef);
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.acronym]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.name]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: ["", [SolidifyValidator]],
      [this.formDefinition.url]: ["", [SolidifyValidator, Validators.pattern(RegexpUtil.url)]],
      [this.formDefinition.organizationalUnits]: [],
    });
  }

  protected bindFormTo(fundingAgency: FundingAgencyExtended): void {
    this.form = this._fb.group({
      [this.formDefinition.acronym]: [fundingAgency.acronym, [Validators.required, SolidifyValidator]],
      [this.formDefinition.name]: [fundingAgency.name, [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: [fundingAgency.description, [SolidifyValidator]],
      [this.formDefinition.url]: [fundingAgency.url, [SolidifyValidator, Validators.pattern(RegexpUtil.url)]],
      [this.formDefinition.organizationalUnits]: [_.map(this.selectedOrgUnits, LocalModelAttributeEnum.resId)],
    });
  }

  protected treatmentBeforeSubmit(fundingAgency: FundingAgencyExtended): FundingAgencyExtended {
    fundingAgency.organizationalUnits = [];
    const listOrgUnitId = this.form.get(this.formDefinition.organizationalUnits).value;
    if (!isNullOrUndefined(listOrgUnitId)) {
      listOrgUnitId.forEach(resId => {
        fundingAgency.organizationalUnits.push({resId: resId});
      });
    }
    return fundingAgency;
  }

  navigateToOrgUnit(organizationalUnit: OrganizationalUnit): void {
    this.navigate([RoutesEnum.adminOrganizationalUnitDetail, organizationalUnit.resId]);
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() acronym: string;
  @PropertyName() name: string;
  @PropertyName() description: string;
  @PropertyName() organizationalUnits: string;
  @PropertyName() url: string;
}
