import {adminFundingAgenciesActionNameSpace} from "@admin/funding-agencies/stores/admin-funding-agencies.action";
import {AdminFundingAgenciesStateModel} from "@admin/funding-agencies/stores/admin-funding-agencies.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {FundingAgency} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RouterExtService} from "@shared/services/router-ext.service";
import {
  OrderEnum,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-submission-policy-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminFundingAgenciesListRoutable extends SharedAbstractListRoutable<FundingAgency, AdminFundingAgenciesStateModel> {
  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("admin.funding-agencies.button.new");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("admin.funding-agencies.button.refresh");
  readonly KEY_BACK_BUTTON: string | undefined = MARK_AS_TRANSLATABLE("admin.button.goBackToAdminHome");
  readonly KEY_PARAM_NAME: keyof FundingAgency & string = "name";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.admin_fundingAgencies, adminFundingAgenciesActionNameSpace, {}, LocalStateEnum.admin);
  }

  conditionDisplayEditButton(model: FundingAgency | undefined): boolean {
    return true;
  }

  conditionDisplayDeleteButton(model: FundingAgency | undefined): boolean {
    return true;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "acronym",
        header: MARK_AS_TRANSLATABLE("admin.funding-agencies.table.header.acronym"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "name",
        header: MARK_AS_TRANSLATABLE("admin.funding-agencies.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }

}
