import {MetadataTypeFormatEnumHelper} from "@admin/metadata-type/helpers/metadata-type-format-enum.helper";
import {adminMetadataTypeActionNameSpace} from "@admin/metadata-type/stores/admin-metadata-type.action";
import {AdminMetadataTypeStateModel} from "@admin/metadata-type/stores/admin-metadata-type.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {MetadataType} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RouterExtService} from "@shared/services/router-ext.service";
import {
  OrderEnum,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-metadata-type-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminMetadataTypeListRoutable extends SharedAbstractListRoutable<MetadataType, AdminMetadataTypeStateModel> {
  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("admin.metadataType.button.new");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("admin.metadataType.button.refresh");
  readonly KEY_BACK_BUTTON: string | undefined = MARK_AS_TRANSLATABLE("admin.button.goBackToAdminHome");
  readonly KEY_PARAM_NAME: keyof MetadataType & string = "name";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.admin_metadataType, adminMetadataTypeActionNameSpace, {}, LocalStateEnum.admin);
  }

  conditionDisplayEditButton(model: MetadataType | undefined): boolean {
    return true;
  }

  conditionDisplayDeleteButton(model: MetadataType | undefined): boolean {
    return true;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "name",
        header: MARK_AS_TRANSLATABLE("admin.metadataType.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "metadataFormat",
        header: MARK_AS_TRANSLATABLE("admin.metadataType.table.header.format"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: MetadataTypeFormatEnumHelper.getListKeyValue(),
      },
      {
        field: "version",
        header: MARK_AS_TRANSLATABLE("admin.metadataType.table.header.version"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      // {
      //   field: "openMetadataTypeId",
      //   header: MARK_AS_TRANSLATABLE("admin.metadataType.table.header.openMetadataTypeId"),
      //   type: FieldTypeEnum.string,
      //   order: OrderEnum.none,
      //   isFilterable: false,
      //   isSortable: true,
      // },
    ];
  }
}
