import {adminNotificationActionNameSpace} from "@admin/notification/stores/admin-notification.action";
import {AdminNotificationStateModel} from "@admin/notification/stores/admin-notification.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {NotificationDlcm} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {NotificationStatusEnumHelper} from "@shared/enums/notification-status.enum";
import {NotificationTypeEnumHelper} from "@shared/enums/notification-type.enum";
import {RouterExtService} from "@shared/services/router-ext.service";
import {
  OrderEnum,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-notification-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminNotificationListRoutable extends SharedAbstractListRoutable<NotificationDlcm, AdminNotificationStateModel> {
  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("admin.notifications.button.new");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("admin.notifications.button.refresh");
  readonly KEY_BACK_BUTTON: string | undefined = MARK_AS_TRANSLATABLE("admin.button.goBackToAdminHome");
  readonly KEY_PARAM_NAME: keyof NotificationDlcm & string = "notificationType";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.admin_notification, adminNotificationActionNameSpace, {canCreate: false}, LocalStateEnum.admin);
  }

  conditionDisplayEditButton(model: NotificationDlcm | undefined): boolean {
    return false;
  }

  conditionDisplayDeleteButton(model: NotificationDlcm | undefined): boolean {
    return false;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "notificationType",
        header: MARK_AS_TRANSLATABLE("admin.notifications.table.header.notificationType"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: NotificationTypeEnumHelper.getListKeyValue(),
      },
      {
        field: "notificationStatus",
        header: MARK_AS_TRANSLATABLE("admin.notifications.table.header.notificationStatus"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
        translate: true,
        filterEnum: NotificationStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "emitter.person.fullName" as any,
        header: MARK_AS_TRANSLATABLE("admin.notifications.table.header.emitter"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: false,
      },
      {
        field: "notifiedOrgUnit.name" as any,
        header: MARK_AS_TRANSLATABLE("admin.notifications.table.header.notifiedOrgUnit"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
    ];
  }
}
