import {adminOaiSetActionNameSpace} from "@admin/oai-set/stores/admin-oai-set.action";
import {AdminOaiSetStateModel} from "@admin/oai-set/stores/admin-oai-set.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {OaiSet} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {RouterExtService} from "@shared/services/router-ext.service";
import {
  OrderEnum,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-oai-set-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminOaiSetListRoutable extends SharedAbstractListRoutable<OaiSet, AdminOaiSetStateModel> {
  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("admin.oai-set.button.new");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("admin.oai-set.button.refresh");
  readonly KEY_BACK_BUTTON: string | undefined = MARK_AS_TRANSLATABLE("admin.button.goBackToAdminHome");
  readonly KEY_PARAM_NAME: keyof OaiSet & string = "name";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.admin_oaiSet, adminOaiSetActionNameSpace, {
      listExtraButtons: [
        {
          color: "primary",
          icon: IconNameEnum.create,
          labelToTranslate: MARK_AS_TRANSLATABLE("admin.oai-set.button.bulkCreate"),
          route: RoutesEnum.adminOaiSetBulkCreate,
          order: 40,
        },
      ],
    }, LocalStateEnum.admin);
  }

  conditionDisplayEditButton(model: OaiSet | undefined): boolean {
    return true;
  }

  conditionDisplayDeleteButton(model: OaiSet | undefined): boolean {
    return true;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "name",
        header: MARK_AS_TRANSLATABLE("admin.oai-set.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "description",
        header: MARK_AS_TRANSLATABLE("admin.oai-set.table.header.description"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "spec",
        header: MARK_AS_TRANSLATABLE("admin.oai-set.table.header.spec"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "enabled",
        header: MARK_AS_TRANSLATABLE("admin.oai-set.table.header.enabled"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        filterEnum: [
          {
            key: "true",
            value: MARK_AS_TRANSLATABLE("admin.oai-set.table.enabled.true"),
          },
          {
            key: "false",
            value: MARK_AS_TRANSLATABLE("admin.oai-set.table.enabled.false"),
          },
        ],
        translate: true,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }
}
