import {PersonRole} from "@admin/models/person-role.model";
import {adminOrgUnitActionNameSpace} from "@admin/orgunit/stores/admin-organizational-unit.action";
import {
  AdminOrganizationalUnitState,
  AdminOrganizationalUnitStateModel,
} from "@admin/orgunit/stores/admin-organizational-unit.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {SystemProperty} from "@app/generated-api/model/system-property.partial.model";
import {AppPersonState} from "@app/stores/person/app-person.state";
import {AppSystemPropertyState} from "@app/stores/system-property/app-system-property.state";
import {
  DisseminationPolicy,
  FundingAgency,
  Institution,
  OrganizationalUnit,
  PreservationPolicy,
  Role,
  SubmissionPolicy,
} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {SharedAbstractCreateRoutable} from "@shared/components/routables/shared-abstract-create/shared-abstract-create.routable";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoleEnum} from "@shared/enums/role.enum";
import {LocalStateModel} from "@shared/models/local-state.model";
import {SharedResearchDomainAction} from "@shared/stores/research-domain/shared-research-domain.action";
import {SharedResearchDomainState} from "@shared/stores/research-domain/shared-research-domain.state";
import {SharedRoleState} from "@shared/stores/role/shared-role.state";
import {SharedSubmissionPolicyState} from "@shared/stores/submission-policy/shared-submission-policy.state";
import {Observable} from "rxjs";
import {
  MemoizedUtil,
  ResourceState,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-orgunit-create-routable",
  templateUrl: "./admin-orgunit-create.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-create/shared-abstract-create.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminOrgunitCreateRoutable extends SharedAbstractCreateRoutable<OrganizationalUnit, AdminOrganizationalUnitStateModel> implements OnInit {
  listSubmissionPoliciesObs: Observable<SubmissionPolicy[]> = ResourceState.list(this._store, SharedSubmissionPolicyState);
  @Select(AdminOrganizationalUnitState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(AdminOrganizationalUnitState.isReadyToBeDisplayedInCreateMode) isReadyToBeDisplayedInCreateModeObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.shared.shared_preservationPolicy.list) listPreservationPoliciesObs: Observable<PreservationPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_disseminationPolicy.list) listDisseminationPoliciesObs: Observable<DisseminationPolicy[]>;
  listRoleObs: Observable<Role[]> = ResourceState.list(this._store, SharedRoleState);
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_fundingAgency.selected) selectedFundingAgenciesObs: Observable<FundingAgency[]>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_institution.selected) selectedInstitutionsObs: Observable<Institution[]>;
  researchDomainSourcesObs: Observable<string[]> = MemoizedUtil.select(this._store, SharedResearchDomainState, state => state.sources);
  systemPropertyObs: Observable<SystemProperty> = MemoizedUtil.select(this._store, AppSystemPropertyState, state => state.current);

  selectedPersonRole: PersonRole[] = [
    {
      isReadonly: true,
      resId: MemoizedUtil.selectSnapshot(this._store, AppPersonState, state => state.current.resId),
      roles: [{resId: RoleEnum.MANAGER}],
    },
  ];

  constructor(protected readonly _store: Store,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef) {
    super(_store, _actions$, _changeDetector, LocalStateEnum.admin_organizationalUnit, adminOrgUnitActionNameSpace, LocalStateEnum.admin);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this._store.dispatch(new SharedResearchDomainAction.GetSource());
  }
}
