import {PersonRole} from "@admin/models/person-role.model";
import {adminOrgUnitActionNameSpace} from "@admin/orgunit/stores/admin-organizational-unit.action";
import {
  AdminOrganizationalUnitState,
  AdminOrganizationalUnitStateModel,
} from "@admin/orgunit/stores/admin-organizational-unit.state";
import {AdminOrgUnitDisseminationPolicyAction} from "@admin/orgunit/stores/dissemination-policy/admin-organizational-unit-dissemination-policy.action";
import {AdminOrgUnitFundingAgencyAction} from "@admin/orgunit/stores/funding-agency/admin-organizational-unit-funding-agency.action";
import {AdminOrgUnitInstitutionAction} from "@admin/orgunit/stores/institution/admin-organizational-unit-institution.action";
import {AdminOrgUnitPersonRoleAction} from "@admin/orgunit/stores/person-role/admin-organizational-unit-person-role.action";
import {AdminOrgUnitPreservationPolicyAction} from "@admin/orgunit/stores/preservation-policy/admin-organizational-unit-preservation-policy.action";
import {AdminOrgUnitSubmissionPolicyAction} from "@admin/orgunit/stores/submission-policy/admin-organizational-unit-submission-policy.action";
import {AdminOrganizationalUnitSubmissionPolicyState} from "@admin/orgunit/stores/submission-policy/admin-organizational-unit-submission-policy.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {SystemProperty} from "@app/generated-api/model/system-property.partial.model";
import {AppSystemPropertyState} from "@app/stores/system-property/app-system-property.state";
import {
  DisseminationPolicy,
  FundingAgency,
  Institution,
  OrganizationalUnit,
  PreservationPolicy,
  Role,
  SubmissionPolicy,
} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {SharedAbstractDetailEditCommonRoutable} from "@shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {LocalStateModel} from "@shared/models/local-state.model";
import {SharedResearchDomainAction} from "@shared/stores/research-domain/shared-research-domain.action";
import {SharedResearchDomainState} from "@shared/stores/research-domain/shared-research-domain.state";
import {Observable} from "rxjs";
import {MemoizedUtil} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-orgunit-detail-edit-routable",
  templateUrl: "./admin-orgunit-detail-edit.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminOrgunitDetailEditRoutable extends SharedAbstractDetailEditCommonRoutable<OrganizationalUnit, AdminOrganizationalUnitStateModel> implements OnInit {
  @Select(AdminOrganizationalUnitState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(AdminOrganizationalUnitState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;
  selectedSubmissionPoliciesObs: Observable<SubmissionPolicy[]> = MemoizedUtil.selected(this._store, AdminOrganizationalUnitSubmissionPolicyState);
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_preservationPolicy.selected) selectedPreservationPoliciesObs: Observable<PreservationPolicy[]>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_disseminationPolicy.selected) selectedDisseminationPoliciesObs: Observable<DisseminationPolicy[]>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_fundingAgency.selected) selectedFundingAgenciesObs: Observable<FundingAgency[]>;
  @Select((state: LocalStateModel) => state.shared.shared_submissionPolicy.list) listSubmissionPoliciesObs: Observable<SubmissionPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_preservationPolicy.list) listPreservationPoliciesObs: Observable<PreservationPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_disseminationPolicy.list) listDisseminationPoliciesObs: Observable<DisseminationPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_role.list) listRoleObs: Observable<Role[]>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_personRole.selected) selectedPersonRoleObs: Observable<PersonRole[]>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_institution.selected) selectedInstitutionsObs: Observable<Institution[]>;
  researchDomainSourcesObs: Observable<string[]> = MemoizedUtil.select(this._store, SharedResearchDomainState, state => state.sources);
  systemPropertyObs: Observable<SystemProperty> = MemoizedUtil.select(this._store, AppSystemPropertyState, state => state.current);

  readonly KEY_PARAM_NAME: keyof OrganizationalUnit & string = "name";

  constructor(protected _store: Store,
              protected _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              public _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.admin_organizationalUnit, adminOrgUnitActionNameSpace, LocalStateEnum.admin);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this._store.dispatch(new SharedResearchDomainAction.GetSource());
  }

  getSubResourceWithParentId(id: string): void {
    this._store.dispatch(new AdminOrgUnitSubmissionPolicyAction.GetAll(id));
    this._store.dispatch(new AdminOrgUnitPreservationPolicyAction.GetAll(id));
    this._store.dispatch(new AdminOrgUnitDisseminationPolicyAction.GetAll(id));
    this._store.dispatch(new AdminOrgUnitPersonRoleAction.GetAll(id));
    this._store.dispatch(new AdminOrgUnitFundingAgencyAction.GetAll(id));
    this._store.dispatch(new AdminOrgUnitInstitutionAction.GetAll(id));
  }
}
