import {
  AdminOrgUnitAction,
  adminOrgUnitActionNameSpace,
} from "@admin/orgunit/stores/admin-organizational-unit.action";
import {AdminOrgUnitDisseminationPolicyAction} from "@admin/orgunit/stores/dissemination-policy/admin-organizational-unit-dissemination-policy.action";
import {
  AdminOrganizationalUnitDisseminationPolicyState,
  AdminOrganizationalUnitDisseminationPolicyStateModel,
} from "@admin/orgunit/stores/dissemination-policy/admin-organizational-unit-dissemination-policy.state";
import {AdminOrgUnitFundingAgencyAction} from "@admin/orgunit/stores/funding-agency/admin-organizational-unit-funding-agency.action";
import {
  AdminOrganizationalUnitFundingAgencyState,
  AdminOrganizationalUnitFundingAgencyStateModel,
} from "@admin/orgunit/stores/funding-agency/admin-organizational-unit-funding-agency.state";
import {AdminOrgUnitInstitutionAction} from "@admin/orgunit/stores/institution/admin-organizational-unit-institution.action";
import {
  AdminOrganizationalUnitInstitutionState,
  AdminOrganizationalUnitInstitutionStateModel,
} from "@admin/orgunit/stores/institution/admin-organizational-unit-institution.state";
import {AdminOrgUnitPersonRoleAction} from "@admin/orgunit/stores/person-role/admin-organizational-unit-person-role.action";
import {
  AdminOrganizationalUnitPersonRoleState,
  AdminOrganizationalUnitPersonRoleStateModel,
} from "@admin/orgunit/stores/person-role/admin-organizational-unit-person-role.state";
import {AdminOrgUnitPreservationPolicyAction} from "@admin/orgunit/stores/preservation-policy/admin-organizational-unit-preservation-policy.action";
import {
  AdminOrganizationalUnitPreservationPolicyState,
  AdminOrganizationalUnitPreservationPolicyStateModel,
} from "@admin/orgunit/stores/preservation-policy/admin-organizational-unit-preservation-policy.state";
import {AdminOrgUnitSubmissionPolicyAction} from "@admin/orgunit/stores/submission-policy/admin-organizational-unit-submission-policy.action";
import {
  AdminOrganizationalUnitSubmissionPolicyState,
  AdminOrganizationalUnitSubmissionPolicyStateModel,
} from "@admin/orgunit/stores/submission-policy/admin-organizational-unit-submission-policy.state";
import {Injectable} from "@angular/core";
import {DepositStateModel} from "@deposit/stores/deposit.state";
import {OrganizationalUnit} from "@models";
import {
  Action,
  Actions,
  ofActionCompleted,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {AdminResourceApiEnum} from "@shared/enums/api.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {SharedDisseminationPolicyAction} from "@shared/stores/dissemination-policy/shared-dissemination-policy.action";
import {SharedPreservationPolicyAction} from "@shared/stores/preservation-policy/shared-preservation-policy.action";
import {SharedRoleAction} from "@shared/stores/role/shared-role.action";
import {SharedSubmissionPolicyAction} from "@shared/stores/submission-policy/shared-submission-policy.action";
import {Observable} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {
  ApiService,
  defaultAssociationStateInitValue,
  defaultRelation2TiersStateInitValue,
  defaultRelation3TiersStateInitValue,
  defaultResourceStateInitValue,
  isArray,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  ModelAttributeEnum,
  NotificationService,
  ObjectUtil,
  OverrideDefaultAction,
  Relation3TiersForm,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  urlSeparator,
} from "solidify-frontend";

export interface AdminOrganizationalUnitStateModel extends ResourceStateModel<OrganizationalUnit> {
  admin_organizationalUnit_submissionPolicy: AdminOrganizationalUnitSubmissionPolicyStateModel;
  admin_organizationalUnit_preservationPolicy: AdminOrganizationalUnitPreservationPolicyStateModel;
  admin_organizationalUnit_disseminationPolicy: AdminOrganizationalUnitDisseminationPolicyStateModel;
  admin_organizationalUnit_personRole: AdminOrganizationalUnitPersonRoleStateModel;
  admin_organizationalUnit_fundingAgency: AdminOrganizationalUnitFundingAgencyStateModel;
  admin_organizationalUnit_institution: AdminOrganizationalUnitInstitutionStateModel;
}

@Injectable()
@State<AdminOrganizationalUnitStateModel>({
  name: LocalStateEnum.admin_organizationalUnit,
  defaults: {
    ...defaultResourceStateInitValue(),
    admin_organizationalUnit_submissionPolicy: {...defaultRelation2TiersStateInitValue()},
    admin_organizationalUnit_preservationPolicy: {...defaultRelation2TiersStateInitValue()},
    admin_organizationalUnit_disseminationPolicy: {...defaultRelation2TiersStateInitValue()},
    admin_organizationalUnit_personRole: {...defaultRelation3TiersStateInitValue()},
    admin_organizationalUnit_fundingAgency: {...defaultAssociationStateInitValue()},
    admin_organizationalUnit_institution: {...defaultAssociationStateInitValue()},
  },
  children: [
    AdminOrganizationalUnitSubmissionPolicyState,
    AdminOrganizationalUnitPreservationPolicyState,
    AdminOrganizationalUnitDisseminationPolicyState,
    AdminOrganizationalUnitPersonRoleState,
    AdminOrganizationalUnitFundingAgencyState,
    AdminOrganizationalUnitInstitutionState,
  ],
})
export class AdminOrganizationalUnitState extends ResourceState<AdminOrganizationalUnitStateModel, OrganizationalUnit> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: adminOrgUnitActionNameSpace,
      routeRedirectUrlAfterSuccessCreateAction: (resId: string) => RoutesEnum.adminOrganizationalUnitDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessUpdateAction: (resId: string) => RoutesEnum.adminOrganizationalUnitDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessDeleteAction: RoutesEnum.adminOrganizationalUnit,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.organizationalUnit.notification.resource.create"),
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.organizationalUnit.notification.resource.delete"),
      notificationResourceUpdateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.organizationalUnit.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.organizationalUnits;
  }

  @Selector()
  static isLoading(state: AdminOrganizationalUnitStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminOrganizationalUnitStateModel): boolean {
    return this.isLoading(state)
      || StoreUtil.isLoadingState(state.admin_organizationalUnit_submissionPolicy)
      || StoreUtil.isLoadingState(state.admin_organizationalUnit_preservationPolicy)
      || StoreUtil.isLoadingState(state.admin_organizationalUnit_disseminationPolicy)
      || StoreUtil.isLoadingState(state.admin_organizationalUnit_personRole)
      || StoreUtil.isLoadingState(state.admin_organizationalUnit_fundingAgency)
      || StoreUtil.isLoadingState(state.admin_organizationalUnit_institution);
  }

  @Selector()
  static currentTitle(state: AdminOrganizationalUnitStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.name;
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminOrganizationalUnitStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current)
      && !isNullOrUndefined(state.admin_organizationalUnit_personRole.selected)
      && !isNullOrUndefined(state.admin_organizationalUnit_preservationPolicy.selected)
      && !isNullOrUndefined(state.admin_organizationalUnit_disseminationPolicy.selected)
      && !isNullOrUndefined(state.admin_organizationalUnit_submissionPolicy.selected)
      && !isNullOrUndefined(state.admin_organizationalUnit_fundingAgency.selected)
      && !isNullOrUndefined(state.admin_organizationalUnit_institution.selected);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminOrganizationalUnitStateModel): boolean {
    return true;
  }

  @Action(AdminOrgUnitAction.LoadResource)
  loadResource(ctx: StateContext<DepositStateModel>, action: AdminOrgUnitAction.LoadResource): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new SharedPreservationPolicyAction.GetAll(),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(SharedPreservationPolicyAction.GetAllSuccess)),
          this.actions$.pipe(ofActionCompleted(SharedPreservationPolicyAction.GetAllFail)),
        ],
      },
      {
        action: new SharedSubmissionPolicyAction.GetAll(),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(SharedSubmissionPolicyAction.GetAllSuccess)),
          this.actions$.pipe(ofActionCompleted(SharedSubmissionPolicyAction.GetAllFail)),
        ],
      },
      {
        action: new SharedDisseminationPolicyAction.GetAll(),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(SharedDisseminationPolicyAction.GetAllSuccess)),
          this.actions$.pipe(ofActionCompleted(SharedDisseminationPolicyAction.GetAllFail)),
        ],
      },
      {
        action: new SharedRoleAction.GetAll(),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(SharedRoleAction.GetAllSuccess)),
          this.actions$.pipe(ofActionCompleted(SharedRoleAction.GetAllFail)),
        ],
      },
    ]).subscribe(success => {
      if (success) {
        ctx.dispatch(new AdminOrgUnitAction.LoadResourceSuccess(action));
      } else {
        ctx.dispatch(new AdminOrgUnitAction.LoadResourceFail(action));
      }
    });
  }

  @OverrideDefaultAction()
  @Action(AdminOrgUnitAction.Create)
  create(ctx: StateContext<AdminOrganizationalUnitStateModel>, action: AdminOrgUnitAction.Create): Observable<OrganizationalUnit> {
    return super.internalCreate(ctx, action)
      .pipe(
        tap(model => {
          this.updateSubResource(model, action, ctx)
            .subscribe(success => {
              if (success) {
                ctx.dispatch(new AdminOrgUnitAction.CreateSuccess(action, model));
              } else {
                ctx.dispatch(new AdminOrgUnitAction.CreateFail(action));
              }
            });
        }),
      );
  }

  @OverrideDefaultAction()
  @Action(AdminOrgUnitAction.Update)
  update(ctx: StateContext<AdminOrganizationalUnitStateModel>, action: AdminOrgUnitAction.Update): Observable<OrganizationalUnit> {
    return super.internalUpdate(ctx, action)
      .pipe(
        tap(model => {
          this.updateSubResource(model, action, ctx)
            .subscribe(success => {
              if (success) {
                ctx.dispatch(new AdminOrgUnitAction.UpdateSuccess(action, model));
              } else {
                ctx.dispatch(new AdminOrgUnitAction.UpdateFail(action));
              }
            });
        }),
      );
  }

  private updateSubResource(model: OrganizationalUnit, action: AdminOrgUnitAction.Create | AdminOrgUnitAction.Update, ctx: StateContext<AdminOrganizationalUnitStateModel>): Observable<boolean> {
    const orgUnitId = model.resId;

    let oldDefaultPreservationPolicyId = undefined;
    const newDefaultPreservationPolicyId = action.modelFormControlEvent.model.defaultPreservationPolicy.resId;
    const newPreservationPoliciesIds = action.modelFormControlEvent.model.preservationPolicies.map(p => p.resId);

    let oldDefaultSubmissionPolicyId = undefined;
    const newDefaultSubmissionPolicyId = action.modelFormControlEvent.model.defaultSubmissionPolicy.resId;
    const newSubmissionPoliciesIds = action.modelFormControlEvent.model.submissionPolicies.map(p => p.resId);

    let oldDefaultDisseminationPolicyId = undefined;
    const newDefaultDisseminationPolicyId = action.modelFormControlEvent.model.defaultDisseminationPolicy.resId;
    const newDisseminationPoliciesIds = action.modelFormControlEvent.model.disseminationPolicies.map(p => p.resId);

    const newFundingAgencyIds = action.modelFormControlEvent.model.fundingAgencies.map(p => p.resId);

    const newInstitutionsIds = action.modelFormControlEvent.model.institutions.map(p => p.resId);

    if (!isNullOrUndefined(ctx.getState().current)) {
      oldDefaultPreservationPolicyId = ObjectUtil.getValue(ctx.getState().current.defaultPreservationPolicy, ModelAttributeEnum.resId) as string;
      oldDefaultSubmissionPolicyId = ObjectUtil.getValue(ctx.getState().current.defaultSubmissionPolicy, ModelAttributeEnum.resId) as string;
      oldDefaultDisseminationPolicyId = ObjectUtil.getValue(ctx.getState().current.defaultDisseminationPolicy, ModelAttributeEnum.resId) as string;
    }
    const newPersonRole = action.modelFormControlEvent.formControl.get("personRole").value as Relation3TiersForm[];
    newPersonRole.forEach((elem) => {
      if (!isNullOrUndefined(elem.listId) && !isArray(elem.listId)) {
        elem.listId = [elem.listId];
      }
    });

    return StoreUtil.dispatchSequentialActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new AdminOrgUnitPreservationPolicyAction.Update(orgUnitId, newPreservationPoliciesIds, oldDefaultPreservationPolicyId, newDefaultPreservationPolicyId),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitPreservationPolicyAction.UpdateSuccess)),
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitPreservationPolicyAction.UpdateFail)),
        ],
      },
      {
        action: new AdminOrgUnitSubmissionPolicyAction.Update(orgUnitId, newSubmissionPoliciesIds, oldDefaultSubmissionPolicyId, newDefaultSubmissionPolicyId),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitSubmissionPolicyAction.UpdateSuccess)),
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitSubmissionPolicyAction.UpdateFail)),
        ],
      },
      {
        action: new AdminOrgUnitDisseminationPolicyAction.Update(orgUnitId, newDisseminationPoliciesIds, oldDefaultDisseminationPolicyId, newDefaultDisseminationPolicyId),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitDisseminationPolicyAction.UpdateSuccess)),
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitDisseminationPolicyAction.UpdateFail)),
        ],
      },
      {
        action: new AdminOrgUnitPersonRoleAction.Update(orgUnitId, newPersonRole),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitPersonRoleAction.UpdateSuccess)),
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitPersonRoleAction.UpdateFail)),
        ],
      },
      {
        action: new AdminOrgUnitFundingAgencyAction.Update(orgUnitId, newFundingAgencyIds),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitFundingAgencyAction.UpdateSuccess)),
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitFundingAgencyAction.UpdateFail)),
        ],
      },
      {
        action: new AdminOrgUnitInstitutionAction.Update(orgUnitId, newInstitutionsIds),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitInstitutionAction.UpdateSuccess)),
          this.actions$.pipe(ofActionCompleted(AdminOrgUnitInstitutionAction.UpdateFail)),
        ],
      },
    ]);
  }
}
