import {OrganizationalUnitRole} from "@admin/models/organizational-unit-role.model";
import {PersonExtended} from "@admin/models/person-extended.model";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Input,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {sharedInstitutionActionNameSpace} from "@app/shared/stores/institution/shared-institution.action";
import {
  Institution,
  Role,
} from "@models";
import {SharedAbstractFormPresentational} from "@shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {PersonOrgUnitRoleMode} from "@shared/components/presentationals/shared-person-orgunit-role/shared-person-orgunit-role.presentational";
import {LocalModelAttributeEnum} from "@shared/enums/model-attribute.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {SharedInstitutionState} from "@shared/stores/institution/shared-institution.state";
import _ from "lodash";
import {
  isNullOrUndefined,
  PropertyName,
  ResourceNameSpace,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-person-form",
  templateUrl: "./admin-person-form.presentational.html",
  styleUrls: ["../../../../../../shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminPersonFormPresentational extends SharedAbstractFormPresentational<PersonExtended> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  @Input()
  selectedOrgUnitRole: OrganizationalUnitRole[];

  @Input()
  selectedInstitutions: Institution[];

  @Input()
  listRole: Role[];

  sharedInstitutionActionNameSpace: ResourceNameSpace = sharedInstitutionActionNameSpace;
  sharedInstitutionState: typeof SharedInstitutionState = SharedInstitutionState;

  get personOrgUnitRoleMode(): typeof PersonOrgUnitRoleMode {
    return PersonOrgUnitRoleMode;
  }

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              private readonly _fb: FormBuilder) {
    super(_changeDetectorRef, _elementRef);
  }

  protected bindFormTo(person: PersonExtended): void {
    this.form = this._fb.group({
      [this.formDefinition.firstName]: [person.firstName, [Validators.required, SolidifyValidator]],
      [this.formDefinition.lastName]: [person.lastName, [Validators.required, SolidifyValidator]],
      [this.formDefinition.orcid]: [person.orcid, [SolidifyValidator]],
      [this.formDefinition.orgUnitRole]: ["", [SolidifyValidator]],
      [this.formDefinition.institutions]: [_.map(this.selectedInstitutions, LocalModelAttributeEnum.resId)],
    });
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.firstName]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.lastName]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.orcid]: ["", [SolidifyValidator]],
      [this.formDefinition.institutions]: [],
      [this.formDefinition.orgUnitRole]: ["", [SolidifyValidator]],
    });
  }

  protected treatmentBeforeSubmit(person: PersonExtended): PersonExtended {
    person.institutions = [];
    const listInstitutions = this.form.get(this.formDefinition.institutions).value;
    if (!isNullOrUndefined(listInstitutions)) {
      listInstitutions.forEach(resId => {
        person.institutions.push({resId: resId});
      });
    }
    return person;
  }

  navigateToInstitution(institution: Institution): void {
    this.navigate([RoutesEnum.adminInstitutionDetail, institution.resId]);
  }

  navigateToContributor(): void {
    this.navigate([RoutesEnum.preservationSpaceContributorDetail, this.model.resId]);
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() firstName: string;
  @PropertyName() lastName: string;
  @PropertyName() orcid: string;
  @PropertyName() orgUnitRole: string;
  @PropertyName() institutions: string;
}
