import {adminPreservationPolicyActionNameSpace} from "@admin/preservation-policy/stores/admin-preservation-policy.action";
import {AdminPreservationPolicyStateModel} from "@admin/preservation-policy/stores/admin-preservation-policy.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {PreservationPolicy} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RouterExtService} from "@shared/services/router-ext.service";
import {
  OrderEnum,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-preservation-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminPreservationPolicyListRoutable extends SharedAbstractListRoutable<PreservationPolicy, AdminPreservationPolicyStateModel> {
  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("admin.preservationPolicy.button.new");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("admin.preservationPolicy.button.refresh");
  readonly KEY_BACK_BUTTON: string | undefined = MARK_AS_TRANSLATABLE("admin.button.goBackToAdminHome");
  readonly KEY_PARAM_NAME: keyof PreservationPolicy & string = "name";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.admin_preservationPolicy, adminPreservationPolicyActionNameSpace, {}, LocalStateEnum.admin);
  }

  conditionDisplayEditButton(model: PreservationPolicy | undefined): boolean {
    return true;
  }

  conditionDisplayDeleteButton(model: PreservationPolicy | undefined): boolean {
    return true;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "name",
        header: MARK_AS_TRANSLATABLE("admin.preservationPolicy.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "dispositionApproval",
        header: MARK_AS_TRANSLATABLE("admin.preservationPolicy.table.header.dispositionApproval"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("admin.preservationPolicy.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: MARK_AS_TRANSLATABLE("admin.preservationPolicy.table.header.lastUpdate.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "retention",
        header: MARK_AS_TRANSLATABLE("admin.preservationPolicy.table.header.retention"),
        type: FieldTypeEnum.number,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: false,
      },
    ];
  }

}
