import {
  ChangeDetectionStrategy,
  Component,
  Inject,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {depositActionNameSpace} from "@app/features/deposit/stores/deposit.action";
import {DepositStateModel} from "@app/features/deposit/stores/deposit.state";
import {SharedAbstractDeleteDialog} from "@app/shared/components/dialogs/shared-abstract-delete/shared-abstract-delete.dialog";
import {DeleteDialog} from "@app/shared/models/delete-dialog.model";
import {Deposit} from "@models";
import {Store} from "@ngxs/store";
import {
  Override,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";
import {LocalStateEnum} from "../../../../../shared/enums/local-state.enum";

@Component({
  selector: "dlcm-deposit-delete-folder-dialog",
  templateUrl: "../../../../../shared/components/dialogs/shared-abstract-delete/shared-abstract-delete.dialog.html",
  styleUrls: ["../../../../../shared/components/dialogs/shared-abstract-delete/shared-abstract-delete.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositDeleteFolderDialog extends SharedAbstractDeleteDialog<Deposit, DepositStateModel> {
  public readonly KEY_TITLE: string = MARK_AS_TRANSLATABLE("deposit.dialog.deleteFolder.title");
  public readonly KEY_MESSAGE: string = MARK_AS_TRANSLATABLE("deposit.dialog.deleteFolder.message");
  public readonly KEY_CONFIRM_BUTTON: string = MARK_AS_TRANSLATABLE("deposit.dialog.deleteFolder.confirm");
  public readonly KEY_CANCEL_BUTTON: string = MARK_AS_TRANSLATABLE("deposit.dialog.deleteFolder.cancel");

  constructor(protected store: Store,
              protected dialogRef: MatDialogRef<DepositDeleteFolderDialog>,
              @Inject(MAT_DIALOG_DATA) public data: DeleteDialog) {
    super(store, dialogRef, data, LocalStateEnum.deposit, depositActionNameSpace);
  }

  @Override()
  confirm(): void {
    this.dialogRef.close(true);
  }
}
