import {
  ChangeDetectionStrategy,
  Component,
  Input,
} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {Deposit} from "@models";
import {
  isFalse,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-banner",
  templateUrl: "./deposit-banner.presentational.html",
  styleUrls: ["./deposit-banner.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositBannerPresentational extends SharedAbstractPresentational {
  private _depositStatus: Deposit.StatusEnum;
  private _canSubmitAction: boolean;
  private _canDoAlterationAction: boolean;
  private _messageReasonUnableSubmit: string;

  @Input()
  set depositStatus(value: Deposit.StatusEnum) {
    this._depositStatus = value;
    this.computeDisplayInfo();
  }

  get depositStatus(): Deposit.StatusEnum {
    return this._depositStatus;
  }

  @Input()
  set canSubmitAction(value: boolean) {
    this._canSubmitAction = value;
    this.computeDisplayInfo();
  }

  get canSubmitAction(): boolean {
    return this._canSubmitAction;
  }

  @Input()
  set canDoAlterationAction(value: boolean) {
    this._canDoAlterationAction = value;
    this.computeDisplayInfo();
  }

  get canDoAlterationAction(): boolean {
    return this._canDoAlterationAction;
  }

  @Input()
  set messageReasonUnableSubmit(value: string) {
    this._messageReasonUnableSubmit = value;
    this.computeDisplayInfo();
  }

  get messageReasonUnableSubmit(): string {
    return this._messageReasonUnableSubmit;
  }

  display: boolean;
  message: string;
  color: "warn" | "info" | "success" | "error";

  constructor() {
    super();
  }

  computeDisplayInfo(): void {
    this.display = true;

    if (this.depositStatus === Deposit.StatusEnum.INPROGRESS && isFalse(this.canDoAlterationAction)) {
      this.display = false;
      return;
    }

    if (isFalse(this.canSubmitAction) && !isNullOrUndefined(this.messageReasonUnableSubmit)) {
      this.message = this.messageReasonUnableSubmit;
      this.color = "warn";
      return;
    }

    if (isNullOrUndefined(this.depositStatus)) {
      this.display = false;
      return;
    }

    if (this.depositStatus === Deposit.StatusEnum.INERROR) {
      this.message = MARK_AS_TRANSLATABLE("deposit.alert.status.error");
      this.color = "error";
      return;
    }

    if (this.depositStatus === Deposit.StatusEnum.REJECTED) {
      this.message = MARK_AS_TRANSLATABLE("deposit.alert.status.rejected");
      this.color = "error";
      return;
    }

    if (this.depositStatus === Deposit.StatusEnum.COMPLETED) {
      this.message = MARK_AS_TRANSLATABLE("deposit.alert.status.completed");
      this.color = "success";
      return;
    }

    if (this.depositStatus === Deposit.StatusEnum.APPROVED ||
      this.depositStatus === Deposit.StatusEnum.CHECKED ||
      this.depositStatus === Deposit.StatusEnum.PAUSED ||
      this.depositStatus === Deposit.StatusEnum.SUBMITTED) {
      this.message = MARK_AS_TRANSLATABLE("deposit.alert.status.processed");
      this.color = "success";
      return;
    }

    this.display = false;
  }
}
