import {PersonExtended} from "@admin/models/person-extended.model";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {depositActionNameSpace} from "@app/features/deposit/stores/deposit.action";
import {
  DepositState,
  DepositStateModel,
} from "@app/features/deposit/stores/deposit.state";
import {SharedAbstractCreateRoutable} from "@app/shared/components/routables/shared-abstract-create/shared-abstract-create.routable";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {DepositFormPresentational} from "@deposit/components/presentationals/deposit-form/deposit-form.presentational";
import {DepositOrganizationalUnitAdditionalFieldsFormAction} from "@deposit/stores/organizational-unit/additional-fields-form/deposit-organizational-unit-additional-fields-form.action";
import {DepositOrganizationalUnitAdditionalFieldsFormState} from "@deposit/stores/organizational-unit/additional-fields-form/deposit-organizational-unit-additional-fields-form.state";
import {DepositOrganizationalUnitState} from "@deposit/stores/organizational-unit/deposit-organizational-unit.state";
import {
  AdditionalFieldsForm,
  Deposit,
  Language,
  OrganizationalUnit,
  Person,
  PreservationPolicy,
  SubmissionPolicy,
} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {SharedOrgUnitPreservationPolicyAction} from "@shared/stores/organizational-unit/preservation-policy/shared-organizational-unit-preservation-policy.action";
import {SharedOrgUnitSubmissionPolicyAction} from "@shared/stores/organizational-unit/submission-policy/shared-organizational-unit-submission-policy.action";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
} from "rxjs/operators";
import {
  CompositionState,
  FormState,
  isNotNullNorUndefined,
  MARK_AS_TRANSLATABLE,
  MemoizedUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-create-routable",
  templateUrl: "./deposit-create.routable.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-create/shared-abstract-create.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositCreateRoutable extends SharedAbstractCreateRoutable<Deposit, DepositStateModel> implements OnInit, OnDestroy {
  @Select(DepositState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(DepositState.isReadyToBeDisplayedInCreateMode) isReadyToBeDisplayedInCreateModeObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.deposit.form) formObs: Observable<FormState>;
  @Select((state: LocalStateModel) => state.shared.shared_language.list) languagesObs: Observable<Language[]>;
  @Select((state: LocalStateModel) => state.shared.shared_person.list) listPersonObs: Observable<PersonExtended[]>;
  @Select((state: LocalStateModel) => state.shared.shared_organizationalUnit.shared_organizationalUnit_preservationPolicy.selected) listPreservationPoliciesObs: Observable<PreservationPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_organizationalUnit.shared_organizationalUnit_submissionPolicy.selected) listSubmissionPoliciesObs: Observable<SubmissionPolicy[]>;
  @Select((state: LocalStateModel) => state.application.application_person.current) personConnectedObs: Observable<Person>;
  @Select((state: LocalStateModel) => state.shared.shared_organizationalUnit.current) selectedOrgUnitObs: Observable<OrganizationalUnit>;
  additionalFieldsFormObs: Observable<AdditionalFieldsForm> = CompositionState.current(this._store, DepositOrganizationalUnitAdditionalFieldsFormState);

  @ViewChild("formPresentational")
  readonly formPresentational: DepositFormPresentational;
  orgUnitObs: Observable<OrganizationalUnit> = MemoizedUtil.select(this._store, DepositOrganizationalUnitState, (state) => state.current);

  listExtraButtons: ExtraButtonToolbar<Deposit>[] = [
    {
      color: "primary",
      typeButton: "flat-button",
      icon: IconNameEnum.save,
      labelToTranslate: MARK_AS_TRANSLATABLE("app.navigation.button.save"),
      order: 40,
      callback: () => this.formPresentational.onSubmit(),
      displayCondition: (resource) => true,
      disableCondition: () => this.formPresentational?.form?.pristine || this.formPresentational?.form?.invalid || this.formPresentational?.formFormly?.invalid,
    },
  ];

  constructor(protected readonly _store: Store,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef) {
    super(_store, _actions$, _changeDetector, LocalStateEnum.deposit, depositActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.subscribe(this.orgUnitObs.pipe(
      distinctUntilChanged(),
      filter(orgUnit => isNotNullNorUndefined(orgUnit)),
    ), orgUnit => {
      this._store.dispatch(new DepositOrganizationalUnitAdditionalFieldsFormAction.GetCurrentMetadataForm(orgUnit.resId));
      this._store.dispatch(new SharedOrgUnitPreservationPolicyAction.GetAll(orgUnit.resId));
      this._store.dispatch(new SharedOrgUnitSubmissionPolicyAction.GetAll(orgUnit.resId));
    });
  }

  getSubResourceWithParentId(id: string): void {
  }

  ngOnDestroy(): void {
    this._store.dispatch(new SharedOrgUnitPreservationPolicyAction.Clear());
    this._store.dispatch(new SharedOrgUnitSubmissionPolicyAction.Clear());
  }
}
