import {Injectable} from "@angular/core";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {
  defaultDepositOrganizationalUnitAdditionalFieldsFormStateModel,
  DepositOrganizationalUnitAdditionalFieldsFormState,
  DepositOrganizationalUnitAdditionalFieldsFormStateModel,
} from "@deposit/stores/organizational-unit/additional-fields-form/deposit-organizational-unit-additional-fields-form.state";
import {depositOrganizationalUnitNameSpace} from "@deposit/stores/organizational-unit/deposit-organizational-unit.action";
import {OrganizationalUnit} from "@models";
import {
  Actions,
  Selector,
  State,
  Store,
} from "@ngxs/store";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ResourceState,
  ResourceStateModel,
} from "solidify-frontend";

export const defaultDepositOrganizationalUnitValue: () => DepositOrganizationalUnitStateModel = () =>
  ({
    ...defaultResourceStateInitValue(),
    deposit_organizationalUnit_additionalFieldsForm: defaultDepositOrganizationalUnitAdditionalFieldsFormStateModel(),
  });

export interface DepositOrganizationalUnitStateModel extends ResourceStateModel<OrganizationalUnit> {
  deposit_organizationalUnit_additionalFieldsForm: DepositOrganizationalUnitAdditionalFieldsFormStateModel;
}

@Injectable()
@State<DepositOrganizationalUnitStateModel>({
  name: LocalStateEnum.deposit_organizationalUnit,
  defaults: {
    ...defaultDepositOrganizationalUnitValue(),
  },
  children: [DepositOrganizationalUnitAdditionalFieldsFormState],
})
export class DepositOrganizationalUnitState extends ResourceState<DepositOrganizationalUnitStateModel, OrganizationalUnit> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: depositOrganizationalUnitNameSpace,
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.authorizedOrganizationalUnits;
  }

  @Selector()
  static currentTitle(state: DepositOrganizationalUnitStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.name;
  }
}
