import {
  ChangeDetectionStrategy,
  Component,
  Input,
  Output,
  ViewChild,
} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {AccessOrganizationalUnit} from "@models";
import {SharedPaginatorPresentational} from "@shared/components/presentationals/shared-paginator/shared-paginator.presentational";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  isArray,
  isUndefined,
  ObjectUtil,
  ObservableUtil,
  Paging,
  QueryParameters,
} from "solidify-frontend";

@Component({
  selector: "dlcm-home-browsing-org-unit-presentational",
  templateUrl: "./home-browsing-org-unit.presentational.html",
  styleUrls: ["./home-browsing-org-unit.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeBrowsingOrgUnitPresentational extends SharedAbstractPresentational {
  @Input()
  listOrgUnit: AccessOrganizationalUnit[];

  @Input()
  orgUnitIdSelected: string;

  @Input()
  isLoading: boolean;

  @Input()
  queryParameters: QueryParameters;

  @Input()
  searchQuery: string;

  getSelectedOrgUnit(): AccessOrganizationalUnit | undefined {
    return this.listOrgUnit.find(value => value.resId === this.orgUnitIdSelected);
  }

  navigateUp(): void {
    if (!isArray(this.listOrgUnit)) {
      return;
    }
    const selectionIndex = this.listOrgUnit.findIndex(value => value.resId === this.orgUnitIdSelected);
    if (selectionIndex === -1 || selectionIndex === 0) {
      this.orgUnitIdSelected = this.listOrgUnit[0].resId;
    } else {
      this.orgUnitIdSelected = this.listOrgUnit[selectionIndex - 1].resId;
    }
  }

  navigateDown(): void {
    if (!isArray(this.listOrgUnit)) {
      return;
    }
    const selectionIndex = this.listOrgUnit.findIndex(value => value.resId === this.orgUnitIdSelected);
    if (selectionIndex === -1) {
      this.orgUnitIdSelected = this.listOrgUnit[0].resId;
    } else if (selectionIndex === this.listOrgUnit.length - 1) {
      this.orgUnitIdSelected = this.listOrgUnit[selectionIndex].resId;
    } else {
      this.orgUnitIdSelected = this.listOrgUnit[selectionIndex + 1].resId;
    }
  }

  private readonly _navigateBS: BehaviorSubject<AccessOrganizationalUnit | undefined> = new BehaviorSubject<AccessOrganizationalUnit | undefined>(undefined);
  @Output("navigateChange")
  readonly navigateObs: Observable<AccessOrganizationalUnit | undefined> = ObservableUtil.asObservable(this._navigateBS);

  private readonly _searchBS: BehaviorSubject<string | undefined> = new BehaviorSubject<string | undefined>(undefined);
  @Output("searchChange")
  readonly searchObs: Observable<string | undefined> = ObservableUtil.asObservable(this._searchBS);

  private readonly _queryParametersBS: BehaviorSubject<QueryParameters | undefined> = new BehaviorSubject<QueryParameters | undefined>(undefined);
  @Output("queryParametersChange")
  readonly queryParametersObs: Observable<QueryParameters | undefined> = ObservableUtil.asObservable(this._queryParametersBS);

  @ViewChild("paginator")
  paginator: SharedPaginatorPresentational;

  trackByFn(index: number, item: AccessOrganizationalUnit): string {
    return item.resId;
  }

  goToArchivesOfOrganizationalUnit(orgUnit: AccessOrganizationalUnit): void {
    this._navigateBS.next(orgUnit);
  }

  search(search: string): void {
    this._searchBS.next(search);
  }

  isSelected(orgUnit: AccessOrganizationalUnit): boolean {
    if (isUndefined(this.orgUnitIdSelected)) {
      return false;
    }
    return orgUnit.resId === this.orgUnitIdSelected;
  }

  pageChange($event: Paging): void {
    const queryParameters = ObjectUtil.clone(this.queryParameters);
    queryParameters.paging = $event;
    this._queryParametersBS.next(queryParameters);
  }
}
