import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  HostListener,
  OnInit,
} from "@angular/core";
import {environment} from "@environments/environment";
import {SharedAbstractPresentational} from "@shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {PollingHelper} from "@shared/helpers/polling.helper";

@Component({
  selector: "dlcm-home-carousel",
  templateUrl: "./home-carousel.presentational.html",
  styleUrls: ["./home-carousel.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeCarouselPresentational extends SharedAbstractPresentational implements OnInit {
  private readonly numberTiles: number = 3; // COMPUTE BY READ NUMBER OF LI
  activeTile: number = 0;
  skipNextChangeDueToHover: boolean = false;
  skipNextChangeDueToManualChangeOfTiles: boolean = false;

  tiles: undefined[];

  @HostListener("mouseenter")
  onMouseEnter(): void {
    this.skipNextChangeDueToHover = true;
  }

  @HostListener("mouseleave")
  onMouseLeave(): void {
    this.skipNextChangeDueToHover = false;
  }

  constructor(private readonly _changeDetector: ChangeDetectorRef) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.tiles = [];
    for (let i = 0; i < this.numberTiles; i++) {
      this.tiles.push(undefined);
    }

    this.subscribe(PollingHelper.startPollingObs({
      intervalRefreshInSecond: environment.frequencyChangeCarouselTileInSecond,
      filter: () => {
        if (this.skipNextChangeDueToManualChangeOfTiles) {
          this.skipNextChangeDueToManualChangeOfTiles = false;
          return false;
        }
        if (this.skipNextChangeDueToHover) {
          return false;
        }
        return true;
      },
      actionToDo: () => this.changeActiveTile(),
    }));
  }

  setActive(number: number): void {
    this.activeTile = number;
    this.skipNextChangeDueToManualChangeOfTiles = true;
  }

  private changeActiveTile(): void {
    this.activeTile++;
    if (this.activeTile >= this.numberTiles) {
      this.activeTile = 0;
    }
    this._changeDetector.detectChanges();
  }
}
