import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  HostBinding,
  OnInit,
  ViewChild,
} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {HomeBrowsingListArchivesRoutable} from "@app/features/home/components/routables/home-browsing-list-archives/home-browsing-list-archives.routable";
import {HomeOrgUnitAction} from "@app/features/home/stores/organizational-unit/home-organizational-unit.action";
import {SharedAbstractRoutable} from "@app/shared/components/routables/shared-abstract/shared-abstract.routable";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {AccessOrganizationalUnit} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Select,
  Store,
} from "@ngxs/store";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {Observable} from "rxjs";
import {tap} from "rxjs/operators";
import {
  isNullOrUndefined,
  QueryParameters,
  StoreUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-home-browsing-routable",
  templateUrl: "./home-browsing.routable.html",
  styleUrls: ["./home-browsing.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeBrowsingRoutable extends SharedAbstractRoutable implements OnInit {
  @Select((state: LocalStateModel) => state.home.home_organizationalUnit.search) searchObs: Observable<string>;
  @Select((state: LocalStateModel) => state.home.home_organizationalUnit.list) listOrgUnitObs: Observable<AccessOrganizationalUnit[]>;
  @Select((state: LocalStateModel) => state.home.home_organizationalUnit.queryParameters) queryParametersObs: Observable<QueryParameters>;
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state.home.home_organizationalUnit)) isLoadingOrgUnitObs: Observable<boolean>;

  @ViewChild(HomeBrowsingListArchivesRoutable)
  archivesRoutable: HomeBrowsingListArchivesRoutable;

  @HostBinding("class.is-org-unit-selected") isOrgUnitSelected: boolean;

  private _orgUnitId: string = undefined;

  set orgUnitId(value: string) {
    this._orgUnitId = value;
    if (isNullOrUndefined(value)) {
      this.isOrgUnitSelected = false;
    } else {
      this.isOrgUnitSelected = true;
    }
    this._changeDetector.detectChanges();
  }

  get orgUnitId(): string {
    return this._orgUnitId;
  }

  constructor(private _route: ActivatedRoute,
              private _store: Store,
              private _changeDetector: ChangeDetectorRef) {
    super();
  }

  ngOnInit(): void {
    this._store.dispatch(new HomeOrgUnitAction.Search(true));

    this.subscribe(this._route.url.pipe(
      tap(param => {
        if (this._route.snapshot.children.length > 0) {
          this.orgUnitId = this._route.snapshot.children[0].url[0].path;
        } else {
          this.orgUnitId = undefined;
        }
      }),
    ));
  }

  navigate($event: AccessOrganizationalUnit): void {
    this._store.dispatch(new Navigate([RoutesEnum.homePageBrowsing, $event.resId]));
  }

  search(search: string, queryParameters?: QueryParameters): void {
    this._store.dispatch(new HomeOrgUnitAction.Search(true, search, queryParameters));
  }

  pagination($event: QueryParameters): void {
    this._store.dispatch(new HomeOrgUnitAction.ChangeQueryParameters($event));
  }
}
