import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {HomeBrowsingListArchivesRoutable} from "@app/features/home/components/routables/home-browsing-list-archives/home-browsing-list-archives.routable";
import {HomeBrowsingRoutable} from "@app/features/home/components/routables/home-browsing/home-browsing.routable";
import {HomeDetailRoutable} from "@app/features/home/components/routables/home-detail/home-detail.routable";
import {HomeSearchRoutable} from "@app/features/home/components/routables/home-search/home-search.routable";
import {
  AppRoutesEnum,
  HomePageRoutesEnum,
} from "@app/shared/enums/routes.enum";
import {
  DlcmData,
  DlcmRoutes,
} from "@app/shared/models/dlcm-route.model";
import {HomeState} from "@home/stores/home.state";
import {HomeOrganizationalUnitState} from "@home/stores/organizational-unit/home-organizational-unit.state";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

const routes: DlcmRoutes = [
  {
    path: "",
    component: HomeSearchRoutable,
    data: {},
  },
  {
    path: HomePageRoutesEnum.detail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: HomeDetailRoutable,
    data: {
      breadcrumbMemoizedSelector: HomeState.currentTitle,
    },
  },
  {
    path: HomePageRoutesEnum.browsing,
    component: HomeBrowsingRoutable,
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.access.browsing"),
    },
    children: [
      {
        path: AppRoutesEnum.paramId,
        component: HomeBrowsingListArchivesRoutable,
        data: {
          breadcrumbMemoizedSelector: HomeOrganizationalUnitState.currentTitle,
        } as DlcmData,
        children: [
          {
            path: HomePageRoutesEnum.detail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
            component: HomeDetailRoutable,
            data: {
              breadcrumbMemoizedSelector: HomeState.currentTitle,
            },
          },
        ],
      },
    ],
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class HomeRoutingModule {
}
