import {Injectable} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {AppUserState} from "@app/stores/user/app-user.state";
import {ArchiveRestrictedRequestAccessDialog} from "@home/components/dialogs/archive-restricted-request-access/archive-restricted-request-access.dialog";
import {NotificationType} from "@models";
import {Store} from "@ngxs/store";
import {ArchiveMetadata} from "@shared/models/business/archive-metadata.model";
import {SharedNotificationAction} from "@shared/stores/notification/shared-notification.action";
import {MetadataUtil} from "@shared/utils/metadata.util";
import {
  take,
  tap,
} from "rxjs/operators";
import {
  isNullOrUndefined,
  ResourceState,
} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class HomeService {
  constructor(private readonly _dialog: MatDialog,
              private readonly _store: Store) {
  }

  requestToBeMember(archive: ArchiveMetadata): void {
    this._dialog.open(ArchiveRestrictedRequestAccessDialog, {
      minWidth: "500px",
    }).afterClosed().pipe(
      take(1),
      tap((message: string | undefined) => {
        if (isNullOrUndefined(message)) {
          return;
        }
        this.sendRequestToBeMember(archive, message);
      }),
    ).subscribe();
  }

  private sendRequestToBeMember(archive: ArchiveMetadata, message: string): void {
    this._store.dispatch(new SharedNotificationAction.Create({
      model: {
        emitter: ResourceState.currentSnapshot(this._store, AppUserState),
        notifiedOrgUnit: {
          resId: MetadataUtil.getOrganizationalUnitResId(archive.metadata),
        },
        requestedResId: archive.resId,
        message: message,
        notificationType: NotificationType.NotificationTypeEnum.ACCESS_DATASET_REQUEST,
      },
    }));
  }
}
