import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {Rss} from "@app/features/home/models/rss.model";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {
  BaseAction,
  BaseSubAction,
  CollectionTyped,
  QueryParameters,
} from "solidify-frontend";

const state = LocalStateEnum.home;

export namespace HomeAction {
  export class ChangeQueryParameters {
    static readonly type: string = `[${state}] Change Query Parameters`;

    constructor(public queryParameters?: QueryParameters) {
    }
  }

  export class ChangeSearchScope {
    static readonly type: string = `[${state}] Change Filter`;

    constructor(public searchScope: SearchScopeEnum) {
    }

  }

  export class Search {
    static readonly type: string = `[${state}] Search`;

    constructor(public resetPagination: boolean, public search?: string, public queryParameters?: QueryParameters) {
    }
  }

  export class SearchSuccess {
    static readonly type: string = `[${state}] Search Success`;

    constructor(public list?: CollectionTyped<ArchiveMetadata> | null | undefined) {
    }
  }

  export class SearchFail {
    static readonly type: string = `[${state}] Search Fail`;
  }

  export class SearchDetail {
    static readonly type: string = `[${state}] Search Detail`;

    constructor(public resId: string) {
    }
  }

  export class SearchDetailSuccess {
    static readonly type: string = `[${state}] Search Detail Success`;

    constructor(public model: ArchiveMetadata) {
    }
  }

  export class SearchDetailFail {
    static readonly type: string = `[${state}] Search Detail Fail`;

    constructor() {
    }
  }

  export class GetRss {
    static readonly type: string = `[${state}] Get Rss`;

    constructor() {
    }
  }

  export class GetRssSuccess {
    static readonly type: string = `[${state}] Get Rss Success`;

    constructor(public rss: Rss) {
    }
  }

  export class GetRssFail {
    static readonly type: string = `[${state}] Get Rss Fail`;

    constructor() {
    }
  }

  export class Download extends BaseAction {
    static readonly type: string = `[${state}] Download`;

    constructor(public archiveMetadata: ArchiveMetadata) {
      super();
    }
  }

  export class DownloadStart extends BaseSubAction<Download> {
    static readonly type: string = `[${state}] Download Start`;
  }

  export class DownloadSuccess extends BaseSubAction<DownloadStart> {
    static readonly type: string = `[${state}] Download Success`;
  }

  export class DownloadFail extends BaseSubAction<Download> {
    static readonly type: string = `[${state}] Download Fail`;
  }

  export class CleanCurrent {
    static readonly type: string = `[${state}] Clean Current`;
  }
}
