import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {
  AssociationNoSqlReadOnlyAction,
  AssociationNoSqlReadOnlyNameSpace,
  QueryParameters,
  TypeDefaultAction,
} from "solidify-frontend";

const state = LocalStateEnum.home_organizationalUnit_archive;

export namespace HomeOrgUnitArchiveAction {

  @TypeDefaultAction(state)
  export class GetAll extends AssociationNoSqlReadOnlyAction.GetAll {
  }

  @TypeDefaultAction(state)
  export class GetAllSuccess extends AssociationNoSqlReadOnlyAction.GetAllSuccess<ArchiveMetadata> {
  }

  @TypeDefaultAction(state)
  export class GetAllFail extends AssociationNoSqlReadOnlyAction.GetAllFail {
  }

  @TypeDefaultAction(state)
  export class GetById extends AssociationNoSqlReadOnlyAction.GetById {
  }

  @TypeDefaultAction(state)
  export class GetByIdSuccess extends AssociationNoSqlReadOnlyAction.GetByIdSuccess<ArchiveMetadata> {
  }

  @TypeDefaultAction(state)
  export class GetByIdFail extends AssociationNoSqlReadOnlyAction.GetByIdFail {
  }

  export class ChangeQueryParameters {
    static readonly type: string = `[${state}] Change Query Parameters`;

    constructor(public parentId: string, public queryParameters: QueryParameters, public keepCurrentContext: boolean = false) {
    }
  }

  export class Clean {
    static readonly type: string = `[${state}] Clean`;
  }
}

export const homeOrgUnitArchiveActionNameSpace: AssociationNoSqlReadOnlyNameSpace = HomeOrgUnitArchiveAction;
