import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {Order} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {OrderStatusEnumHelper} from "@order/enums/order-status.enum";
import {orderAllOrderActionNameSpace} from "@order/features/all-order/stores/order-all-order.action";
import {OrderAllOrderStateModel} from "@order/features/all-order/stores/order-all-order.state";
import {OrderQueryTypeEnumHelper} from "@order/helpers/order-query-type-enum.helper";
import {SharedAbstractListRoutable} from "@shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RouterExtService} from "@shared/services/router-ext.service";
import {
  OrderEnum,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-order-all-order-list-routable",
  templateUrl: "../../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrderAllOrderListRoutable extends SharedAbstractListRoutable<Order, OrderAllOrderStateModel> implements OnInit {
  readonly KEY_CREATE_BUTTON: string = MARK_AS_TRANSLATABLE("order.allOrder.create");
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("order.allOrder.refresh");
  readonly KEY_BACK_BUTTON: string = MARK_AS_TRANSLATABLE("order.allOrder.back");
  readonly KEY_PARAM_NAME: keyof Order & string = "name";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.order_allOrder, orderAllOrderActionNameSpace, {
      canCreate: true,
    }, LocalStateEnum.order);
  }

  conditionDisplayEditButton(model: Order | undefined): boolean {
    return model.status !== Order.StatusEnum.READY;
  }

  conditionDisplayDeleteButton(model: Order | undefined): boolean {
    return true;
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "queryType",
        header: MARK_AS_TRANSLATABLE("order.allOrder.table.header.queryType"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: OrderQueryTypeEnumHelper.getListKeyValue(),
      },
      {
        field: "name",
        header: MARK_AS_TRANSLATABLE("order.allOrder.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "status",
        header: MARK_AS_TRANSLATABLE("order.allOrder.table.header.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: OrderStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("order.allOrder.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: MARK_AS_TRANSLATABLE("order.allOrder.table.header.lastUpdate.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }
}
