import {
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {
  ActivatedRoute,
  Router,
} from "@angular/router";
import {AppCartDipAction} from "@app/stores/cart/dip/app-cart-dip.action";
import {environment} from "@environments/environment";
import {
  Aip,
  Dip,
  Order,
} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {OrderMyOrderFormPresentational} from "@order/features/my-order/components/presentationals/my-order-form/order-my-order-form.presentational";
import {OrderMyOrderDipAction} from "@order/features/my-order/stores/dip/order-my-order-dip.action";
import {OrderMyOrderDipState} from "@order/features/my-order/stores/dip/order-my-order-dip.state";
import {
  OrderMyOrderAction,
  orderMyOrderActionNameSpace,
} from "@order/features/my-order/stores/order-my-order.action";
import {
  OrderMyOrderState,
  OrderMyOrderStateModel,
} from "@order/features/my-order/stores/order-my-order.state";
import {OrderMyOrderStatusHistoryAction} from "@order/features/my-order/stores/status-history/order-my-order-status-history.action";
import {OrderMyOrderStatusHistoryState} from "@order/features/my-order/stores/status-history/order-my-order-status-history.state";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {SharedAbstractDetailEditCommonRoutable} from "@shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable";
import {AccessLevelEnumHelper} from "@shared/enums/business/access-level-enum.helper";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {SessionStorageEnum} from "@shared/enums/session-storage.enum";
import {PollingHelper} from "@shared/helpers/polling.helper";
import {SessionStorageHelper} from "@shared/helpers/session-storage.helper";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {SecurityService} from "@shared/services/security.service";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
  take,
} from "rxjs/operators";
import {
  AssociationState,
  isNullOrUndefined,
  MemoizedUtil,
  OverrideProperty,
  QueryParameters,
  ResourceActionHelper,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-order-my-order-detail-edit-routable",
  templateUrl: "./order-my-order-detail-edit.routable.html",
  styleUrls: ["./order-my-order-detail-edit.routable.scss"],
})
export class OrderMyOrderDetailEditRoutable extends SharedAbstractDetailEditCommonRoutable<Order, OrderMyOrderStateModel> implements OnInit, OnDestroy {
  @Select(OrderMyOrderState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(OrderMyOrderState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;

  historyObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, OrderMyOrderStatusHistoryState, state => state.history);
  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, OrderMyOrderStatusHistoryState);
  queryParametersObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, OrderMyOrderStatusHistoryState, state => state.queryParameters);

  listDipObs: Observable<Aip[]> = AssociationState.selected(this._store, OrderMyOrderDipState);
  isLoadingDipObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, OrderMyOrderDipState);

  @ViewChild("formPresentational")
  readonly formPresentational: OrderMyOrderFormPresentational;

  readonly KEY_PARAM_NAME: keyof Order & string = "name";

  @OverrideProperty()
  readonly editAvailable: boolean = false;

  listExtraButtons: ExtraButtonToolbar<Order>[] = [
    {
      color: "primary",
      icon: IconNameEnum.submit,
      displayCondition: current => !this.isEdit && !isNullOrUndefined(current) && current.status === Order.StatusEnum.INPROGRESS,
      callback: () => this.submit(),
      labelToTranslate: MARK_AS_TRANSLATABLE("order.myOrder.button.submit"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.resume,
      displayCondition: current => !this.isEdit && !isNullOrUndefined(current) && current.status === Order.StatusEnum.INERROR,
      callback: () => this.resume(),
      labelToTranslate: MARK_AS_TRANSLATABLE("order.myOrder.button.resume"),
      order: 40,
    },
  ];
  private readonly INTERVAL_REFRESH_IN_SECOND: number = 5;

  ngOnInit(): void {
    super.ngOnInit();

    this.subscribe(PollingHelper.startPollingObs({
      intervalRefreshInSecond: this.INTERVAL_REFRESH_IN_SECOND,
      continueUntil: () => isNullOrUndefined(this.current) || (this.current.status !== Order.StatusEnum.READY && this.current.status !== Order.StatusEnum.INERROR),
      actionToDo: () => this._store.dispatch(ResourceActionHelper.getById(this._resourceNameSpace, this._resId, true)),
    }));

    this.subscribe(this.currentObs.pipe(
      distinctUntilChanged(),
      filter(current => !isNullOrUndefined(current) && (this.current.status === Order.StatusEnum.READY || this.current.status === Order.StatusEnum.INERROR)),
      take(1),
      ),
      (current) => {
        this._store.dispatch(new OrderMyOrderDipAction.GetAll(this._resId));
        SessionStorageHelper.removeItemInList(SessionStorageEnum.newOrderAvailable, this._resId);
      });

    this.subscribe(PollingHelper.startPollingObs({
      // Manage case when we are on the page that become ready
      intervalRefreshInSecond: 1,
      filter: () => (this.current.status === Order.StatusEnum.READY || this.current.status === Order.StatusEnum.INERROR) && SessionStorageHelper.getListItem(SessionStorageEnum.newOrderAvailable)
        .includes(this._resId),
      actionToDo: () => {
        SessionStorageHelper.removeItemInList(SessionStorageEnum.newOrderAvailable, this._resId);
      },
    }));
  }

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog,
              protected readonly _router: Router,
              private readonly _securityService: SecurityService) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.order_myOrder, orderMyOrderActionNameSpace, LocalStateEnum.order);
  }

  get orderStatusEnum(): typeof Order.StatusEnum {
    return Order.StatusEnum;
  }

  get accessLevelEnumHelper(): typeof AccessLevelEnumHelper {
    return AccessLevelEnumHelper;
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
    this.cleanState();
  }

  getSubResourceWithParentId(id: string): void {
  }

  showHistory(): void {
    const dialogRef = this._dialog.open(SharedHistoryDialog, {
      width: environment.modalWidth,
      data: {
        parentId: null,
        resourceResId: this._resId,
        name: this._state,
        statusHistory: this.historyObs,
        isLoading: this.isLoadingHistoryObs,
        queryParametersObs: this.queryParametersObs,
        state: OrderMyOrderStatusHistoryAction,
      } as StatusHistoryDialog,
    });
  }

  private resume(): void {
    this._store.dispatch(new OrderMyOrderAction.Resume(this._resId));
  }

  private submit(): void {
    this._store.dispatch(new OrderMyOrderAction.Submit(this._resId));
  }

  downloadDip(dip: Dip): void {
    this._store.dispatch(new AppCartDipAction.Download(dip.resId));
  }
}
