import {
  ChangeDetectionStrategy,
  Component,
  Input,
} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ApplicationRoleEnum} from "@app/shared/enums/application-role.enum";
import {
  PreservationSpaceRoutesEnum,
  RoutesEnum,
} from "@app/shared/enums/routes.enum";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {Tab} from "@shared/components/containers/shared-tabs/shared-tabs.container";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {SessionStorageEnum} from "@shared/enums/session-storage.enum";
import {SessionStorageHelper} from "@shared/helpers/session-storage.helper";
import {Observable} from "rxjs";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

@Component({
  selector: "dlcm-preservation-space-home-routable",
  templateUrl: "./preservation-space-home.routable.html",
  styleUrls: ["./preservation-space-home.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PreservationSpaceHomeRoutable extends SharedAbstractPresentational {

  //isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this.store, PreservationSpaceState);
  isLoadingObs: Observable<boolean>;

  @Input()
  isLoading: boolean;

  userRolesObs: ApplicationRoleEnum[];

  private get rootUrl(): string[] {
    return [RoutesEnum.preservationSpace];
  }

  preservationSpaceTabs: Tab[] = [
    {
      id: "ORGANIZATIONAL_UNIT",
      icon: IconNameEnum.organizationalUnit,
      titleToTranslate: MARK_AS_TRANSLATABLE("admin.organizationalUnit.home.title"),
      suffixUrl: PreservationSpaceRoutesEnum.organizationalUnit,
      route: () => [...this.rootUrl, PreservationSpaceRoutesEnum.organizationalUnit],
    },
    {
      id: "CONTRIBUTOR",
      icon: IconNameEnum.contributor,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.contributor.home.title"),
      suffixUrl: PreservationSpaceRoutesEnum.contributor,
      route: () => [...this.rootUrl, PreservationSpaceRoutesEnum.contributor],
    },
    {
      id: "NOTIFICATION_RECEIVED",
      icon: IconNameEnum.requestInbox,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notificationInbox.home.title"),
      suffixUrl: PreservationSpaceRoutesEnum.notificationInbox,
      route: () => [...this.rootUrl, PreservationSpaceRoutesEnum.notificationInbox],
      numberNew: () => SessionStorageHelper.getListItem(SessionStorageEnum.notificationInboxPending).length,
    },
    {
      id: "NOTIFICATION_SENT",
      icon: IconNameEnum.requestSent,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notificationSent.home.title"),
      suffixUrl: PreservationSpaceRoutesEnum.notificationSent,
      route: () => [...this.rootUrl, PreservationSpaceRoutesEnum.notificationSent],
    },
  ];

  constructor(private store: Store) {
    super();
    this.userRolesObs = this.store.selectSnapshot((state: LocalStateModel) => state.application.userRoles);
  }

  navigate(path: RoutesEnum): void {
    this.store.dispatch(new Navigate([path]));
  }

  private currentTab: Tab;

  setCurrentTab($event: Tab): void {
    this.currentTab = $event;
  }

}
