import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {ContributorDepositAction} from "@app/features/preservation-space/contributor/stores/contributor-deposit/contributor-deposit.action";
import {ContributorDepositState} from "@app/features/preservation-space/contributor/stores/contributor-deposit/contributor-deposit.state";
import {contributorActionNameSpace} from "@app/features/preservation-space/contributor/stores/contributor.action";
import {
  ContributorState,
  ContributorStateModel,
} from "@app/features/preservation-space/contributor/stores/contributor.state";
import {Contributor} from "@app/generated-api/model/contributor.partial.model";
import {appAuthorizedOrganizationalUnitNameSpace} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.action";
import {AppAuthorizedOrganizationalUnitState} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.state";
import {Deposit} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {SharedAbstractDetailEditCommonRoutable} from "@shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {
  DepositRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {Observable} from "rxjs";
import {
  CompositionState,
  MemoizedUtil,
  OrderEnum,
  OverrideProperty,
  QueryParameters,
  ResourceNameSpace,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-detail-edit-routable",
  templateUrl: "./contributor-detail-edit.routable.html",
  styleUrls: ["./contributor-detail-edit.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class ContributorDetailEditRoutable extends SharedAbstractDetailEditCommonRoutable<Contributor, ContributorStateModel> implements OnInit {
  @Select(ContributorState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(ContributorState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;

  listDepositObs: Observable<Deposit[]> = CompositionState.list(this._store, ContributorDepositState);
  queryParametersDepositObs: Observable<QueryParameters> = CompositionState.queryParameters(this._store, ContributorDepositState);
  isLoadingDepositObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, ContributorDepositState);

  appAuthorizedOrganizationalUnitNameSpace: ResourceNameSpace = appAuthorizedOrganizationalUnitNameSpace;
  appAuthorizedOrganizationalUnitState: typeof AppAuthorizedOrganizationalUnitState = AppAuthorizedOrganizationalUnitState;

  @OverrideProperty()
  readonly editAvailable: boolean = false;

  @OverrideProperty()
  readonly deleteAvailable: boolean = false;

  readonly KEY_PARAM_NAME: keyof Contributor & string = undefined;
  columns: DataTableColumns[] = [
    {
      field: "title",
      header: MARK_AS_TRANSLATABLE("deposit.table.header.title"),
      type: FieldTypeEnum.string,
      order: OrderEnum.none,
      isFilterable: false,
      isSortable: false,
    },
    {
      field: "organizationalUnitId" as any,
      header: MARK_AS_TRANSLATABLE("deposit.table.header.organizationalUnit"),
      type: FieldTypeEnum.searchableSingleSelect,
      order: OrderEnum.none,
      component: DataTableComponentEnum.organizationalUnitName,
      isFilterable: false,
      isSortable: false,
      resourceNameSpace: this.appAuthorizedOrganizationalUnitNameSpace,
      resourceState: this.appAuthorizedOrganizationalUnitState as any,
    },
    {
      field: "publicationDate",
      header: MARK_AS_TRANSLATABLE("deposit.table.header.publicationDate"),
      type: FieldTypeEnum.date,
      order: OrderEnum.none,
      isFilterable: false,
      isSortable: false,
    },
    {
      field: "creation.when" as any,
      header: MARK_AS_TRANSLATABLE("deposit.table.header.creation.when"),
      type: FieldTypeEnum.datetime,
      order: OrderEnum.none,
      isFilterable: true,
      isSortable: true,
      width: "145px",
    },
    {
      field: "lastUpdate.when" as any,
      header: MARK_AS_TRANSLATABLE("deposit.table.header.lastUpdate.when"),
      type: FieldTypeEnum.datetime,
      order: OrderEnum.descending,
      isFilterable: true,
      isSortable: true,
    },
  ];

  constructor(protected _store: Store,
              protected _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              public _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservationSpace_contributor, contributorActionNameSpace, LocalStateEnum.preservationSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  getSubResourceWithParentId(id: string): void {
    this._store.dispatch(new ContributorDepositAction.GetAll(id));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this._store.dispatch(new ContributorDepositAction.ChangeQueryParameters(this._resId, queryParameters, true));
  }

  goToDeposit(deposit: Deposit): void {
    this._store.dispatch(new Navigate([RoutesEnum.deposit, deposit.organizationalUnitId, DepositRoutesEnum.detail, deposit.resId]));
  }

  navigateToPerson(): void {
    this.navigate([RoutesEnum.adminPersonDetail, this._resId]);
  }

}
