import {Injectable} from "@angular/core";
import {contributorDepositActionNameSpace} from "@app/features/preservation-space/contributor/stores/contributor-deposit/contributor-deposit.action";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Deposit} from "@models";
import {
  Actions,
  Selector,
  State,
  Store,
} from "@ngxs/store";
import {ApiResourceNameEnum} from "@shared/enums/api-resource-name.enum";
import {PreIngestResourceApiEnum} from "@shared/enums/api.enum";
import {
  ApiService,
  CompositionState,
  CompositionStateModel,
  defaultCompositionStateInitValue,
  isNullOrUndefined,
  NotificationService,
  StoreUtil,
} from "solidify-frontend";

export const defaultContributorDepositStateModel: () => ContributorDepositStateModel = () =>
  ({
    ...defaultCompositionStateInitValue(),
  });

export interface ContributorDepositStateModel extends CompositionStateModel<Deposit> {
}

@Injectable()
@State<ContributorDepositStateModel>({
  name: LocalStateEnum.preservationSpace_contributor_deposit,
  defaults: {
    ...defaultContributorDepositStateModel(),
  },
})
export class ContributorDepositState extends CompositionState<ContributorDepositStateModel, Deposit> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: contributorDepositActionNameSpace,
      resourceName: ApiResourceNameEnum.DEPOSIT,
    });
  }

  protected get _urlResource(): string {
    return PreIngestResourceApiEnum.contributors;
  }

  @Selector()
  static currentTitle(state: ContributorDepositStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.title;
  }

  @Selector()
  static isReadyToBeDisplayed(state: ContributorDepositStateModel): boolean {
    return !isNullOrUndefined(state.current);
  }

  @Selector()
  static isLoading(state: ContributorDepositStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }
}
