import {Injectable} from "@angular/core";
import {
  ContributorDepositState,
  ContributorDepositStateModel,
  defaultContributorDepositStateModel,
} from "@app/features/preservation-space/contributor/stores/contributor-deposit/contributor-deposit.state";
import {contributorActionNameSpace} from "@app/features/preservation-space/contributor/stores/contributor.action";
import {Contributor} from "@app/generated-api/model/contributor.partial.model";
import {PreIngestResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {
  Actions,
  Selector,
  State,
  Store,
} from "@ngxs/store";
import {SecurityService} from "@shared/services/security.service";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
} from "solidify-frontend";

export interface ContributorStateModel extends ResourceStateModel<Contributor> {
  preservationSpace_contributor_deposit: ContributorDepositStateModel;
}

@Injectable()
@State<ContributorStateModel>({
  name: LocalStateEnum.preservationSpace_contributor,
  defaults: {
    ...defaultResourceStateInitValue(),
    preservationSpace_contributor_deposit: defaultContributorDepositStateModel(),
  },
  children: [
    ContributorDepositState,
  ],
})
export class ContributorState extends ResourceState<ContributorStateModel, Contributor> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              private _securityService: SecurityService) {
    super(apiService, store, notificationService, actions$, {nameSpace: contributorActionNameSpace});
  }

  protected get _urlResource(): string {
    return PreIngestResourceApiEnum.contributors;
  }

  @Selector()
  static isLoadingWithDependency(state: ContributorStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isLoading(state: ContributorStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static currentTitle(state: ContributorStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.fullName;
  }

  @Selector()
  static isReadyToBeDisplayed(state: ContributorStateModel): boolean {
    return !isNullOrUndefined(state.current);
  }
}
