import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {
  NotificationDlcm,
  NotificationStatus,
  NotificationType,
} from "@models";
import {SharedAbstractFormPresentational} from "@shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {NotificationStatusEnumHelper} from "@shared/enums/notification-status.enum";
import {NotificationTypeEnumHelper} from "@shared/enums/notification-type.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {SecurityService} from "@shared/services/security.service";
import {sharedOrgUnitActionNameSpace} from "@shared/stores/organizational-unit/shared-organizational-unit.action";
import {SharedOrganizationalUnitState} from "@shared/stores/organizational-unit/shared-organizational-unit.state";
import {sharedPersonActionNameSpace} from "@shared/stores/person/shared-person.action";
import {SharedPersonState} from "@shared/stores/person/shared-person.state";
import {
  DateUtil,
  EnumUtil,
  PropertyName,
  ResourceNameSpace,
  SolidifyValidator,
} from "solidify-frontend";
import NotificationStatusEnum = NotificationStatus.NotificationStatusEnum;
import NotificationTypeEnum = NotificationType.NotificationTypeEnum;

@Component({
  selector: "dlcm-preservation-space-notification-form",
  templateUrl: "./preservation-space-notification-form.presentational.html",
  styleUrls: ["../../../../../../shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PreservationSpaceNotificationFormPresentational extends SharedAbstractFormPresentational<NotificationDlcm> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  listNotificationType: NotificationTypeEnum[] = EnumUtil.convertToArray(NotificationTypeEnum);
  listNotificationStatus: NotificationStatusEnum[] = EnumUtil.convertToArray(NotificationStatusEnum);

  get notificationTypeEnumHelper(): typeof NotificationTypeEnumHelper {
    return NotificationTypeEnumHelper;
  }

  get notificationStatusEnumHelper(): typeof NotificationStatusEnumHelper {
    return NotificationStatusEnumHelper;
  }

  sharedPersonActionNameSpace: ResourceNameSpace = sharedPersonActionNameSpace;
  sharedPersonState: typeof SharedPersonState = SharedPersonState;

  sharedOrgUnitActionNameSpace: ResourceNameSpace = sharedOrgUnitActionNameSpace;
  sharedOrganizationalUnitState: typeof SharedOrganizationalUnitState = SharedOrganizationalUnitState;

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              private readonly _fb: FormBuilder,
              public readonly securityService: SecurityService,
              public readonly breakpointService: BreakpointService) {
    super(_changeDetectorRef, _elementRef);
  }

  protected bindFormTo(notificationDlcm: NotificationDlcm): void {
    this.form = this._fb.group({
      [this.formDefinition.notificationType]: [notificationDlcm.notificationType, [Validators.required, SolidifyValidator]],
      [this.formDefinition.notificationStatus]: [notificationDlcm.notificationStatus, [Validators.required, SolidifyValidator]],
      [this.formDefinition.emitterId]: [notificationDlcm.emitter["person"]?.resId, [Validators.required, SolidifyValidator]],
      [this.formDefinition.message]: [notificationDlcm.message, [Validators.required, SolidifyValidator]],
      [this.formDefinition.creation]: [DateUtil.convertDateToDateTimeString(new Date(notificationDlcm.creation.when)), [Validators.required, SolidifyValidator]],
      [this.formDefinition.lastUpdate]: [DateUtil.convertDateToDateTimeString(new Date(notificationDlcm.lastUpdate.when)), [Validators.required, SolidifyValidator]],
      [this.formDefinition.notifiedOrgUnitId]: [notificationDlcm.notifiedOrgUnit?.resId, [Validators.required, SolidifyValidator]],
    });
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.notificationType]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.notificationStatus]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.emitterId]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.message]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.creation]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.lastUpdate]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.notifiedOrgUnitId]: ["", [Validators.required, SolidifyValidator]],
    });
  }

  protected treatmentBeforeSubmit(notificationDlcm: NotificationDlcm): NotificationDlcm {
    return notificationDlcm;
  }

  navigateToOrgUnit(resId: string): void {
    this.navigate([RoutesEnum.preservationSpaceOrganizationalUnitDetail, resId]);
  }

  navigateToPerson(resId: string): void {
    this.navigate([RoutesEnum.adminPersonDetail, resId]);
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() notificationType: string;
  @PropertyName() notificationStatus: string;
  @PropertyName() emitterId: string;
  @PropertyName() message: string;
  @PropertyName() creation: string;
  @PropertyName() lastUpdate: string;
  @PropertyName() notifiedOrgUnitId: string;
}
