import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {NotificationModeEnum} from "@app/features/preservation-space/notification/enums/notification-mode.enum";
import {NotificationHelper} from "@app/features/preservation-space/notification/helper/notification.helper";
import {
  PreservationSpaceNotificationState,
  PreservationSpaceNotificationStateModel,
} from "@app/features/preservation-space/notification/stores/preservation-space-notification.state";
import {PreservationSpaceNotificationStatusHistoryAction} from "@app/features/preservation-space/notification/stores/status-history/preservation-space-notification-status-history.action";
import {PreservationSpaceNotificationStatusHistoryState} from "@app/features/preservation-space/notification/stores/status-history/preservation-space-notification-status-history.state";
import {environment} from "@environments/environment";
import {
  NotificationDlcm,
  NotificationStatus,
  NotificationType,
} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {SharedAbstractDetailEditCommonRoutable} from "@shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {
  OrganizationalUnitRoutesEnum,
  PreservationSpaceRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {Observable} from "rxjs";
import {
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  MemoizedUtil,
  NotificationService,
  Override,
  OverrideProperty,
} from "solidify-frontend";
import {QueryParameters} from "solidify-frontend/lib/models";
import {
  PreservationSpaceNotificationAction,
  preservationSpaceNotificationActionNameSpace,
} from "../../../stores/preservation-space-notification.action";
import NotificationStatusEnum = NotificationStatus.NotificationStatusEnum;
import NotificationTypeEnum = NotificationType.NotificationTypeEnum;

@Component({
  selector: "dlcm-preservation-space-notification-detail-edit-routable",
  templateUrl: "./preservation-space-notification-detail-edit.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PreservationSpaceNotificationDetailEditRoutable extends SharedAbstractDetailEditCommonRoutable<NotificationDlcm, PreservationSpaceNotificationStateModel> implements OnInit {
  @Select(PreservationSpaceNotificationState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(PreservationSpaceNotificationState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;

  historyObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, PreservationSpaceNotificationStatusHistoryState, state => state.history);
  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, PreservationSpaceNotificationStatusHistoryState);
  queryParametersObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, PreservationSpaceNotificationStatusHistoryState, state => state.queryParameters);

  @OverrideProperty()
  readonly editAvailable: boolean = false;
  @OverrideProperty()
  readonly deleteAvailable: boolean = false;

  mode: NotificationModeEnum;
  modeObs: Observable<NotificationModeEnum> = MemoizedUtil.select(this._store, PreservationSpaceNotificationState, state => state.mode);
  readonly KEY_PARAM_NAME: keyof NotificationDlcm & string = "notificationType";

  listExtraButtons: ExtraButtonToolbar<NotificationDlcm>[] = [
    {
      color: "primary",
      icon: IconNameEnum.navigate,
      displayCondition: current => !isNullOrUndefined(current) && this.mode === NotificationModeEnum.inbox && current.notificationStatus !== NotificationStatusEnum.ACCEPTED,
      callback: (current) => this._process(current),
      labelToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.navigation.button.process"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.approve,
      displayCondition: current => !isNullOrUndefined(current) && this.mode === NotificationModeEnum.inbox && current.notificationStatus !== NotificationStatusEnum.ACCEPTED,
      callback: () => this._accept(),
      labelToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.navigation.button.accept"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.unapprove,
      displayCondition: current => !isNullOrUndefined(current) && this.mode === NotificationModeEnum.inbox && current.notificationStatus !== NotificationStatusEnum.REFUSED,
      callback: () => this._refuse(),
      labelToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.navigation.button.refuse"),
      order: 40,
    },
    {
      color: "primary",
      icon: IconNameEnum.wait,
      displayCondition: current => !isNullOrUndefined(current) && this.mode === NotificationModeEnum.inbox && current.notificationStatus !== NotificationStatusEnum.WAITING,
      callback: () => this._wait(),
      labelToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.navigation.button.wait"),
      order: 40,
    },
  ];

  constructor(protected _store: Store,
              protected _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              private readonly _notificationService: NotificationService,
              public _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservationSpace_notification, preservationSpaceNotificationActionNameSpace, LocalStateEnum.preservationSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this._retrieveModeFromUrl();
    this.subscribe(this.modeObs, mode => this.mode = mode);
  }

  getSubResourceWithParentId(id: string): void {
  }

  private _retrieveModeFromUrl(): void {
    this._store.dispatch(new PreservationSpaceNotificationAction.SetMode(NotificationHelper.getMode(this._route), undefined));
  }

  @Override()
  backToList(): void {
    const url = this._route.snapshot.parent.url.toString();
    this.mode = NotificationModeEnum.inbox;
    let route = RoutesEnum.preservationSpaceNotificationInbox;
    if (url === PreservationSpaceRoutesEnum.notificationSent) {
      route = RoutesEnum.preservationSpaceNotificationSent;
      this.mode = NotificationModeEnum.sent;
    }
    const orgUnitId = MemoizedUtil.selectSnapshot(this._store, PreservationSpaceNotificationState, state => state.orgUnitId);
    this._store.dispatch(new Navigate([route], NotificationHelper.getUrlQueryParam(orgUnitId)));
  }

  private _accept(): void {
    this._store.dispatch(new PreservationSpaceNotificationAction.Accept(this._resId));
  }

  private _refuse(): void {
    this._store.dispatch(new PreservationSpaceNotificationAction.Refuse(this._resId));
  }

  private _wait(): void {
    this._store.dispatch(new PreservationSpaceNotificationAction.Wait(this._resId));
  }

  private _process(notification: NotificationDlcm): void {
    switch (notification.notificationType) {
      case NotificationTypeEnum.JOIN_ORG_UNIT_REQUEST:
        this._processJoinOrgUnitRequest(notification);
        break;
      case NotificationTypeEnum.ACCESS_DATASET_REQUEST:
      default:
        this._notificationService.showInformation(MARK_AS_TRANSLATABLE("preservationSpace.notifications.notification.processActionNotAvailableForThisType"), {type: notification.notificationType});
        break;
    }
  }

  private _processJoinOrgUnitRequest(notification: NotificationDlcm): void {
    this._store.dispatch(new Navigate([RoutesEnum.preservationSpaceOrganizationalUnitDetail, notification.notifiedOrgUnit.resId, OrganizationalUnitRoutesEnum.edit], {
      [NotificationHelper.KEY_ROLE_ID]: "APPROVER", // TODO Change when backend store this info
      [NotificationHelper.KEY_PERSON_ID]: notification.emitter.person.resId,
    }));
  }

  showHistory(): void {
    const dialogRef = this._dialog.open(SharedHistoryDialog, {
      width: environment.modalWidth,
      data: {
        parentId: null,
        resourceResId: this._resId,
        name: this._state,
        statusHistory: this.historyObs,
        isLoading: this.isLoadingHistoryObs,
        queryParametersObs: this.queryParametersObs,
        state: PreservationSpaceNotificationStatusHistoryAction,
      } as StatusHistoryDialog,
    });
  }

}
