import {PersonRole} from "@admin/models/person-role.model";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {OrgunitRequestAccessDialog} from "@app/features/preservation-space/organizational-unit/components/dialogs/orgunit-request-access/orgunit-request-access.dialog";
import {orgUnitActionNameSpace} from "@app/features/preservation-space/organizational-unit/stores/organizational-unit.action";
import {
  OrganizationalUnitState,
  OrganizationalUnitStateModel,
} from "@app/features/preservation-space/organizational-unit/stores/organizational-unit.state";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {AppUserState} from "@app/stores/user/app-user.state";
import {
  NotificationType,
  OrganizationalUnit,
  PreservationPolicy,
  ResearchDomain,
  Role,
  SubmissionPolicy,
} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {SharedAbstractDetailEditCommonRoutable} from "@shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable";
import {SharedNotificationAction} from "@shared/stores/notification/shared-notification.action";
import {SharedResearchDomainAction} from "@shared/stores/research-domain/shared-research-domain.action";
import {SharedResearchDomainState} from "@shared/stores/research-domain/shared-research-domain.state";
import {Observable} from "rxjs";
import {tap} from "rxjs/operators";
import {
  isNullOrUndefined,
  MemoizedUtil,
  OverrideProperty,
  ResourceState,
} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-detail-edit-routable",
  templateUrl: "./orgunit-detail-edit.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitDetailEditRoutable extends SharedAbstractDetailEditCommonRoutable<OrganizationalUnit, OrganizationalUnitStateModel> implements OnInit {
  @Select(OrganizationalUnitState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(OrganizationalUnitState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;
  @Select(OrganizationalUnitState.currentUserIsManager) currentUserIsManager: Observable<boolean>;
  @Select((state: LocalStateModel) => state.shared.shared_submissionPolicy.list) listSubmissionPoliciesObs: Observable<SubmissionPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_preservationPolicy.list) listPreservationPoliciesObs: Observable<PreservationPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_role.list) listRoleObs: Observable<Role[]>;
  @Select((state: LocalStateModel) => state.shared.shared_researchDomain.list) listResearchDomainObs: Observable<ResearchDomain[]>;
  @Select((state: LocalStateModel) => state.organizationalUnit.organizationalUnit_personRole.selected) selectedPersonRoleObs: Observable<PersonRole[]>;
  researchDomainSourcesObs: Observable<string[]> = MemoizedUtil.select(this._store, SharedResearchDomainState, state => state.sources);

  readonly KEY_PARAM_NAME: keyof OrganizationalUnit & string = "name";

  @OverrideProperty()
  readonly deleteAvailable: boolean = false;

  constructor(protected _store: Store,
              protected _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              public _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.organizationalUnit, orgUnitActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this._store.dispatch(new SharedResearchDomainAction.GetSource());
  }

  getSubResourceWithParentId(id: string): void {
  }

  requestToBeMember(): void {
    this.subscribe(this._dialog.open(OrgunitRequestAccessDialog, {
      minWidth: "500px",
    }).afterClosed().pipe(
      tap((message: string | undefined) => {
        if (isNullOrUndefined(message)) {
          return;
        }
        this.sendRequestToBeMember(message);
      }),
    ));
  }

  private sendRequestToBeMember(message: string): void {
    this._store.dispatch(new SharedNotificationAction.Create({
      model: {
        emitter: ResourceState.currentSnapshot(this._store, AppUserState),
        notifiedOrgUnit: {
          resId: this._resId,
        },
        message: message,
        notificationType: NotificationType.NotificationTypeEnum.JOIN_ORG_UNIT_REQUEST,
      },
    }));
  }
}
