import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {OrgunitListRoutable} from "@app/features/preservation-space/organizational-unit/components/routables/orgunit-list/orgunit-list.routable";
import {
  AppRoutesEnum,
  OrganizationalUnitRoutesEnum,
} from "@shared/enums/routes.enum";
import {DlcmRoutes} from "@shared/models/dlcm-route.model";
import {OrgunitDetailEditRoutable} from "@app/features/preservation-space/organizational-unit/components/routables/orgunit-detail-edit/orgunit-detail-edit.routable";
import {OrganizationalUnitComputeIsManagerGuardService} from "@shared/guards/organizational-unit-compute-is-manager.service";
import {OrganizationalUnitRoleGuardService} from "@shared/guards/organizational-unit-role-guard.service";
import {CanDeactivateGuard} from "@shared/services/can-deactivate-guard.service";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";
import {OrganizationalUnitState} from "./stores/organizational-unit.state";

const routes: DlcmRoutes = [
  {
    path: "",
    component: OrgunitListRoutable,
    data: {},
  },
  {
    path: OrganizationalUnitRoutesEnum.detail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: OrgunitDetailEditRoutable,
    data: {
      breadcrumbMemoizedSelector: OrganizationalUnitState.currentTitle,
    },
    canActivate: [OrganizationalUnitComputeIsManagerGuardService],
    children: [
      {
        path: OrganizationalUnitRoutesEnum.edit,
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.organizationalUnit.edit"),
        },
        canActivate: [OrganizationalUnitRoleGuardService],
        canDeactivate: [CanDeactivateGuard],
      },
    ],
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class OrganizationalUnitRoutingModule {
}
