import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {PreservationSpaceHomeRoutable} from "@app/features/preservation-space/components/routables/preservation-space-home/preservation-space-home.routable";
import {
  AppRoutesEnum,
  PreservationSpaceRoutesEnum,
} from "@app/shared/enums/routes.enum";
import {DlcmRoutes} from "@app/shared/models/dlcm-route.model";
import {ApplicationRolePermissionEnum} from "@shared/enums/application-role-permission.enum";
import {ApplicationRoleGuardService} from "@shared/guards/application-role-guard.service";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

const routes: DlcmRoutes = [
  {
    path: AppRoutesEnum.root,
    redirectTo: PreservationSpaceRoutesEnum.organizationalUnit,
    pathMatch: "full",
  },
  {
    path: AppRoutesEnum.root,
    component: PreservationSpaceHomeRoutable,
    data: {},
    children: [
      {
        path: PreservationSpaceRoutesEnum.organizationalUnit,
        // @ts-ignore Dynamic import
        loadChildren: () => import("./organizational-unit/organizational-unit.module").then(m => m.OrganizationalUnitModule),
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.organizationalUnit.root"),
          permission: ApplicationRolePermissionEnum.userPermission,
        },
        canActivate: [ApplicationRoleGuardService],
      },
      {
        path: PreservationSpaceRoutesEnum.contributor,
        // @ts-ignore Dynamic import
        loadChildren: () => import("./contributor/contributor.module").then(m => m.ContributorModule),
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.contributor.root"),
        },
        canActivate: [ApplicationRoleGuardService],
      },
      {
        path: PreservationSpaceRoutesEnum.notificationInbox,
        // @ts-ignore Dynamic import
        loadChildren: () => import("./notification/notification.module").then(m => m.PreservationSpaceNotificationModule),
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.notificationInbox.root"),
        },
        canActivate: [ApplicationRoleGuardService],
      },
      {
        path: PreservationSpaceRoutesEnum.notificationSent,
        // @ts-ignore Dynamic import
        loadChildren: () => import("./notification/notification.module").then(m => m.PreservationSpaceNotificationModule),
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.notificationSent.root"),
        },
        canActivate: [ApplicationRoleGuardService],
      },
    ],
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class PreservationSpaceRoutingModule {
}
