import {NgModule} from "@angular/core";
import {AipDownloadedNotFoundRoutable} from "@preservation/aip-downloaded/components/routables/aip-downloaded-not-found/aip-downloaded-not-found.routable";
import {callbackAipTabBreadcrumb} from "@preservation/aip/aip-routing.module";
import {AipSharedModule} from "@preservation/aip/aip-shared.module";
import {AipAipRoutable} from "@preservation/aip/components/routables/aip-aip/aip-aip.routable";
import {AipDetailEditRoutable} from "@preservation/aip/components/routables/aip-detail-edit/aip-detail-edit.routable";
import {AipListRoutable} from "@preservation/aip/components/routables/aip-list/aip-list.routable";
import {AipMetadataRoutable} from "@preservation/aip/components/routables/aip-metadata/aip-metadata.routable";
import {AipTabsRoutable} from "@preservation/aip/components/routables/aip-tabs/aip-tabs.routable";
import {PreservationAipState} from "@preservation/aip/stores/aip.state";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@shared/enums/routes.enum";
import {DlcmRoutes} from "@shared/models/dlcm-route.model";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

const routes: DlcmRoutes = [
  {
    path: PreservationPlanningRoutesEnum.aipDownloadedNotFound + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: AipDownloadedNotFoundRoutable,
  },
  // {
  // path: PreservationPlanningRoutesEnum.preservationPlanning + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipDownloaded,
  // redirectTo: PreservationPlanningRoutesEnum.aipDownloaded + AppRoutesEnum.separator + "all",
  // pathMatch: "full",
  // },
  // {
  //   path: PreservationPlanningRoutesEnum.storagionNumber + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
  //   redirectTo: PreservationPlanningRoutesEnum.storagionNumber + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipMetadata,
  //   pathMatch: "full",
  {
    path: AppRoutesEnum.root,
    redirectTo: PreservationPlanningRoutesEnum.aipTabAll,
    pathMatch: "full",
  },
  {
    path: PreservationPlanningRoutesEnum.aipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    redirectTo: PreservationPlanningRoutesEnum.aipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipMetadata,
    pathMatch: "full",
  },
  // },
  {
    path: AppRoutesEnum.root,
    component: AipTabsRoutable,
    data: {},
    children: [
      {
        path: PreservationPlanningRoutesEnum.tab,
        component: AipListRoutable,
        data: {
          breadcrumb: callbackAipTabBreadcrumb,
          noBreadcrumbLink: true,
        },
      },
    ],
  },
  {
    path: PreservationPlanningRoutesEnum.aipDownloadedDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: AipDetailEditRoutable,
    data: {
      breadcrumbMemoizedSelector: PreservationAipState.currentAipName,
    },
    children: [
      {
        path: PreservationPlanningRoutesEnum.aipDownloadedMetadata,
        component: AipMetadataRoutable,
        data: {
          breadscrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.aipDownloaded.metadata"),
          noBreadcrumbLink: true,
        },
      },
      {
        path: PreservationPlanningRoutesEnum.aipDownloadedFiles,
        component: AipAipRoutable,
        data: {
          breadscrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.aipDownloaded.file"),
          noBreadcrumbLink: true,
        },
      },
    ],
  },
];

@NgModule({
  imports: [AipSharedModule.forChild(routes)],
  exports: [AipSharedModule],
})
export class AipDownloadedRoutingModule {
}
