import {
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {PreservationAipStatusAction} from "@app/features/preservation/aip-status/stores/preservation-aip-status.action";
import {PreservationAipStatusState} from "@app/features/preservation/aip-status/stores/preservation-aip-status.state";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {AipCopySearchTypeEnum} from "@shared/enums/business/aip-copy-search-type.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {AipCopyList} from "@shared/models/business/aip-copy-list.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {Observable} from "rxjs";
import {
  MemoizedUtil,
  OrderEnum,
  QueryParameters,
  ResourceState,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-aip-status-home-routable",
  templateUrl: "./aip-status-home.routable.html",
  styleUrls: ["./aip-status-home.routable.scss"],
})
export class AipStatusHomeRoutable extends SharedAbstractPresentational implements OnInit {
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, PreservationAipStatusState);
  listObs: Observable<AipCopyList[]> = ResourceState.list(this._store, PreservationAipStatusState);
  queryParametersObs: Observable<QueryParameters> = ResourceState.queryParameters(this._store, PreservationAipStatusState);
  columns: DataTableColumns<AipCopyList>[];

  constructor(private readonly _store: Store,
              private readonly _changeDetector: ChangeDetectorRef) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.defineColumns();
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "resId",
        header: MARK_AS_TRANSLATABLE("admin.preservation.aipStatuses.table.header.resId"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: false,
      },
      {
        field: "copies",
        header: MARK_AS_TRANSLATABLE("admin.preservation.aipStatuses.table.header.name"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: false,
        component: DataTableComponentEnum.aipStatusNamePresentational,
      },
      {
        field: "copies",
        header: MARK_AS_TRANSLATABLE("admin.preservation.aipStatuses.table.header.organizationalUnit"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        component: DataTableComponentEnum.aipStatusOrgUnitPresentational,
        isFilterable: false,
        isSortable: false,
      },
      {
        field: "status",
        header: MARK_AS_TRANSLATABLE("admin.preservation.aipStatuses.table.header.summary"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: false,
        filterableField: "searchType",
        filterEnum: [
          {
            key: AipCopySearchTypeEnum.ALL_COPIES_COMPLETED,
            value: MARK_AS_TRANSLATABLE("ALL_COPIES_COMPLETED"),
          },
          {
            key: AipCopySearchTypeEnum.SOME_COPY_IN_ERROR,
            value: MARK_AS_TRANSLATABLE("SOME_COPY_IN_ERROR"),
          },
          {
            key: AipCopySearchTypeEnum.SOME_COPY_MISSING,
            value: MARK_AS_TRANSLATABLE("SOME_COPY_MISSING"),
          },
          {
            key: AipCopySearchTypeEnum.SOME_COPY_MISC_STATUS,
            value: MARK_AS_TRANSLATABLE("SOME_COPY_MISC_STATUS"),
          },
        ],
        translate: true,
        component: DataTableComponentEnum.aipStatusSummaryPresentational,
      },
      {
        field: "creationDate",
        header: MARK_AS_TRANSLATABLE("admin.preservation.aipStatuses.table.header.creationDate"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        filterableField: "creation.when",
        sortableField: "creation.when",
      },
    ];
  }

  getAll(queryParameters?: QueryParameters): void {
    this._store.dispatch(new PreservationAipStatusAction.GetAll(queryParameters, true));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this._store.dispatch(new PreservationAipStatusAction.ChangeQueryParameters(queryParameters, true));
    this._changeDetector.detectChanges(); // Allow to display spinner the first time
  }

  back(): void {
    this._store.dispatch(new Navigate([RoutesEnum.preservationPlanning]));
  }

  deleteCache(): void {
    this._store.dispatch(new PreservationAipStatusAction.DeleteCache());
  }
}
