import {NgModule} from "@angular/core";
import {Params} from "@angular/router";
import {AipDetailEditRoutable} from "@app/features/preservation/aip/components/routables/aip-detail-edit/aip-detail-edit.routable";
import {AipHomeRoutable} from "@app/features/preservation/aip/components/routables/aip-home/aip-home.routable";
import {AipDetailGuardService} from "@preservation/aip/aip-detail-guard.service";
import {AipSharedModule} from "@preservation/aip/aip-shared.module";
import {AipAipRoutable} from "@preservation/aip/components/routables/aip-aip/aip-aip.routable";
import {AipListRoutable} from "@preservation/aip/components/routables/aip-list/aip-list.routable";
import {AipMetadataRoutable} from "@preservation/aip/components/routables/aip-metadata/aip-metadata.routable";
import {AipNotFoundRoutable} from "@preservation/aip/components/routables/aip-not-found/aip-not-found.routable";
import {AipStoragionRootRoutable} from "@preservation/aip/components/routables/aip-storagion-root/aip-storagion-root.routable";
import {AipTabsRoutable} from "@preservation/aip/components/routables/aip-tabs/aip-tabs.routable";
import {PreservationAipState} from "@preservation/aip/stores/aip.state";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@shared/enums/routes.enum";
import {DlcmRoutes} from "@shared/models/dlcm-route.model";
import {
  MappingObjectUtil,
  StringUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

export const callbackAipStoragionBreadcrumb = (params: Params) => MappingObjectUtil.get(params, PreservationPlanningRoutesEnum.storagionNumberWithoutPrefixParam);
export const callbackAipTabBreadcrumb = (params: Params) => {
  const tab = MappingObjectUtil.get(params, PreservationPlanningRoutesEnum.tabWithoutPrefixParam);
  let name = StringUtil.stringEmpty;
  if (tab === PreservationPlanningRoutesEnum.aipTabAll) {
    name = MARK_AS_TRANSLATABLE("app.aip.tabs.all");
  } else if (tab === PreservationPlanningRoutesEnum.aipTabUnit) {
    name = MARK_AS_TRANSLATABLE("app.aip.tabs.units");
  } else if (tab === PreservationPlanningRoutesEnum.aipTabCollections) {
    name = MARK_AS_TRANSLATABLE("app.aip.tabs.collections");
  }
  return name;
};

const routes: DlcmRoutes = [
  {
    path: AppRoutesEnum.root,
    component: AipHomeRoutable,
    data: {},
  },
  {
    path: PreservationPlanningRoutesEnum.storagionNumber,
    redirectTo: PreservationPlanningRoutesEnum.storagionNumber + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipList + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipTabAll,
    pathMatch: "full",
  },
  {
    path: PreservationPlanningRoutesEnum.storagionNumber + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    redirectTo: PreservationPlanningRoutesEnum.storagionNumber + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.aipMetadata,
    pathMatch: "full",
  },
  {
    path: PreservationPlanningRoutesEnum.storagionNumber,
    component: AipStoragionRootRoutable,
    data: {
      breadcrumb: callbackAipStoragionBreadcrumb,
    },
    children: [
      {
        path: PreservationPlanningRoutesEnum.aipNotFound + AppRoutesEnum.separator + AppRoutesEnum.paramId,
        component: AipNotFoundRoutable,
      },
      {
        path: PreservationPlanningRoutesEnum.aipList,
        component: AipTabsRoutable,
        data: {},
        children: [
          {
            path: PreservationPlanningRoutesEnum.tab,
            component: AipListRoutable,
            data: {
              breadcrumb: callbackAipTabBreadcrumb,
              noBreadcrumbLink: true,
            },
          },
        ],
      },
      {
        path: PreservationPlanningRoutesEnum.aipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
        component: AipDetailEditRoutable,
        data: {
          breadcrumbMemoizedSelector: PreservationAipState.currentAipName,
        },
        canActivate: [AipDetailGuardService],
        children: [
          {
            path: PreservationPlanningRoutesEnum.aipMetadata,
            component: AipMetadataRoutable,
            data: {
              breadscrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.aip.metadata"),
              noBreadcrumbLink: true,
            },
          },
          {
            path: PreservationPlanningRoutesEnum.aipFiles,
            component: AipAipRoutable,
            data: {
              breadscrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.aip.file"),
              noBreadcrumbLink: true,
            },
          },
        ],
      },
    ],
  },
];

@NgModule({
  imports: [AipSharedModule.forChild(routes)],
  exports: [AipSharedModule],
})
export class AipRoutingModule {
}
