import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {environment} from "@environments/environment";
import {Aip} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {AipFormPresentational} from "@preservation/aip/components/presentationals/aip-form/aip-form.presentational";
import {AipDataFile} from "@preservation/aip/models/aip-data-file.model";
import {PreservationAipAipAction} from "@preservation/aip/stores/aip-aip/aip-aip.action";
import {PreservationAipAipState} from "@preservation/aip/stores/aip-aip/aip-aip.state";
import {PreservationAipAipStatusHistoryAction} from "@preservation/aip/stores/aip-aip/status-history/aip-aip-status-history.action";
import {PreservationAipAipStatusHistoryState} from "@preservation/aip/stores/aip-aip/status-history/aip-aip-status-history.state";
import {preservationAipActionNameSpace} from "@preservation/aip/stores/aip.action";
import {PreservationAipStateModel} from "@preservation/aip/stores/aip.state";
import {
  SharedAipDetailDialog,
  SharedAipDetailDialogData,
} from "@shared/components/dialogs/shared-aip-detail/shared-aip-detail.dialog";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {ComplianceLevelEnumHelper} from "@shared/enums/business/compliance-level.enum";
import {DataFileStatusEnumHelper} from "@shared/enums/business/data-file-status.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@shared/enums/routes.enum";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {Observable} from "rxjs";
import {
  CompositionState,
  MemoizedUtil,
  OrderEnum,
  QueryParameters,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-aip-aip-routable",
  templateUrl: "./aip-aip.routable.html",
  styleUrls: ["./aip-aip.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AipAipRoutable extends SharedAbstractDetailEditRoutable<Aip, PreservationAipStateModel> implements OnInit {
  isLoadingDataFileObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, PreservationAipAipState);
  listDataFileObs: Observable<Aip[]> = CompositionState.list(this._store, PreservationAipAipState);
  queryParametersObs: Observable<QueryParameters> = CompositionState.queryParameters(this._store, PreservationAipAipState);

  isLoadingHistoryObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, PreservationAipAipStatusHistoryState);
  aipHistoryObs: Observable<StatusHistory[]> = MemoizedUtil.select(this._store, PreservationAipAipStatusHistoryState, state => state.history);
  queryParametersHistoryObs: Observable<QueryParameters> = MemoizedUtil.select(this._store, PreservationAipAipStatusHistoryState, state => state.queryParameters);

  @ViewChild("formPresentational")
  readonly formPresentational: AipFormPresentational;

  columns: DataTableColumns<Aip>[];
  actions: DataTableActions<Aip>[];

  readonly KEY_PARAM_NAME: keyof Aip & string = undefined;

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservation_aip, preservationAipActionNameSpace, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.getCurrentModelOnParent();

    this.columns = [
      {
        field: "info.name" as any,
        header: MARK_AS_TRANSLATABLE("preservation.aip.aip.table.aipName"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isSortable: true,
        isFilterable: false,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "info.status" as any,
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        translate: true,
        filterEnum: DataFileStatusEnumHelper.getListKeyValue(),
      },
      {
        field: "info.complianceLevel" as any,
        header: MARK_AS_TRANSLATABLE("deposit.datafile.table.complianceLevel"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        translate: true,
        component: DataTableComponentEnum.conformityLevelStar,
        filterEnum: ComplianceLevelEnumHelper.getListKeyValue(),
      },
    ];

    this.actions = [
      {
        logo: IconNameEnum.download,
        callback: (aip: Aip) => this.downloadAipAip(this._resId, aip),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.download"),
        displayOnCondition: (aip: Aip) => true,
      },
      {
        logo: IconNameEnum.internalLink,
        callback: (aip: Aip) => this.goToAip(aip),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.goToAip"),
        displayOnCondition: (aip: Aip) => true,
      },
    ];
  }

  private getCurrentModelOnParent(): void {
    this._resId = this._route.snapshot.parent.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    this.getAipById(this._resId);
  }

  private getAipById(id: string): void {
    this.getSubResourceWithParentId(id);
  }

  getSubResourceWithParentId(id: string): void {
    this._store.dispatch(new PreservationAipAipAction.GetAll(id, null, true));
    // this._store.dispatch(new PreservationSipDataFileAction.GetListFolder(id));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this._store.dispatch(new PreservationAipAipAction.ChangeQueryParameters(this._resId, queryParameters, true));
  }

  refresh(): void {
    this._store.dispatch(new PreservationAipAipAction.Refresh(this._resId));
  }

  download($event: AipDataFile): void {
    this._store.dispatch(new PreservationAipAipAction.DownloadAip(this._resId, $event));
  }

  showDetail(aip: Aip): void {
    const dialogRef = this._dialog.open(SharedAipDetailDialog, {
      data: {
        aip: aip,
        resId: aip.resId,
        parentId: this._resId,
        isReadonly: true,
        isLoading: this.isLoadingHistoryObs,
        statusHistory: this.aipHistoryObs,
        queryParametersObs: this.queryParametersHistoryObs,
        state: PreservationAipAipStatusHistoryAction,
        name: LocalStateEnum.preservation_aip,
        buttons: [
          {
            labelToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.download"),
            color: "primary",
            icon: IconNameEnum.download,
            callback: (current) => this.downloadAipAip(this._resId, aip),
            order: 2,
          },
        ],
      } as SharedAipDetailDialogData<Aip>,
      width: environment.modalWidth,
    });
  }

  private downloadAipAip(parentId: string, aip: Aip): void {
    this._store.dispatch(new PreservationAipAipAction.DownloadAip(parentId, aip));
  }

  private goToAip(aip: Aip): void {
    const storagion_number = this._route.snapshot.parent.parent.paramMap.get(PreservationPlanningRoutesEnum.storagionNumberWithoutPrefixParam);
    this._store.dispatch(new PreservationAipAipAction.GoToAip(aip, storagion_number));
  }

}
