import {
  ChangeDetectionStrategy,
  Component,
} from "@angular/core";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {SharedAbstractRoutable} from "@shared/components/routables/shared-abstract/shared-abstract.routable";
import {BaseResourceApiEnum} from "@shared/enums/api.enum";
import {ApplicationRoleEnum} from "@shared/enums/application-role.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {
  AppRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {LocalStateModel} from "@shared/models/local-state.model";
import {Storage} from "@shared/models/storage.model";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

@Component({
  selector: "dlcm-aip-home-routable",
  templateUrl: "./aip-home.routable.html",
  styleUrls: ["./aip-home.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AipHomeRoutable extends SharedAbstractRoutable {

  userRolesObs: ApplicationRoleEnum[];

  aipResources: AipResource[] = [];

  constructor(private readonly _store: Store) {
    super();
    this.userRolesObs = this._store.selectSnapshot((state: LocalStateModel) => state.application.userRoles);
  }

  navigate(path: RoutesEnum): void {
    this._store.dispatch(new Navigate([path]));
  }

  getAipResources(): AipResource[] {
    const storagionUrls: Storage[] = BaseResourceApiEnum.archivalStorageList;
    let i = 1;
    storagionUrls.forEach(url => {
      this.aipResources.push(
        {
          index: url.index,
          avatarIcon: IconNameEnum.storagion,
          titleToTranslate: MARK_AS_TRANSLATABLE("app.aip.storage"),
          path: RoutesEnum.preservationAip + AppRoutesEnum.separator + url.index,
          isVisible: () => true,
          url: url.url,
        });
      i++;
    });
    return this.aipResources;
  }

  back(): void {
    this._store.dispatch(new Navigate([RoutesEnum.preservationPlanning]));
  }
}

interface AipResource {
  index: number;
  avatarIcon: IconNameEnum;
  titleToTranslate: string;
  path: RoutesEnum;
  isVisible: () => boolean;
  url: string;
}
