import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {Aip} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {AipFormPresentational} from "@preservation/aip/components/presentationals/aip-form/aip-form.presentational";
import {preservationAipActionNameSpace} from "@preservation/aip/stores/aip.action";
import {
  PreservationAipState,
  PreservationAipStateModel,
} from "@preservation/aip/stores/aip.state";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {Observable} from "rxjs";

@Component({
  selector: "dlcm-aip-metadata-routable",
  templateUrl: "./aip-metadata.routable.html",
  styleUrls: ["./aip-metadata.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})

export class AipMetadataRoutable extends SharedAbstractDetailEditRoutable<Aip, PreservationAipStateModel> implements OnInit, OnDestroy {
  @Select(PreservationAipState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(PreservationAipState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;

  @ViewChild("formPresentational")
  readonly formPresentational: AipFormPresentational;

  readonly KEY_PARAM_NAME: keyof Aip & string = undefined;

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservation_aip, preservationAipActionNameSpace, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
  }

  protected getSubResourceWithParentId(id: string): void {
  }
}
