import {
  ChangeDetectionStrategy,
  Component,
  OnInit,
} from "@angular/core";
import {
  ActivatedRoute,
  NavigationEnd,
  Router,
} from "@angular/router";
import {Storage} from "@app/shared/models/storage.model";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {SharedAbstractRoutable} from "@shared/components/routables/shared-abstract/shared-abstract.routable";
import {BaseResourceApiEnum} from "@shared/enums/api.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {tap} from "rxjs/internal/operators/tap";
import {
  distinctUntilChanged,
  filter,
} from "rxjs/operators";

@Component({
  selector: "dlcm-aip-not-found-routable",
  templateUrl: "./aip-not-found.routable.html",
  styleUrls: ["./aip-not-found.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AipNotFoundRoutable extends SharedAbstractRoutable implements OnInit {
  aipId: string;
  storagion_number: number | undefined;

  listStorage: Storage[];

  constructor(private readonly _store: Store,
              private readonly _route: ActivatedRoute,
              private readonly _router: Router) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.retrieveAttribute();

    this.subscribe(this._router.events
      .pipe(
        filter(event => event instanceof NavigationEnd),
        distinctUntilChanged(),
        tap(event => {
          this.retrieveAttribute();
        }),
      ),
    );
  }

  retrieveAttribute(): void {
    this.storagion_number = +this._route.snapshot.parent.paramMap.get(PreservationPlanningRoutesEnum.storagionNumberWithoutPrefixParam);
    this.aipId = this._route.snapshot.paramMap.get(AppRoutesEnum.paramIdWithoutPrefixParam);
    this.listStorage = BaseResourceApiEnum.archivalStorageList.filter(storage => storage.index !== this.storagion_number);
  }

  back(): void {
    this.navigate([RoutesEnum.preservationAip]);
  }

  navigateToStorageAipDetail(storage: Storage): void {
    this.navigate([RoutesEnum.preservationAip, storage.index, PreservationPlanningRoutesEnum.aipDetail, this.aipId]);
  }

  private navigate(path: (string | number)[]): void {
    this._store.dispatch(new Navigate(path));
  }
}
