import {ActivatedRoute} from "@angular/router";
import {Store} from "@ngxs/store";
import {AipTabEnum} from "@preservation/aip/enums/aip-tab.enum";
import {
  AccessResourceApiEnum,
  ArchivalStorageResourceApiEnum
} from "@shared/enums/api.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@shared/enums/routes.enum";
import {
  isNullOrUndefined,
  isNumber,
  StringUtil,
} from "solidify-frontend";

export class AipHelper {
  static getTabRouteSelected(route: ActivatedRoute): PreservationPlanningRoutesEnum | undefined {
    const children = route.snapshot.children;
    if (children.length > 0 && children[0].url.length > 0) {
      const mode = children[0].url[0].path;
      return mode as PreservationPlanningRoutesEnum;
    }
    return undefined;
  }

  static getTabSelectedIndexWithRoute(tabRouteSelected: PreservationPlanningRoutesEnum): AipTabEnum | undefined {
    if (tabRouteSelected === PreservationPlanningRoutesEnum.aipMetadata) {
      return AipTabEnum.metadata;
    }
    if (tabRouteSelected === PreservationPlanningRoutesEnum.aipFiles) {
      return AipTabEnum.file;
    }
    return undefined;
  }

  static getTabRouteSelectedWithTabIndex(tabIndexSelected: AipTabEnum): PreservationPlanningRoutesEnum | undefined {
    if (tabIndexSelected === AipTabEnum.metadata) {
      return PreservationPlanningRoutesEnum.aipMetadata;
    }
    if (tabIndexSelected === AipTabEnum.file) {
      return PreservationPlanningRoutesEnum.aipFiles;
    }
    return undefined;
  }

  static generateUrlResource(store: Store, storagionNumber: number | undefined = undefined): string {
    if (isNullOrUndefined(store)) {
      return StringUtil.stringEmpty;
    }

    const url = store.selectSnapshot(state => state.router.state.url);
    if (url.includes(PreservationPlanningRoutesEnum.aipDownloaded)) {
      return AccessResourceApiEnum.aipDownloaded;
    }

    if (isNullOrUndefined(storagionNumber)) {
      if (url.includes(PreservationPlanningRoutesEnum.aipDetail)) {
        const urlParts = url.split(AppRoutesEnum.separator);
        storagionNumber = urlParts[urlParts.indexOf(PreservationPlanningRoutesEnum.aipDetail) - 1];
      } else {
        storagionNumber = store.selectSnapshot(state => state.router.state.root.children[0].children[0].children[0].params[PreservationPlanningRoutesEnum.storagionNumberWithoutPrefixParam]);
      }
    }

    const storagion = ArchivalStorageResourceApiEnum.aipStorages.find(aip => aip.index === +storagionNumber);
    if (isNullOrUndefined(storagion)) {
      if (String(storagionNumber) === PreservationPlanningRoutesEnum.job) {
        //in case the request comes from preservation job to show the history status, return the principal storagion url
         return ArchivalStorageResourceApiEnum.aip;
      }
      throw new Error(`The storagion index '${storagionNumber}' is not find in setting`);
    }
    return storagion.url;
  }

  static determineMode(store: Store): AipMode {
    const url = store.selectSnapshot(state => state.router.state.url);
    if (url.includes(PreservationPlanningRoutesEnum.aipDownloaded)) {
      return AipMode.DOWNLOADED_AIP;
    }
    return AipMode.AIP;
  }
}

export enum AipMode {
  AIP = "AIP",
  DOWNLOADED_AIP = "DOWNLOADED_AIP",
}
