import {
  Inject,
  Injectable,
} from "@angular/core";
import {WINDOW} from "@app/app.module";
import {ApiActionEnum} from "@app/shared/enums/api-action.enum";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {ArchivalStorageResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Aip} from "@models";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {
  PreservationAipAipAction,
  preservationAipAipActionNameSpace,
} from "@preservation/aip/stores/aip-aip/aip-aip.action";
import {
  PreservationAipAipStatusHistoryState,
  PreservationAipAipStatusHistoryStateModel,
} from "@preservation/aip/stores/aip-aip/status-history/aip-aip-status-history.state";
import {PreservationAipAction} from "@preservation/aip/stores/aip.action";
import {DownloadService} from "@shared/services/download.service";
import {defaultStatusHistoryInitValue} from "@shared/stores/status-history/status-history.state";
import {
  ApiService,
  CompositionState,
  CompositionStateModel,
  defaultCompositionStateInitValue,
  isNullOrUndefined,
  NotificationService,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

export const defaultAipAipValue: () => PreservationAipAipStateModel = () =>
  ({
    ...defaultCompositionStateInitValue(),
    preservation_aip_aip_statusHistory: {...defaultStatusHistoryInitValue()},
  });

export interface PreservationAipAipStateModel extends CompositionStateModel<Aip> {
  preservation_aip_aip_statusHistory: PreservationAipAipStatusHistoryStateModel;
}

@Injectable()
@State<PreservationAipAipStateModel>({
  name: LocalStateEnum.preservation_aip_aip,
  defaults: {
    ...defaultAipAipValue(),
    preservation_aip_aip_statusHistory: {...defaultStatusHistoryInitValue()},
  },
  children: [
    PreservationAipAipStatusHistoryState,
  ],
})
export class PreservationAipAipState extends CompositionState<PreservationAipAipStateModel, Aip> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              @Inject(WINDOW) private _window: Window,
              private downloadService: DownloadService) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationAipAipActionNameSpace,
      resourceName: ApiResourceNameEnum.AIP,
    });
  }

  protected get _urlResource(): string {
    let uid = "";
    if (isNullOrUndefined(this.store)) {
      return "";
    }
    const url = this.store.selectSnapshot(state => state.router.state.url);
    if (url.includes("detail")) {
      const urlParts = url.split("/");
      uid = urlParts[urlParts.indexOf("detail") - 1];
    } else {
      uid = this.store.selectSnapshot(state => state.router.state.root.children[0].children[0].children[0].params["id"]);
    }
    return ArchivalStorageResourceApiEnum.aipStorages.find(node => node.index === Number(uid)).url;
  }

  @Action(PreservationAipAipAction.Refresh)
  refresh(ctx: StateContext<PreservationAipAipStateModel>, action: PreservationAipAipAction.Refresh): void {
    ctx.dispatch(new PreservationAipAipAction.GetAll(action.parentId, undefined, true));
  }

  @Action(PreservationAipAipAction.DownloadAip)
  downloadAip(ctx: StateContext<PreservationAipAipStateModel>, action: PreservationAipAipAction.DownloadAip): void {
    const url = `${this._urlResource}/${action.parentId}/${this._resourceName}/${action.aip.resId}/${ApiActionEnum.DL}`;
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.deposit.file.download"), true);
    this.downloadService.download(url, action.aip.info.name, action.aip.archiveSize).subscribe(() => {
      this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.deposit.file.downloadWithSuccess"), true);
    });
  }

  @Action(PreservationAipAipAction.GoToAip)
  goToAip(ctx: StateContext<PreservationAipAipStateModel>, action: PreservationAipAipAction.GoToAip): void {
    ctx.patchState({
      current: undefined,
      list: undefined,
    });
    ctx.dispatch(new PreservationAipAction.GoToAip(action.aip, action.storagion_number));
  }
}
