import {Injectable} from "@angular/core";
import {ArchivalStorageResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Aip} from "@models";

import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {preservationAipAipStatusHistoryNamespace} from "@preservation/aip/stores/aip-aip/status-history/aip-aip-status-history.action";
import {
  defaultStatusHistoryInitValue,
  StatusHistoryState,
  StatusHistoryStateModel,
} from "@shared/stores/status-history/status-history.state";
import {
  ApiService,
  isNullOrUndefined,
  NotificationService,
} from "solidify-frontend";

export interface PreservationAipAipStatusHistoryStateModel extends StatusHistoryStateModel<Aip> {
}

@Injectable()
@State<PreservationAipAipStatusHistoryStateModel>({
  name: LocalStateEnum.preservation_aip_aip_statusHistory,
  defaults: {
    ...defaultStatusHistoryInitValue(),
  },
})
export class PreservationAipAipStatusHistoryState extends StatusHistoryState<PreservationAipAipStatusHistoryStateModel, Aip> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationAipAipStatusHistoryNamespace,
    });
  }

  protected get _urlResource(): string {
    let uid = "";
    if (isNullOrUndefined(this.store)) {
      return "";
    }
    const url = this.store.selectSnapshot(state => state.router.state.url);
    if (url.includes("detail")) {
      const urlParts = url.split("/");
      uid = urlParts[urlParts.indexOf("detail") - 1];
    } else {
      uid = this.store.selectSnapshot(state => state.router.state.root.children[0].children[0].children[0].params["id"]);
    }

    return ArchivalStorageResourceApiEnum.aipStorages.find(aip => aip.index === +uid).url;
  }
}
