import {Injectable} from "@angular/core";
import {preservationAipOrgUnitActionNameSpace} from "@app/features/preservation/aip/stores/organizational-unit/aip-organizational-unit.action";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {AccessOrganizationalUnit} from "@models";
import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
} from "solidify-frontend";
import {environment} from "../../../../../../environments/environment";

export const defaultAipOrgUnitValue: () => PreservationAipOrganizationalUnitStateModel = () =>
  ({
    ...defaultResourceStateInitValue(),
    queryParameters: new QueryParameters(environment.defaultPageSize),
    search: null,
  });

export interface PreservationAipOrganizationalUnitStateModel extends ResourceStateModel<AccessOrganizationalUnit> {
  search: string;
}

@Injectable()
@State<PreservationAipOrganizationalUnitStateModel>({
  name: LocalStateEnum.preservation_aip_organizationalUnit,
  defaults: {
    ...defaultAipOrgUnitValue(),
  },
})
export class PreservationAipOrganizationalUnitState extends ResourceState<PreservationAipOrganizationalUnitStateModel, AccessOrganizationalUnit> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationAipOrgUnitActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.organizationalUnits;
  }

}
