import {Injectable} from "@angular/core";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Aip} from "@models";

import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {AipHelper} from "@preservation/aip/helpers/aip.helper";
import {preservationAipStatusHistoryNamespace} from "@preservation/aip/stores/status-history/aip-status-history.action";
import {
  defaultStatusHistoryInitValue,
  StatusHistoryState,
  StatusHistoryStateModel,
} from "@shared/stores/status-history/status-history.state";
import {
  ApiService,
  NotificationService,
} from "solidify-frontend";

export interface PreservationAipStatusHistoryStateModel extends StatusHistoryStateModel<Aip> {
}

@Injectable()
@State<PreservationAipStatusHistoryStateModel>({
  name: LocalStateEnum.preservation_aip_statusHistory,
  defaults: {
    ...defaultStatusHistoryInitValue(),
  },
})
export class PreservationAipStatusHistoryState extends StatusHistoryState<PreservationAipStatusHistoryStateModel, Aip> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationAipStatusHistoryNamespace,
    });
  }

  protected get _urlResource(): string {
    return AipHelper.generateUrlResource(this.store);
  }
}
