import {
  ChangeDetectionStrategy,
  Component,
} from "@angular/core";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {TranslateService} from "@ngx-translate/core";
import {Store} from "@ngxs/store";
import {
  SharedAbstractHomeRoutable,
  SharedHomeTileModel,
} from "@shared/components/routables/shared-abstract-home/shared-abstract-home.routable";
import {ApplicationRolePermissionEnum} from "@shared/enums/application-role-permission.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

@Component({
  selector: "dlcm-preservation-home-routable",
  templateUrl: "./preservation-home.routable.html",
  styleUrls: ["./preservation-home.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PreservationHomeRoutable extends SharedAbstractHomeRoutable {

  _tiles: SharedHomeTileModel[] = [
    {
      avatarIcon: IconNameEnum.sip,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservation.sip.home.title"),
      subtitleToTranslate: MARK_AS_TRANSLATABLE("preservation.sip.home.subtitle"),
      path: RoutesEnum.preservationSip,
      permission: ApplicationRolePermissionEnum.noPermission,
    },
    {
      avatarIcon: IconNameEnum.aip,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservation.aip.home.title"),
      subtitleToTranslate: MARK_AS_TRANSLATABLE("preservation.aip.home.subtitle"),
      path: RoutesEnum.preservationAip,
      permission: ApplicationRolePermissionEnum.noPermission,
    },
    {
      avatarIcon: IconNameEnum.dip,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservation.dip.home.title"),
      subtitleToTranslate: MARK_AS_TRANSLATABLE("preservation.dip.home.subtitle"),
      path: RoutesEnum.preservationDip,
      permission: ApplicationRolePermissionEnum.noPermission,
    },
    {
      avatarIcon: IconNameEnum.monitoring,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservation.monitoring.home.title"),
      subtitleToTranslate: MARK_AS_TRANSLATABLE("preservation.monitoring.home.subtitle"),
      path: RoutesEnum.preservationMonitoring,
      permission: ApplicationRolePermissionEnum.rootPermission,
    },
    {
      avatarIcon: IconNameEnum.jobs,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservation.jobs.home.title"),
      subtitleToTranslate: MARK_AS_TRANSLATABLE("preservation.jobs.home.subtitle"),
      path: RoutesEnum.preservationJob,
      permission: ApplicationRolePermissionEnum.noPermission,
    },
    {
      avatarIcon: IconNameEnum.archivingStatus,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservation.aipStats.home.title"),
      subtitleToTranslate: MARK_AS_TRANSLATABLE("preservation.aipStats.home.subtitle"),
      path: RoutesEnum.preservationAipStatuses,
      permission: ApplicationRolePermissionEnum.rootPermission,
    },
    {
      avatarIcon: IconNameEnum.aipDownloaded,
      titleToTranslate: MARK_AS_TRANSLATABLE("preservation.aipDownloaded.home.title"),
      subtitleToTranslate: MARK_AS_TRANSLATABLE("preservation.aipDownloaded.home.subtitle"),
      path: RoutesEnum.preservationAipDownloaded,
      permission: ApplicationRolePermissionEnum.noPermission,
    },
  ];

  constructor(protected readonly _store: Store,
              protected readonly _translate: TranslateService) {
    super(_store, _translate);
  }
}
