import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {DipStateModel} from "@app/features/preservation/dip/stores/dip.state";
import {SharedAbstractListRoutable} from "@app/shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {appAuthorizedOrganizationalUnitNameSpace} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.action";
import {AppAuthorizedOrganizationalUnitState} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.state";
import {
  Dip,
  PreservationJob,
} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {
  PreservationDipAction,
  preservationDipActionNameSpace,
} from "@preservation/dip/stores/dip.action";
import {PackageStatusEnumHelper} from "@shared/enums/business/package-status.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {RouterExtService} from "@shared/services/router-ext.service";
import {
  OrderEnum,
  Override,
  ResourceNameSpace,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-dip-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DipListRoutable extends SharedAbstractListRoutable<Dip, DipStateModel> implements OnInit {
  readonly KEY_CREATE_BUTTON: string = undefined;
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("preservation.dip.refresh");
  readonly KEY_BACK_BUTTON: string = MARK_AS_TRANSLATABLE("preservation.dip.back");
  readonly KEY_PARAM_NAME: keyof Dip & string = undefined;

  appAuthorizedOrganizationalUnitNameSpace: ResourceNameSpace = appAuthorizedOrganizationalUnitNameSpace;
  appAuthorizedOrganizationalUnitState: typeof AppAuthorizedOrganizationalUnitState = AppAuthorizedOrganizationalUnitState;

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.preservation_dip, preservationDipActionNameSpace, {
      canCreate: false,
    }, LocalStateEnum.preservation);
  }

  conditionDisplayEditButton(model: Dip | undefined): boolean {
    return false;
  }

  conditionDisplayDeleteButton(model: Dip | undefined): boolean {
    return false;
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "info.name" as any,
        header: MARK_AS_TRANSLATABLE("preservation.dip.table.header.title"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "info.organizationalUnitId" as any,
        header: MARK_AS_TRANSLATABLE("preservation.dip.table.header.organizationalUnit"),
        type: FieldTypeEnum.searchableSingleSelect,
        order: OrderEnum.none,
        component: DataTableComponentEnum.organizationalUnitName,
        isFilterable: true,
        isSortable: true,
        resourceNameSpace: this.appAuthorizedOrganizationalUnitNameSpace,
        resourceState: this.appAuthorizedOrganizationalUnitState as any,
        filterableField: "info.organizationalUnitId" as any,
        sortableField: "info.organizationalUnitId" as any,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("preservation.dip.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: MARK_AS_TRANSLATABLE("preservation.dip.table.header.lastUpdate.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "info.status" as any,
        header: MARK_AS_TRANSLATABLE("preservation.dip.table.header.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: PackageStatusEnumHelper.getListKeyValue(),
      },
    ];
  }

  @Override()
  protected defineActions(): DataTableActions<PreservationJob>[] {
    return [
      {
        logo: IconNameEnum.download,
        callback: (model: Dip) => this.download(model),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.download"),
        displayOnCondition: (model: Dip) => true,
      },
    ];
  }

  private download(dip: Dip): void {
    this._store.dispatch(new PreservationDipAction.Download(dip.resId));
  }
}
