import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {
  DipState,
  DipStateModel,
} from "@app/features/preservation/dip/stores/dip.state";
import {DepositFormPresentational} from "@deposit/components/presentationals/deposit-form/deposit-form.presentational";
import {Dip} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {preservationDipActionNameSpace} from "@preservation/dip/stores/dip.action";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {Observable} from "rxjs";

@Component({
  selector: "dlcm-dip-metadata-routable",
  templateUrl: "./dip-metadata.routable.html",
  styleUrls: ["./dip-metadata.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})

export class DipMetadataRoutable extends SharedAbstractDetailEditRoutable<Dip, DipStateModel> implements OnInit, OnDestroy {
  @Select(DipState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(DipState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;

  @ViewChild("formPresentational")
  readonly formPresentational: DepositFormPresentational;

  readonly KEY_PARAM_NAME: keyof Dip & string = undefined;

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservation_dip, preservationDipActionNameSpace, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
  }

  protected getSubResourceWithParentId(id: string): void {
  }
}
