import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@app/shared/enums/routes.enum";
import {DlcmRoutes} from "@app/shared/models/dlcm-route.model";
import {DipDetailEditRoutable} from "@preservation/dip/components/routables/dip-detail-edit/dip-detail-edit.routable";
import {DipFileRoutable} from "@preservation/dip/components/routables/dip-file/dip-file.routable";
import {DipListRoutable} from "@preservation/dip/components/routables/dip-list/dip-list.routable";
import {DipMetadataRoutable} from "@preservation/dip/components/routables/dip-metadata/dip-metadata.routable";
import {DipState} from "@preservation/dip/stores/dip.state";
import {CanDeactivateGuard} from "@shared/services/can-deactivate-guard.service";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

const routes: DlcmRoutes = [
  {
    path: AppRoutesEnum.root,
    component: DipListRoutable,
    data: {},
  },
  {
    path: PreservationPlanningRoutesEnum.dipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    redirectTo: PreservationPlanningRoutesEnum.dipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.dipMetadata,
    pathMatch: "full",
  },
  {
    path: PreservationPlanningRoutesEnum.dipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: DipDetailEditRoutable,
    data: {
      breadcrumbMemoizedSelector: DipState.currentTitle,
    },
    children: [
      {
        path: PreservationPlanningRoutesEnum.dipMetadata,
        component: DipMetadataRoutable,
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.dip.metadata"),
          noBreadcrumbLink: true,
        },
        children: [
          {
            path: PreservationPlanningRoutesEnum.dipEdit,
            data: {
              breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.dip.edit"),
            },
            canDeactivate: [CanDeactivateGuard],
          },
        ],
      },
      {
        path: PreservationPlanningRoutesEnum.dipFiles,
        component: DipFileRoutable,
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.dip.file"),
          noBreadcrumbLink: true,
        },
        children: [
          {
            path: PreservationPlanningRoutesEnum.dipEdit,
            data: {
              breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.dip.edit"),
            },
            canDeactivate: [CanDeactivateGuard],
          },
        ],
      },
    ],
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class DipRoutingModule {
}
