import {
  Inject,
  Injectable,
} from "@angular/core";
import {WINDOW} from "@app/app.module";
import {
  PreservationDipDataFileAction,
  preservationDipDataFileActionNameSpace,
} from "@app/features/preservation/dip/stores/data-file/dip-data-file.action";
import {ApiActionEnum} from "@app/shared/enums/api-action.enum";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {AccessResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {DipDataFile} from "@preservation/dip/models/dip-data-file.model";
import {
  DipDataFileStatusHistoryState,
  DipDataFileStatusHistoryStateModel,
} from "@preservation/dip/stores/data-file/status-history/dip-data-file-status-history.state";
import {DownloadService} from "@shared/services/download.service";
import {defaultStatusHistoryInitValue} from "@shared/stores/status-history/status-history.state";
import {Observable} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {
  ApiService,
  CompositionState,
  CompositionStateModel,
  defaultCompositionStateInitValue,
  NotificationService,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

export const defaultDipDataFileValue: () => DipDataFileStateModel = () =>
  ({
    ...defaultCompositionStateInitValue(),
    preservation_dip_dataFile_statusHistory: defaultStatusHistoryInitValue(),
  });

export interface DipDataFileStateModel extends CompositionStateModel<DipDataFile> {
  preservation_dip_dataFile_statusHistory: DipDataFileStatusHistoryStateModel;
}

@Injectable()
@State<DipDataFileStateModel>({
  name: LocalStateEnum.preservation_dip_dataFile,
  defaults: {
    ...defaultDipDataFileValue(),
  },
  children: [
    DipDataFileStatusHistoryState,
  ],
})
export class DipDataFileState extends CompositionState<DipDataFileStateModel, DipDataFile> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              @Inject(WINDOW) private _window: Window,
              private downloadService: DownloadService) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationDipDataFileActionNameSpace,
      resourceName: ApiResourceNameEnum.DATAFILE,
    });
  }

  protected get _urlResource(): string {
    return AccessResourceApiEnum.dip;
  }

  @Action(PreservationDipDataFileAction.Refresh)
  refresh(ctx: StateContext<DipDataFileStateModel>, action: PreservationDipDataFileAction.Refresh): void {
    ctx.dispatch(new PreservationDipDataFileAction.GetAll(action.parentId, undefined, true));
  }

  @Action(PreservationDipDataFileAction.Download)
  download(ctx: StateContext<DipDataFileStateModel>, action: PreservationDipDataFileAction.Download): void {
    const url = `${this._urlResource}/${action.parentId}/${this._resourceName}/${action.dataFile.resId}/${ApiActionEnum.DL}`;
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.deposit.file.download"), true);
    this.downloadService.download(url, action.dataFile.fileName, action.dataFile.fileSize).subscribe(() => {
      this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("notification.deposit.file.downloadWithSuccess"));
    });
  }

  @Action(PreservationDipDataFileAction.Resume)
  resume(ctx: StateContext<DipDataFileStateModel>, action: PreservationDipDataFileAction.Resume): Observable<void> {
    return this.apiService.post<void>(`${this._urlResource}/${action.parentId}/${this._resourceName}/${action.dataFile.resId}/${ApiActionEnum.RESUME}`)
      .pipe(
        tap(() => {
          this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.deposit.file.resumed"), true);
          ctx.dispatch(new PreservationDipDataFileAction.GetAll(action.parentId, undefined, true));
        }),
      );
  }
}
