import {
  ChangeDetectorRef,
  Component,
  Inject,
  OnInit,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {PreservationJobExecutionState} from "@app/features/preservation/job/stores/preservation-job-execution/preservation-job-execution.state";
import {PreservationJobState} from "@app/features/preservation/job/stores/preservation-job.state";
import {PreservationJob} from "@models";
import {
  Actions,
  ofActionCompleted,
  Store,
} from "@ngxs/store";
import {PreservationJobExecutionAction} from "@preservation/job/stores/preservation-job-execution/preservation-job-execution.action";
import {SharedAbstractContainer} from "@shared/components/containers/shared-abstract/shared-abstract.container";
import {JobStatusEnum} from "@shared/enums/business/job-status.enum";
import {JobExecution} from "@shared/models/business/job-execution.model";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {Observable} from "rxjs";
import {tap} from "rxjs/operators";
import {
  CompositionState,
  KeyValue,
  MemoizedUtil,
  QueryParameters,
  ResourceState,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-job-execution-detail-dialog",
  templateUrl: "./job-execution-detail.dialog.html",
  styleUrls: ["./job-execution-detail.dialog.scss"],
})
export class JobExecutionDetailDialog extends SharedAbstractContainer implements OnInit {
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, PreservationJobExecutionState);
  listObs: Observable<JobExecution[]> = CompositionState.list(this._store, PreservationJobExecutionState);
  queryParametersObs: Observable<QueryParameters> = CompositionState.queryParameters(this._store, PreservationJobExecutionState);
  job: Observable<PreservationJob> = ResourceState.current(this._store, PreservationJobState);
  columns: DataTableColumns<JobExecution>[];
  listKeyValue: KeyValue[];

  get jobStatusEnum(): typeof JobStatusEnum {
    return JobStatusEnum;
  }

  constructor(private readonly _store: Store,
              private readonly _changeDetector: ChangeDetectorRef,
              private readonly _route: ActivatedRoute,
              private readonly _actions$: Actions,
              protected readonly _dialogRef: MatDialogRef<JobExecutionDetailDialog>,
              @Inject(MAT_DIALOG_DATA) public jobExecution: JobExecution) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.listKeyValue = [
      {
        key: MARK_AS_TRANSLATABLE("preservation.jobExecutionDetail.status"),
        value: this.jobExecution.status,
      },
      {
        key: MARK_AS_TRANSLATABLE("preservation.jobExecutionDetail.runNumber"),
        value: "" + this.jobExecution.runNumber,
      },
      {
        key: MARK_AS_TRANSLATABLE("preservation.jobExecutionDetail.totalItems"),
        value: "" + (this.jobExecution.ignoredItems + this.jobExecution.inErrorItems + this.jobExecution.processedItems),
      },
      {
        key: MARK_AS_TRANSLATABLE("preservation.jobExecutionDetail.ignoredItems"),
        value: "" + this.jobExecution.ignoredItems,
      },
      {
        key: MARK_AS_TRANSLATABLE("preservation.jobExecutionDetail.inErrorItems"),
        value: "" + this.jobExecution.inErrorItems,
      },
      {
        key: MARK_AS_TRANSLATABLE("preservation.jobExecutionDetail.processedItems"),
        value: "" + this.jobExecution.processedItems,
      },
    ];
  }

  resume(): void {
    this._store.dispatch(new PreservationJobExecutionAction.Resume(this.jobExecution.jobId, this.jobExecution.resId));
    this.subscribe(this._actions$.pipe(ofActionCompleted(PreservationJobExecutionAction.ResumeSuccess))
      .pipe(
        tap(result => {
          if (result.result.successful) {
            this._dialogRef.close();
          }
        }),
      ));
  }

  start(): void {
    this._store.dispatch(new PreservationJobExecutionAction.Start(this.jobExecution.jobId, this.jobExecution));
    this.subscribe(this._actions$.pipe(ofActionCompleted(PreservationJobExecutionAction.StartSuccess))
      .pipe(
        tap(result => {
          if (result.result.successful) {
            this._dialogRef.close();
          }
        }),
      ));
  }
}
