import {
  ChangeDetectorRef,
  Component,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {
  ActivatedRoute,
  Router,
  RouterStateSnapshot,
} from "@angular/router";
import {JobExecutionListContainer} from "@app/features/preservation/job/components/containers/job-execution-list/job-execution-list.container";
import {
  PreservationJobAction,
  preservationJobActionNameSpace,
} from "@app/features/preservation/job/stores/preservation-job.action";
import {
  PreservationJobState,
  PreservationJobStateModel,
} from "@app/features/preservation/job/stores/preservation-job.state";
import {
  Actions,
  ofActionCompleted,
  Select,
  Store,
} from "@ngxs/store";
import {PreservationJobExtended} from "@preservation/job/models/preservation-job-extended.model";
import {PreservationJobExecutionState} from "@preservation/job/stores/preservation-job-execution/preservation-job-execution.state";
import {SharedAbstractDetailEditCommonRoutable} from "@shared/components/routables/shared-abstract-detail-edit-common/shared-abstract-detail-edit-common.routable";
import {JobStatusEnum} from "@shared/enums/business/job-status.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {PreservationPlanningRoutesEnum} from "@shared/enums/routes.enum";
import {JobExecution} from "@shared/models/business/job-execution.model";
import {JobRecurrence} from "@shared/models/business/job-recurrence.model";
import {ExtraButtonToolbar} from "@shared/models/extra-button-toolbar.model";
import {LocalStateModel} from "@shared/models/local-state.model";
import {Observable} from "rxjs";
import {tap} from "rxjs/operators";
import {
  CompositionState,
  isNullOrUndefined,
  MemoizedUtil,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";
import JobRecurrenceEnum = JobRecurrence.JobRecurrenceEnum;

@Component({
  selector: "dlcm-job-detail-edit-routable",
  templateUrl: "./job-detail-edit.routable.html",
  styleUrls: ["./job-detail-edit.routable.scss"],
})
export class JobDetailEditRoutable extends SharedAbstractDetailEditCommonRoutable<PreservationJobExtended, PreservationJobStateModel> implements OnInit {
  @Select(PreservationJobState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(PreservationJobState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.router.state) urlStateObs: Observable<RouterStateSnapshot>;

  listObs: Observable<JobExecution[]> = CompositionState.list(this._store, PreservationJobExecutionState);

  @ViewChild("jobExecutionList")
  readonly jobExecutionList: JobExecutionListContainer;

  readonly KEY_PARAM_NAME: keyof PreservationJobExtended & string = "name";
  preservationJob: boolean | undefined = undefined;

  listExtraButtons: ExtraButtonToolbar<PreservationJobExtended>[] = [
    {
      color: "primary",
      icon: IconNameEnum.run,
      labelToTranslate: MARK_AS_TRANSLATABLE("preservation.jobExecution.button.start"),
      callback: () => this.jobExecutionList.start(),
      displayCondition: (resource) => !this.isEdit && resource && (resource.jobRecurrence !== JobRecurrenceEnum.ONCE || MemoizedUtil.select(this._store, PreservationJobExecutionState, state => !isNullOrUndefined(state.list) && state.list.length === 0)),
      order: 41,
    },
    {
      color: "primary",
      icon: IconNameEnum.resume,
      labelToTranslate: MARK_AS_TRANSLATABLE("preservation.jobExecution.button.resume"),
      callback: () => this.jobExecutionList.resume(),
      displayCondition: (resource) => !this.isEdit && resource && (resource.lastExecutionStatus === JobStatusEnum.IN_ERROR),
      order: 41,
    },
  ];

  constructor(protected _store: Store,
              protected _route: ActivatedRoute,
              protected _router: Router,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              public _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservation_job, preservationJobActionNameSpace, LocalStateEnum.preservation);
  }

  getSubResourceWithParentId(id: string): void {
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.subscribe(this.urlStateObs
      .pipe(
        tap(urLState => {
          if (urLState) {
            const url = urLState.url;
            this.preservationJob = !url.includes(PreservationPlanningRoutesEnum.execution);
          }
        }),
      ));
  }

  start(): void {
    this._store.dispatch(new PreservationJobAction.VerifyBeforeStart(this._resId));
    this.subscribe(this._actions$.pipe(ofActionCompleted(PreservationJobAction.StartSuccess)),
      result => {
        if (result.result.successful === true) {
          this.jobExecutionList.getAll();
        }
      });
  }

  refresh(): void {
    this.jobExecutionList.getAll();
  }
}
