import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {JobCreateRoutable} from "@app/features/preservation/job/components/routables/job-create/job-create.routable";
import {JobDetailEditRoutable} from "@app/features/preservation/job/components/routables/job-detail-edit/job-detail-edit.routable";
import {JobListRoutable} from "@app/features/preservation/job/components/routables/job-list/job-list.routable";
import {PreservationJobState} from "@app/features/preservation/job/stores/preservation-job.state";
import {DlcmRoutes} from "@app/shared/models/dlcm-route.model";
import {JobExecutionDetailRoutable} from "@preservation/job/components/routables/job-execution-detail/job-execution-detail.routable";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@shared/enums/routes.enum";
import {CanDeactivateGuard} from "@shared/services/can-deactivate-guard.service";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

const routes: DlcmRoutes = [
  {
    path: AppRoutesEnum.root,
    component: JobListRoutable,
    data: {},
    children: [],
  },
  {
    path: PreservationPlanningRoutesEnum.jobCreate,
    component: JobCreateRoutable,
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.job.create"),
    },
    canDeactivate: [CanDeactivateGuard],
  },
  {
    path: PreservationPlanningRoutesEnum.jobDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: JobDetailEditRoutable,
    data: {
      breadcrumbMemoizedSelector: PreservationJobState.currentTitle,
    },
    children: [
      {
        path: PreservationPlanningRoutesEnum.execution + AppRoutesEnum.separator + AppRoutesEnum.paramIdExecution,
        component: JobExecutionDetailRoutable,
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.job.execution"),
        },
      },
      {
        path: PreservationPlanningRoutesEnum.jobEdit,
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.job.edit"),
        },
        canDeactivate: [CanDeactivateGuard],
      },
    ],
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class PreservationJobRoutingModule {
}
