import {Injectable} from "@angular/core";
import {
  preservationMonitorActionNameSpace,
  PreservationMonitoringAction,
} from "@app/features/preservation/monitoring/stores/preservation-monitoring.action";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Navigate} from "@ngxs/router-plugin";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {
  PreservationMonitoringWebServiceState,
  PreservationMonitoringWebServiceStateModel,
} from "@preservation/monitoring/stores/web-service/preservation-monitoring-web-service.state";
import {PreservationPlanningResourceApiEnum} from "@shared/enums/api.enum";
import {PreservationPlanningMonitor} from "@shared/models/business/preservation-planning-monitor.model";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  BaseState,
  BaseStateModel,
  defaultBaseStateInitValue,
  NotificationService,
  SolidifyStateError,
  StoreUtil,
} from "solidify-frontend";

export interface PreservationMonitoringStateModel extends BaseStateModel {
  preservationPlanningMonitor: PreservationPlanningMonitor | undefined;
  preservation_monitoring_webService: PreservationMonitoringWebServiceStateModel | undefined;
}

@Injectable()
@State<PreservationMonitoringStateModel>({
  name: LocalStateEnum.preservation_monitoring,
  defaults: {
    ...defaultBaseStateInitValue(),
    preservationPlanningMonitor: undefined,
    preservation_monitoring_webService: undefined,
  },
  children: [
    PreservationMonitoringWebServiceState,
  ],
})
export class PreservationMonitoringState extends BaseState<PreservationMonitoringStateModel> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationMonitorActionNameSpace,
    }, PreservationMonitoringState);
  }

  protected get _urlResource(): string {
    return PreservationPlanningResourceApiEnum.monitor;
  }

  @Action(PreservationMonitoringAction.Get)
  get(ctx: StateContext<PreservationMonitoringStateModel>, action: PreservationMonitoringAction.Get): Observable<PreservationPlanningMonitor> {
    let reset = {};
    if (!action.keepCurrentContext) {
      reset = {
        preservationPlanningMonitor: undefined,
      };
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      ...reset,
    });
    return this.apiService.get<PreservationPlanningMonitor>(this._urlResource, null)
      .pipe(
        StoreUtil.cancelUncompleted(ctx, this.actions$, [PreservationMonitoringAction.Get, Navigate]),
        tap((preservationPlanningMonitor: PreservationPlanningMonitor) => {
          ctx.dispatch(new PreservationMonitoringAction.GetSuccess(action, preservationPlanningMonitor));
        }),
        catchError(error => {
          ctx.dispatch(new PreservationMonitoringAction.GetFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationMonitoringAction.GetSuccess)
  getAllSuccess(ctx: StateContext<PreservationMonitoringStateModel>, action: PreservationMonitoringAction.GetSuccess): void {
    ctx.patchState({
      preservationPlanningMonitor: action.preservationPlanningMonitor,
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(PreservationMonitoringAction.GetFail)
  getAllFail(ctx: StateContext<PreservationMonitoringStateModel>, action: PreservationMonitoringAction.GetFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }
}
