import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Navigate} from "@ngxs/router-plugin";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {
  PreservationMonitoringWebServiceAction,
  preservationMonitorWebServiceActionNameSpace,
} from "@preservation/monitoring/stores/web-service/preservation-monitoring-web-service.action";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  BaseState,
  BaseStateModel,
  defaultBaseStateInitValue,
  NotificationService,
  SolidifyStateError,
  StoreUtil,
} from "solidify-frontend";

export interface PreservationMonitoringWebServiceStateModel extends BaseStateModel {
  result: object | undefined;
}

@Injectable()
@State<PreservationMonitoringWebServiceStateModel>({
  name: LocalStateEnum.preservation_monitoring_webService,
  defaults: {
    ...defaultBaseStateInitValue(),
    result: undefined,
  },
  children: [],
})
export class PreservationMonitoringWebServiceState extends BaseState<PreservationMonitoringWebServiceStateModel> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              private readonly _httpClient: HttpClient) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationMonitorWebServiceActionNameSpace,
    }, PreservationMonitoringWebServiceState);
  }

  protected get _urlResource(): string {
    return undefined;
  }

  @Action(PreservationMonitoringWebServiceAction.Get)
  get(ctx: StateContext<PreservationMonitoringWebServiceStateModel>, action: PreservationMonitoringWebServiceAction.Get): Observable<object> {
    let reset = {};
    if (!action.keepCurrentContext) {
      reset = {
        result: undefined,
      };
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      ...reset,
    });

    return this.apiService.getByIdInPath(action.apiUrl).pipe(
      StoreUtil.cancelUncompleted(ctx, this.actions$, [PreservationMonitoringWebServiceAction.Get, Navigate]),
      tap((result: object) => {
        ctx.dispatch(new PreservationMonitoringWebServiceAction.GetSuccess(action, result));
      }),
      catchError(error => {
        ctx.dispatch(new PreservationMonitoringWebServiceAction.GetFail(action));
        throw new SolidifyStateError(this, error);
      }),
    );
  }

  @Action(PreservationMonitoringWebServiceAction.GetSuccess)
  getAllSuccess(ctx: StateContext<PreservationMonitoringWebServiceStateModel>, action: PreservationMonitoringWebServiceAction.GetSuccess): void {
    ctx.patchState({
      result: action.result,
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(PreservationMonitoringWebServiceAction.GetFail)
  getAllFail(ctx: StateContext<PreservationMonitoringWebServiceStateModel>, action: PreservationMonitoringWebServiceAction.GetFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }
}
