import {NgModule} from "@angular/core";
import {
  RouterModule,
  Routes,
} from "@angular/router";
import {PreservationHomeRoutable} from "@app/features/preservation/components/routables/preservation-home/preservation-home.routable";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@app/shared/enums/routes.enum";
import {ApplicationRoleGuardService} from "@shared/guards/application-role-guard.service";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

const routes: Routes = [
  {
    path: AppRoutesEnum.root,
    component: PreservationHomeRoutable,
    data: {},
  },
  {
    path: PreservationPlanningRoutesEnum.monitoring,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./monitoring/preservation-monitoring.module").then(m => m.PreservationMonitoringModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.monitoring.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: PreservationPlanningRoutesEnum.aipStatuses,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./aip-status/preservation-aip-status.module").then(m => m.PreservationAipStatusModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.aip-statuses.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: PreservationPlanningRoutesEnum.job,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./job/preservation-job.module").then(m => m.PreservationJobModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.job.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: PreservationPlanningRoutesEnum.aipStatuses,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./aip-status/preservation-aip-status.module").then(m => m.PreservationAipStatusModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.aip-statuses.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: PreservationPlanningRoutesEnum.job,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./job/preservation-job.module").then(m => m.PreservationJobModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.job.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: PreservationPlanningRoutesEnum.aip,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./aip/aip.module").then(m => m.AipModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.aip.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: PreservationPlanningRoutesEnum.sip,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./sip/sip.module").then(m => m.SipModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.sip.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: PreservationPlanningRoutesEnum.dip,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./dip/dip.module").then(m => m.DipModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.dip.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: PreservationPlanningRoutesEnum.aipDownloaded,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./aip-downloaded/aip-downloaded.module").then(m => m.AipDownloadedModule),
    data: {
      breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.aip-downloaded.root"),
    },
    canActivate: [ApplicationRoleGuardService],
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class PreservationRoutingModule {

}
