import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Input,
  OnInit,
  Output,
} from "@angular/core";
import {FormBuilder} from "@angular/forms";
import {SharedAbstractFormPresentational} from "@app/shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {BaseFormDefinition} from "@app/shared/models/base-form-definition.model";
import {
  License,
  Sip,
} from "@models";
import {SipExtended} from "@preservation/sip/models/sip-extended.model";
import {BaseResourceApiEnum} from "@shared/enums/api.enum";
import {AccessLevelEnumHelper} from "@shared/enums/business/access-level-enum.helper";
import {
  PackageStatusEnum,
  PackageStatusEnumHelper,
} from "@shared/enums/business/package-status.enum";
import {
  AppRoutesEnum,
  DepositRoutesEnum,
  PreservationPlanningRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {Storage} from "@shared/models/storage.model";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {sharedLicenseActionNameSpace} from "@shared/stores/license/shared-license.action";
import {SharedLicenseState} from "@shared/stores/license/shared-license.state";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  KeyValue,
  ObservableUtil,
  PropertyName,
  ResourceNameSpace,
  urlSeparator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-sip-form",
  templateUrl: "./sip-form.presentational.html",
  styleUrls: ["./sip-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SipFormPresentational extends SharedAbstractFormPresentational<Sip> implements OnInit {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  accessEnumValues: KeyValue[] = AccessLevelEnumHelper.getListKeyValue();
  packageStatusEnumValues: KeyValue[] = PackageStatusEnumHelper.getListKeyValue();

  sharedLicenseActionNameSpace: ResourceNameSpace = sharedLicenseActionNameSpace;
  sharedLicenseState: typeof SharedLicenseState = SharedLicenseState;

  @Input()
  isReady: boolean = false;

  listStoragions: Storage[] = BaseResourceApiEnum.archivalStorageList;

  get packageStatusEnum(): typeof PackageStatusEnum {
    return PackageStatusEnum;
  }

  private readonly _orgUnitValueBS: BehaviorSubject<string | undefined> = new BehaviorSubject<string | undefined>(undefined);
  @Output("orgUnitChange")
  readonly orgUnitValueObs: Observable<string | undefined> = ObservableUtil.asObservable(this._orgUnitValueBS);

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              private readonly _fb: FormBuilder,
              public readonly breakpointService: BreakpointService) {
    super(_changeDetectorRef, _elementRef);
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  protected initNewForm(): void {
  }

  protected bindFormTo(sip: SipExtended): void {
    this.form = this._fb.group({
      [this.formDefinition.organizationalUnitName]: [sip.organizationalUnit.name, []],
      [this.formDefinition.resId]: [sip.resId, []],
      [this.formDefinition.name]: [sip.info.name, []],
      [this.formDefinition.year]: ["", []],
      [this.formDefinition.status]: [sip.info.status, []],
      [this.formDefinition.access]: [sip.info.access, []],
      [this.formDefinition.licenseId]: [sip.info.licenseId, []],
      [this.formDefinition.complianceLevel]: [sip.info.complianceLevel, []],
    });
    this.isValidWhenDisable = this.form.valid;
  }

  protected treatmentBeforeSubmit(sip: SipExtended): SipExtended {
    return sip;
  }

  getAipPath(aipId: string, storagionIndex: number): string {
    return "#" + urlSeparator + AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.aip + urlSeparator + storagionIndex + urlSeparator + PreservationPlanningRoutesEnum.aipDetail + urlSeparator + aipId;
  }

  getDepositPath(orgUnitId: string, depositId: string): string {
    return "#" + urlSeparator + RoutesEnum.deposit + urlSeparator + orgUnitId + urlSeparator + DepositRoutesEnum.detail + urlSeparator + depositId;
  }

  licenceCallback: (value: License) => string = (value: License) => value.title;
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() organizationalUnitName: string;
  @PropertyName() resId: string;
  @PropertyName() name: string;
  @PropertyName() year: string;
  @PropertyName() status: string;
  @PropertyName() access: string;
  @PropertyName() licenseId: string;
  @PropertyName() complianceLevel: string;
  @PropertyName() deposit: string;
  @PropertyName() aip: string;
}
