import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {SipStateModel} from "@app/features/preservation/sip/stores/sip.state";
import {SharedAbstractListRoutable} from "@app/shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {appAuthorizedOrganizationalUnitNameSpace} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.action";
import {AppAuthorizedOrganizationalUnitState} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.state";
import {
  PreservationJob,
  Sip,
} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {
  PreservationSipAction,
  preservationSipActionNameSpace,
} from "@preservation/sip/stores/sip.action";
import {PackageStatusEnumHelper} from "@shared/enums/business/package-status.enum";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {RouterExtService} from "@shared/services/router-ext.service";
import {
  OrderEnum,
  Override,
  ResourceNameSpace,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-sip-list-routable",
  templateUrl: "../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SipListRoutable extends SharedAbstractListRoutable<Sip, SipStateModel> implements OnInit {
  readonly KEY_CREATE_BUTTON: string = undefined;
  readonly KEY_REFRESH_BUTTON: string = MARK_AS_TRANSLATABLE("preservation.sip.refresh");
  readonly KEY_BACK_BUTTON: string = MARK_AS_TRANSLATABLE("preservation.sip.back");
  readonly KEY_PARAM_NAME: keyof Sip & string = undefined;

  appAuthorizedOrganizationalUnitNameSpace: ResourceNameSpace = appAuthorizedOrganizationalUnitNameSpace;
  appAuthorizedOrganizationalUnitState: typeof AppAuthorizedOrganizationalUnitState = AppAuthorizedOrganizationalUnitState;

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, LocalStateEnum.preservation_sip, preservationSipActionNameSpace, {
      canCreate: false,
    }, LocalStateEnum.preservation);
  }

  conditionDisplayEditButton(model: Sip | undefined): boolean {
    return false;
  }

  conditionDisplayDeleteButton(model: Sip | undefined): boolean {
    return false;
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "info.name" as any,
        header: MARK_AS_TRANSLATABLE("preservation.sip.table.header.title"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "organizationalUnit.name" as any,
        header: MARK_AS_TRANSLATABLE("preservation.sip.table.header.organizationalUnit"),
        type: FieldTypeEnum.searchableSingleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        resourceNameSpace: this.appAuthorizedOrganizationalUnitNameSpace,
        resourceState: this.appAuthorizedOrganizationalUnitState as any,
        filterableField: "info.organizationalUnitId" as any,
        sortableField: "info.organizationalUnitId" as any,
      },
      {
        field: "creation.when" as any,
        header: MARK_AS_TRANSLATABLE("preservation.sip.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: MARK_AS_TRANSLATABLE("preservation.sip.table.header.lastUpdate.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "info.status" as any,
        header: MARK_AS_TRANSLATABLE("preservation.sip.table.header.status"),
        type: FieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: PackageStatusEnumHelper.getListKeyValue(),
      },
    ];
  }

  @Override()
  protected defineActions(): DataTableActions<PreservationJob>[] {
    return [
      {
        logo: IconNameEnum.download,
        callback: (model: Sip) => this.download(model),
        placeholder: MARK_AS_TRANSLATABLE("crud.list.action.download"),
        displayOnCondition: (model: Sip) => true,
      },
    ];
  }

  private download(sip: Sip): void {
    this._store.dispatch(new PreservationSipAction.Download(sip.resId));
  }
}
