import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {
  SipState,
  SipStateModel,
} from "@app/features/preservation/sip/stores/sip.state";
import {DepositFormPresentational} from "@deposit/components/presentationals/deposit-form/deposit-form.presentational";
import {Sip} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {preservationSipActionNameSpace} from "@preservation/sip/stores/sip.action";
import {SharedAbstractDetailEditRoutable} from "@shared/components/routables/shared-abstract-detail-edit/shared-abstract-detail-edit.routable";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {Observable} from "rxjs";

@Component({
  selector: "dlcm-sip-metadata-routable",
  templateUrl: "./sip-metadata.routable.html",
  styleUrls: ["./sip-metadata.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})

export class SipMetadataRoutable extends SharedAbstractDetailEditRoutable<Sip, SipStateModel> implements OnInit, OnDestroy {
  @Select(SipState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(SipState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;

  @ViewChild("formPresentational")
  readonly formPresentational: DepositFormPresentational;

  readonly KEY_PARAM_NAME: keyof Sip & string = undefined;

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _dialog: MatDialog) {
    super(_store, _route, _actions$, _changeDetector, _dialog, LocalStateEnum.preservation_sip, preservationSipActionNameSpace, LocalStateEnum.preservation);
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
  }

  protected getSubResourceWithParentId(id: string): void {
  }
}
