import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@app/shared/enums/routes.enum";
import {DlcmRoutes} from "@app/shared/models/dlcm-route.model";
import {SipDetailEditRoutable} from "@preservation/sip/components/routables/sip-detail-edit/sip-detail-edit.routable";
import {SipFileRoutable} from "@preservation/sip/components/routables/sip-file/sip-file.routable";
import {SipListRoutable} from "@preservation/sip/components/routables/sip-list/sip-list.routable";
import {SipMetadataRoutable} from "@preservation/sip/components/routables/sip-metadata/sip-metadata.routable";
import {SipState} from "@preservation/sip/stores/sip.state";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";

const routes: DlcmRoutes = [
  {
    path: AppRoutesEnum.root,
    component: SipListRoutable,
    data: {},
  },
  {
    path: PreservationPlanningRoutesEnum.sipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    redirectTo: PreservationPlanningRoutesEnum.sipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId + AppRoutesEnum.separator + PreservationPlanningRoutesEnum.sipMetadata,
    pathMatch: "full",
  },
  {
    path: PreservationPlanningRoutesEnum.sipDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: SipDetailEditRoutable,
    data: {
      breadcrumbMemoizedSelector: SipState.currentTitle,
    },
    children: [
      {
        path: PreservationPlanningRoutesEnum.sipMetadata,
        component: SipMetadataRoutable,
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.sip.metadata"),
          noBreadcrumbLink: true,
        },
      },
      {
        path: PreservationPlanningRoutesEnum.sipFiles,
        component: SipFileRoutable,
        data: {
          breadcrumb: MARK_AS_TRANSLATABLE("breadcrumb.preservation.sip.file"),
          noBreadcrumbLink: true,
        },
      },
    ],
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class SipRoutingModule {
}
