import {
  Inject,
  Injectable,
} from "@angular/core";
import {WINDOW} from "@app/app.module";
import {
  PreservationSipDataFileAction,
  preservationSipDataFileActionNameSpace,
} from "@app/features/preservation/sip/stores/data-file/sip-data-file.action";
import {ApiActionEnum} from "@app/shared/enums/api-action.enum";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {IngestResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {SipDataFile} from "@preservation/sip/models/sip-data-file.model";
import {
  SipDataFileStatusHistoryState,
  SipDataFileStatusHistoryStateModel,
} from "@preservation/sip/stores/data-file/status-history/sip-data-file-status-history.state";
import {DownloadService} from "@shared/services/download.service";
import {defaultStatusHistoryInitValue} from "@shared/stores/status-history/status-history.state";
import {Observable} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {
  ApiService,
  CompositionState,
  CompositionStateModel,
  defaultCompositionStateInitValue,
  NotificationService,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

export const defaultSipDataFileValue: () => SipDataFileStateModel = () =>
  ({
    ...defaultCompositionStateInitValue(),
    preservation_sip_dataFile_statusHistory: defaultStatusHistoryInitValue(),
  });

export interface SipDataFileStateModel extends CompositionStateModel<SipDataFile> {
  preservation_sip_dataFile_statusHistory: SipDataFileStatusHistoryStateModel;
}

@Injectable()
@State<SipDataFileStateModel>({
  name: LocalStateEnum.preservation_sip_dataFile,
  defaults: {
    ...defaultSipDataFileValue(),
  },
  children: [
    SipDataFileStatusHistoryState,
  ],
})
export class SipDataFileState extends CompositionState<SipDataFileStateModel, SipDataFile> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              @Inject(WINDOW) private _window: Window,
              private downloadService: DownloadService) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: preservationSipDataFileActionNameSpace,
      resourceName: ApiResourceNameEnum.DATAFILE,
    });
  }

  protected get _urlResource(): string {
    return IngestResourceApiEnum.sip;
  }

  @Action(PreservationSipDataFileAction.Refresh)
  refresh(ctx: StateContext<SipDataFileStateModel>, action: PreservationSipDataFileAction.Refresh): void {
    ctx.dispatch(new PreservationSipDataFileAction.GetAll(action.parentId, undefined, true));
  }

  @Action(PreservationSipDataFileAction.Download)
  download(ctx: StateContext<SipDataFileStateModel>, action: PreservationSipDataFileAction.Download): void {
    const url = `${this._urlResource}/${action.parentId}/${this._resourceName}/${action.dataFile.resId}/${ApiActionEnum.DL}`;
    this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.sip.file.download"), true);
    this.downloadService.download(url, action.dataFile.fileName, action.dataFile.fileSize).subscribe(() => {
      this.notificationService.showSuccess(MARK_AS_TRANSLATABLE("notification.sip.file.downloadWithSuccess"));
    });
  }

  @Action(PreservationSipDataFileAction.Resume)
  resume(ctx: StateContext<SipDataFileStateModel>, action: PreservationSipDataFileAction.Resume): Observable<void> {
    return this.apiService.post<void>(`${this._urlResource}/${action.parentId}/${this._resourceName}/${action.dataFile.resId}/${ApiActionEnum.RESUME}`)
      .pipe(
        tap(() => {
          this.notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.sip.file.resumed"), true);
          ctx.dispatch(new PreservationSipDataFileAction.GetAll(action.parentId, undefined, true));
        }),
      );
  }
}
