import {
  Inject,
  OnInit,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {SharedAbstractContainer} from "@app/shared/components/containers/shared-abstract/shared-abstract.container";
import {DeleteDialog} from "@app/shared/models/delete-dialog.model";
import {Store} from "@ngxs/store";
import {
  ResourceActionHelper,
  ResourceNameSpace,
  ResourceStateModel,
} from "solidify-frontend";
import {LocalStateEnum} from "../../../enums/local-state.enum";

export abstract class SharedAbstractDeleteDialog<TResource, UStateResource extends ResourceStateModel<TResource>> extends SharedAbstractContainer implements OnInit {
  paramMessage: { name: string } = {name: ""};

  public abstract readonly KEY_TITLE: string;
  public abstract readonly KEY_MESSAGE: string;
  public abstract readonly KEY_CONFIRM_BUTTON: string;
  public abstract readonly KEY_CANCEL_BUTTON: string;

  protected constructor(protected store: Store,
                        protected dialogRef: MatDialogRef<SharedAbstractDeleteDialog<TResource, UStateResource>>,
                        @Inject(MAT_DIALOG_DATA) public data: DeleteDialog,
                        protected state: LocalStateEnum,
                        private resourceActionNameSpace: ResourceNameSpace) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.paramMessage.name = this.data.name;
  }

  confirm(): void {
    this.store.dispatch(ResourceActionHelper.delete(this.resourceActionNameSpace, this.data.resId));
    this.dialogRef.close();
  }
}
