import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialog,
  MatDialogRef,
} from "@angular/material/dialog";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {Tab} from "@shared/components/containers/shared-tabs/shared-tabs.container";
import {
  SharedDetailDialog,
  SharedDetailDialogData,
} from "@shared/components/dialogs/shared-detail-dialog/shared-detail.dialog";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {PreservationSpaceRoutesEnum} from "@shared/enums/routes.enum";
import {FileVisualizerHelper} from "@shared/filevisualizer/helpers/file-visualizer.helper";
import {FileInput} from "@shared/filevisualizer/models/file-info.model";
import {
  AbstractFileVisualizer,
  FILE_VISUALIZERS,
} from "@shared/filevisualizer/services/abstract-file-visualizer.service";
import {DataFile} from "@shared/models/business/data-file.model";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {
  BaseResource,
  MARK_AS_TRANSLATABLE,
  NotificationService,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-file-detail-dialog",
  templateUrl: "./shared-file-detail.dialog.html",
  styleUrls: ["../shared-detail-dialog/shared-detail.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedFileDetailDialog<TResource extends BaseResource> extends SharedDetailDialog<TResource> implements OnInit {
  previewAvailable: boolean;
  visualizationErrorMessage: string;

  readonly TAB_ID_INFORMATIONS: string = "INFORMATIONS";
  readonly TAB_ID_HISTORY: string = "HISTORY";
  readonly TAB_ID_PREVIEW: string = "PREVIEW";

  tabs: Tab[] = [
    {
      id: this.TAB_ID_INFORMATIONS,
      icon: IconNameEnum.information,
      titleToTranslate: MARK_AS_TRANSLATABLE("crud.list.action.info"),
    },
    {
      id: this.TAB_ID_HISTORY,
      icon: IconNameEnum.history,
      titleToTranslate: MARK_AS_TRANSLATABLE("crud.list.action.history"),
      suffixUrl: PreservationSpaceRoutesEnum.contributor,
    },
    {
      id: this.TAB_ID_PREVIEW,
      icon: IconNameEnum.preview,
      titleToTranslate: MARK_AS_TRANSLATABLE("deposit.file.button.showPreview"),
      suffixUrl: PreservationSpaceRoutesEnum.notificationInbox,
      conditionDisplay: () => this.previewAvailable,
    },
  ];

  constructor(protected readonly _store: Store,
              protected readonly _dialog: MatDialog,
              protected readonly _actions$: Actions,
              protected readonly _breakpointService: BreakpointService,
              protected readonly _notificationService: NotificationService,
              protected readonly dialogRef: MatDialogRef<SharedDetailDialog<TResource>>,
              @Inject(FILE_VISUALIZERS) protected readonly _fileVisualizers: AbstractFileVisualizer[],
              @Inject(MAT_DIALOG_DATA) public data: SharedFileDetailDialogData<TResource>) {
    super(_store, _dialog, _actions$, _breakpointService, _notificationService, dialogRef, data);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.previewAvailable = FileVisualizerHelper.canHandle(this._fileVisualizers, {
      dataFile: this.data.dataFile,
      fileExtension: FileVisualizerHelper.getFileExtension(this.data.dataFile.fileName),
    });
  }

  getFileInput(): FileInput {
    return {
      dataFile: this.data.dataFile,
    };
  }
}

export interface SharedFileDetailDialogData<TResource extends BaseResource> extends SharedDetailDialogData<TResource> {
  dataFile: DataFile;
}
