import {
  ChangeDetectionStrategy,
  Component,
  Input,
} from "@angular/core";
import {ChangeInfo} from "@app/generated-api/model/change-info.partial.model";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {
  BaseResourceType,
  DateUtil,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-button-info-creation",
  templateUrl: "./shared-button-info-creation.presentational.html",
  styleUrls: ["./shared-button-info-creation.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedButtonInfoCreationPresentational extends SharedAbstractPresentational {
  creationDate: string | undefined;
  creationWho: string | undefined;
  updatedDate: string | undefined;
  updatedWho: string | undefined;
  messageToTranslate: string | undefined;

  @Input()
  set resource(value: BaseResourceType) {
    this._init();
    if (isNullOrUndefined(value)) {
      return;
    }
    const creationInfos: ChangeInfo = value["creation"];
    if (isNotNullNorUndefined(creationInfos)) {
      if (isNotNullNorUndefined(creationInfos.when)) {
        this.creationDate = DateUtil.convertDateToDateTimeString(new Date(creationInfos.when));
      }
      if (isNotNullNorUndefined(creationInfos.who)) {
        this.creationWho = creationInfos.who;
      }
    }
    const lastUpdatesInfos: ChangeInfo = value["lastUpdate"];
    if (isNotNullNorUndefined(lastUpdatesInfos)) {
      if (isNotNullNorUndefined(lastUpdatesInfos.when)) {
        this.updatedDate = DateUtil.convertDateToDateTimeString(new Date(lastUpdatesInfos.when));
      }
      if (isNotNullNorUndefined(lastUpdatesInfos.who)) {
        this.updatedWho = lastUpdatesInfos.who;
      }
    }

    if (isNotNullNorUndefined(this.creationDate) && isNotNullNorUndefined(this.creationWho)) {
      if (isNotNullNorUndefined(this.updatedDate) && isNotNullNorUndefined(this.updatedWho)) {
        this.messageToTranslate = MARK_AS_TRANSLATABLE("app.buttonInfoCreation.creationAndUpdate");
      } else {
        this.messageToTranslate = MARK_AS_TRANSLATABLE("app.buttonInfoCreation.creationOnly");
      }
    }
  }

  constructor() {
    super();
  }

  private _init(): void {
    this.messageToTranslate = undefined;
    this.creationDate = undefined;
    this.creationWho = undefined;
    this.updatedDate = undefined;
    this.updatedWho = undefined;
  }

  getParam(): object {
    return {
      creationDate: this.creationDate,
      creationWho: this.creationWho,
      updatedDate: this.updatedDate,
      updatedWho: this.updatedWho,
    };
  }
}
