import {
  ChangeDetectionStrategy,
  Component,
  Input,
  OnInit,
} from "@angular/core";
import {
  ControlValueAccessor,
  FormControl,
  NG_VALUE_ACCESSOR,
} from "@angular/forms";
import {SharedAbstractPresentational} from "@shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {
  ComplianceLevelEnum,
  ComplianceLevelNumberEnum,
} from "@shared/enums/business/compliance-level.enum";
import {
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-compliance-level-rating",
  templateUrl: "./shared-compliance-level.presentational.html",
  styleUrls: ["./shared-compliance-level.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: SharedComplianceLevelPresentational,
    },
  ],
})

export class SharedComplianceLevelPresentational extends SharedAbstractPresentational implements OnInit, ControlValueAccessor {
  @Input()
  formControl: FormControl;

  @Input()
  value: ComplianceLevelEnum;

  @Input()
  withLabel: boolean = true;

  @Input()
  center: boolean = false;

  starsNumber: number = 3;
  label: string | undefined = MARK_AS_TRANSLATABLE("aip.form.complianceLevel");
  tooltipToTranslate: string | undefined;

  ngOnInit(): void {
    this.extractComplianceLevelNumber();
  }

  extractComplianceLevelNumber(): number {
    let level: ComplianceLevelEnum = this.value;
    if (!isNullOrUndefined(this.formControl)) {
      level = this.formControl.value;
    }
    let complianceLevel: number = 0;
    switch (level) {
      case ComplianceLevelEnum.NOT_ASSESSED: {
        complianceLevel = ComplianceLevelNumberEnum.NOT_ASSESSED;
        this.tooltipToTranslate = ComplianceLevelEnum.NOT_ASSESSED;
        break;
      }
      case ComplianceLevelEnum.NO_COMPLIANCE: {
        complianceLevel = ComplianceLevelNumberEnum.NO_COMPLIANCE;
        this.tooltipToTranslate = ComplianceLevelEnum.NO_COMPLIANCE;
        break;
      }
      case ComplianceLevelEnum.WEAK_COMPLIANCE: {
        complianceLevel = ComplianceLevelNumberEnum.WEAK_COMPLIANCE;
        this.tooltipToTranslate = ComplianceLevelEnum.WEAK_COMPLIANCE;
        break;
      }
      case ComplianceLevelEnum.AVERAGE_COMPLIANCE: {
        complianceLevel = ComplianceLevelNumberEnum.AVERAGE_COMPLIANCE;
        this.tooltipToTranslate = ComplianceLevelEnum.AVERAGE_COMPLIANCE;
        break;
      }
      case ComplianceLevelEnum.FULL_COMPLIANCE: {
        complianceLevel = ComplianceLevelNumberEnum.FULL_COMPLIANCE;
        this.tooltipToTranslate = ComplianceLevelEnum.FULL_COMPLIANCE;
        break;
      }
    }
    return complianceLevel;
  }

  writeValue(value: ComplianceLevelEnum): void {
  }

  registerOnChange(fn: any): void {
  }

  registerOnTouched(fn: any): void {
  }

  setDisabledState?(isDisabled: boolean): void {
  }
}
