import {
  KeyValue,
  MARK_AS_TRANSLATABLE,
} from "solidify-frontend";

export enum DataFileStatusEnum {
  IN_ERROR = "IN_ERROR",
  READY = "READY",
  RECEIVED = "RECEIVED",
  CHANGE_RELATIVE_LOCATION = "CHANGE_RELATIVE_LOCATION",
  TO_PROCESS = "TO_PROCESS",
  PROCESSED = "PROCESSED",
  FILE_FORMAT_IDENTIFIED = "FILE_FORMAT_IDENTIFIED",
  FILE_FORMAT_UNKNOWN = "FILE_FORMAT_UNKNOWN",
  VIRUS_CHECKED = "VIRUS_CHECKED",
  CLEANING = "CLEANING",
  CLEANED = "CLEANED",
  IGNORED_FILE = "IGNORED_FILE",
  EXCLUDED_FILE = "EXCLUDED_FILE"
}

export class DataFileStatusEnumHelper {
  static getListKeyValue(): KeyValue[] {
    return [
      {
        key: DataFileStatusEnum.IN_ERROR,
        value: MARK_AS_TRANSLATABLE("IN_ERROR"),
      },
      {
        key: DataFileStatusEnum.READY,
        value: MARK_AS_TRANSLATABLE("READY"),
      },
      {
        key: DataFileStatusEnum.RECEIVED,
        value: MARK_AS_TRANSLATABLE("RECEIVED"),
      },
      {
        key: DataFileStatusEnum.CHANGE_RELATIVE_LOCATION,
        value: MARK_AS_TRANSLATABLE("CHANGE_RELATIVE_LOCATION"),
      },
      {
        key: DataFileStatusEnum.TO_PROCESS,
        value: MARK_AS_TRANSLATABLE("TO_PROCESS"),
      },
      {
        key: DataFileStatusEnum.PROCESSED,
        value: MARK_AS_TRANSLATABLE("PROCESSED"),
      },
      {
        key: DataFileStatusEnum.FILE_FORMAT_IDENTIFIED,
        value: MARK_AS_TRANSLATABLE("FILE_FORMAT_IDENTIFIED"),
      },
      {
        key: DataFileStatusEnum.FILE_FORMAT_UNKNOWN,
        value: MARK_AS_TRANSLATABLE("FILE_FORMAT_UNKNOWN"),
      },
      {
        key: DataFileStatusEnum.VIRUS_CHECKED,
        value: MARK_AS_TRANSLATABLE("VIRUS_CHECKED"),
      },
      {
        key: DataFileStatusEnum.CLEANING,
        value: MARK_AS_TRANSLATABLE("CLEANING"),
      },
      {
        key: DataFileStatusEnum.EXCLUDED_FILE,
        value: MARK_AS_TRANSLATABLE("EXCLUDED_FILE"),
      },
      {
        key: DataFileStatusEnum.IGNORED_FILE,
        value: MARK_AS_TRANSLATABLE("IGNORED_FILE"),
      },
      {
        key: DataFileStatusEnum.CLEANED,
        value: MARK_AS_TRANSLATABLE("CLEANED"),
      },
    ];
  }

  static getExplanation(status: DataFileStatusEnum): string {
    switch (status) {
      case DataFileStatusEnum.READY:
        return MARK_AS_TRANSLATABLE("dataFileQuickStatus.ready");
      case DataFileStatusEnum.IN_ERROR:
        return MARK_AS_TRANSLATABLE("dataFileQuickStatus.inError");
      case DataFileStatusEnum.EXCLUDED_FILE:
        return MARK_AS_TRANSLATABLE("dataFileQuickStatus.excludedFile");
      case DataFileStatusEnum.IGNORED_FILE:
        return MARK_AS_TRANSLATABLE("dataFileQuickStatus.ignoredFile");
      case DataFileStatusEnum.CLEANED:
        return MARK_AS_TRANSLATABLE("dataFileQuickStatus.cleaned");
      default:
        return MARK_AS_TRANSLATABLE("dataFileQuickStatus.pending");
    }
  }
}
