import {DataFileStatusEnum} from "@shared/enums/business/data-file-status.enum";
import {FileInput} from "@shared/filevisualizer/models/file-info.model";
import {AbstractFileVisualizer} from "@shared/filevisualizer/services/abstract-file-visualizer.service";
import {isTruthyObject} from "solidify-frontend";

export class FileVisualizerHelper {
  static canHandle(fileVisualizers: AbstractFileVisualizer[], fileInfo: FileInput): boolean {
    if (!isTruthyObject(fileInfo)) {
      return false;
    }
    if (fileInfo.dataFile.status !== DataFileStatusEnum.READY) {
      return false;
    }
    return fileVisualizers.some(value => value.canHandleWithSize(fileInfo) && value.canHandle(fileInfo));
  }

  static getFileExtension(fileName: string): string {
    return fileName?.split(".").pop();
  }
}
