import {Injectable} from "@angular/core";
import {environment} from "@environments/environment";
import {FileInput} from "@shared/filevisualizer/models/file-info.model";
import {AbstractFileVisualizer} from "@shared/filevisualizer/services/abstract-file-visualizer.service";
import {isTruthyObject} from "solidify-frontend";

@Injectable()
export class ImageFileVisualizerService implements AbstractFileVisualizer {

  type: string = "imagePlugin";

  img: HTMLImageElement;

  canHandle(fileInfo: FileInput): boolean {
    if (!isTruthyObject(fileInfo)) {
      return false;
    } else {
      return this.canHandleByExtension(fileInfo) || this.canHandleByMimeTypeContentType(fileInfo) || this.canHandleByPuid(fileInfo);
    }
  }

  canHandleByExtension(fileInfo: FileInput): boolean {
    if (!isTruthyObject(fileInfo)) {
      return false;
    } else {
      return environment.visualizationImageExtensions.some(value => value === fileInfo.fileExtension);
    }
  }

  isVisualizationOnGoing(fileInfo: FileInput, domElement: Element): boolean {
    return isTruthyObject(this.img);
  }

  canHandleWithSize(fileInfo: FileInput): boolean {
    if (!isTruthyObject(fileInfo)) {
      return false;
    } else {
      return fileInfo.dataFile.fileSize <= environment.visualizationMaxSizeFile;
    }
  }

  canHandleByMimeTypeContentType(fileInfo: FileInput): boolean {
    if (!isTruthyObject(fileInfo)) {
      return false;
    } else {
      return environment.visualizationImageContentType.some(value => value === fileInfo.dataFile.fileFormat.contentType) || environment.visualizationImageMimeType.some(value => value === fileInfo.dataFile.fileFormat.contentType);
    }
  }

  canHandleByPuid(fileInfo: FileInput): boolean {
    if (!isTruthyObject(fileInfo)) {
      return false;
    } else {
      return environment.visualizationImagePronomId.some(value => value === fileInfo.dataFile.fileFormat.puid);
    }
  }

  closeVisualizer(fileInfo: FileInput, domElement: Element): void {
    domElement.remove();
  }

  openVisualizer(fileInfo: FileInput, domElement: Element): void {
    this.img = document.createElement("img");
    const blob = new Blob([fileInfo.blob], {type: fileInfo.blob.type});
    const url = URL.createObjectURL(blob);
    this.img.src = url;
    this.img.style.objectFit = "contain";
    this.img.height = 404;
    this.img.width = 500;
    domElement.appendChild(this.img);
    domElement.classList.add("zoom-in");
  }

  doAction(fileInfo: FileInput, domElement: Element): void {
    if (domElement.classList.contains("zoom-in")) {
      domElement.removeChild(this.img);
      this.img.height = 1000;
      this.img.width = 1000;
      domElement.classList.remove("zoom-in");
      domElement.classList.add("zoom-out");
      domElement.appendChild(this.img);
      return;
    }
    if (domElement.classList.contains("zoom-out")) {
      domElement.removeChild(this.img);
      this.img.height = 500;
      this.img.width = 500;
      domElement.classList.remove("zoom-out");
      domElement.classList.add("zoom-in");
      domElement.appendChild(this.img);
      return;
    }

  }

}
