import {Injectable} from "@angular/core";
import {
  ActivatedRouteSnapshot,
  CanActivate,
  Router,
} from "@angular/router";
import {ApplicationRoleEnum} from "@app/shared/enums/application-role.enum";
import {DlcmData} from "@app/shared/models/dlcm-route.model";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {PermissionUtil} from "@app/shared/utils/permission.util";
import {AppState} from "@app/stores/app.state";
import {Store} from "@ngxs/store";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {Observable} from "rxjs";
import {
  filter,
  map,
  take,
} from "rxjs/operators";
import {
  isNullOrUndefined,
  MemoizedUtil,
} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class ApplicationRoleGuardService implements CanActivate {
  constructor(public router: Router, public store: Store) {
  }

  canActivate(route: ActivatedRouteSnapshot): Observable<boolean> {
    const data = route.data as DlcmData;
    const permission = isNullOrUndefined(data) ? undefined : data.permission;
    return MemoizedUtil.select(this.store, AppState, state => state.isApplicationInitialized).pipe(
      filter(isApplicationInitialized => isApplicationInitialized),
      take(1),
      map(() => {
        const currentUserRole = this.store.selectSnapshot<ApplicationRoleEnum[]>((state: LocalStateModel) => state.application.userRoles);
        const isLoggedIn = this.store.selectSnapshot<boolean>((state: LocalStateModel) => state.application.isLoggedIn);
        const isAuthorized = PermissionUtil.isUserHavePermission(isLoggedIn, permission, currentUserRole);
        if (isAuthorized) {
          return true;
        }
        this.router.navigate([RoutesEnum.homePage]);
        return false;
      }),
    );
  }
}
