import {
  Observable,
  timer,
} from "rxjs";
import {
  filter,
  map,
  takeWhile,
  tap,
} from "rxjs/operators";
import {isUndefined} from "solidify-frontend";

export class PollingHelper<Type> {
  static startPollingObs<Type>(config: PollingConfigInfo<Type>): Observable<Type> {
    const intervalInMillisecond = config.intervalRefreshInSecond * 1000;
    return timer(intervalInMillisecond, intervalInMillisecond).pipe(
      takeWhile(() => isUndefined(config.continueUntil) || config.continueUntil()),
      tap(() => isUndefined(config.doBeforeFilter) || config.doBeforeFilter()),
      filter(() => isUndefined(config.filter) || config.filter()),
      map(() => config.actionToDo()),
    );
  }
}

export interface PollingConfigInfo<Type> {
  intervalRefreshInSecond: number;
  doBeforeFilter?: () => void | undefined;
  continueUntil?: () => boolean | undefined;
  filter?: () => boolean | undefined;
  actionToDo: () => Type;
}
