import {Params} from "@angular/router";
import {environment} from "@environments/environment";
import {
  isEmptyString,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MappingObject,
  MappingObjectUtil,
  StringUtil,
} from "solidify-frontend";

export class UrlQueryParamHelper {
  private static readonly _QUERY_PARAM_OAUTH_CODE: string = "code";
  private static readonly _QUERY_PARAM_OAUTH_STATE: string = "state";
  private static readonly _QUERY_PARAM_ORCID: string = environment.orcidQueryParam;
  private static readonly _QUERY_PARAM_START_CHAR: string = "?";
  private static readonly _QUERY_PARAM_SEPARATOR_CHAR: string = "&";
  private static readonly _QUERY_PARAM_VALUE_AFFECTION_CHAR: string = "=";

  static getQueryParamMappingObject(rawQueryParam: string): MappingObject<string | undefined> {
    const mappingObject = new Map<string, string>();
    if (isNullOrUndefined(rawQueryParam)) {
      return mappingObject;
    }
    const listQueryParam = this._getListQueryParameters(rawQueryParam);
    if (listQueryParam.length === 0) {
      return mappingObject;
    }
    listQueryParam.forEach(queryParam => {
      const queryParamSplitted = queryParam.split(this._QUERY_PARAM_VALUE_AFFECTION_CHAR);
      mappingObject.set(queryParamSplitted[0], queryParamSplitted.length > 1 ? queryParamSplitted[1] : undefined);
    });

    return mappingObject;
  }

  static getQueryParamOAuth2(): string {
    // NEED TO PRESERVE QUERY PARAM WHEN ROOTING USING HASH STRATEGY IS INITIALIZED
    // NECESSARY FOR OAUTH 2 PROCESS (SHOULD PRESERVED CODE AND STATE IN QUERY PARAM)
    const queryParamString = this._getQueryParameters();
    if (isNullOrUndefined(queryParamString)) {
      return StringUtil.stringEmpty;
    }
    let listQueryParam = this._getListQueryParameters(queryParamString);
    listQueryParam = this._preserveOnlyQueryParametersOauth(listQueryParam);

    if (listQueryParam.length === 0) {
      return StringUtil.stringEmpty;
    }
    return this._getQueryParametersString(listQueryParam);
  }

  static getQueryParameterWithoutUnwantedQueryParam(): Params | undefined {
    const queryParamString = this._getQueryParameters();
    const params = {};
    if (isNullOrUndefined(queryParamString)) {
      return params;
    }
    let listQueryParam = this._getListQueryParameters(queryParamString);
    listQueryParam = this._removeOnlyUnwantedQueryParameters(listQueryParam);

    if (listQueryParam.length === 0) {
      return params;
    }
    listQueryParam.forEach(queryParam => {
      const queryParamSpitted = queryParam.split(this._QUERY_PARAM_VALUE_AFFECTION_CHAR);
      MappingObjectUtil.set(params, queryParamSpitted[0], queryParamSpitted.length > 1 ? queryParamSpitted[1] : StringUtil.stringEmpty);
    });
    return params;
  }

  static contains(desiredQueryParam: string): boolean {
    if (isNullOrUndefined(desiredQueryParam) || isEmptyString(desiredQueryParam)) {
      return false;
    }
    const queryParamString = this._getQueryParameters();
    if (isNullOrUndefined(queryParamString)) {
      return false;
    }
    const listQueryParam = this._getListQueryParameters(queryParamString);
    return isNotNullNorUndefined(listQueryParam.find(queryParam => queryParam.startsWith(desiredQueryParam + this._QUERY_PARAM_VALUE_AFFECTION_CHAR)));
  }

  private static _preserveOnlyQueryParametersOauth(listQueryParam: string[]): string[] {
    return listQueryParam.filter(query => query.startsWith(this._QUERY_PARAM_OAUTH_CODE + this._QUERY_PARAM_VALUE_AFFECTION_CHAR)
      || query.startsWith(this._QUERY_PARAM_OAUTH_STATE + this._QUERY_PARAM_VALUE_AFFECTION_CHAR));
  }

  private static _removeOnlyUnwantedQueryParameters(listQueryParam: string[]): string[] {
    return listQueryParam.filter(query => !query.startsWith(this._QUERY_PARAM_OAUTH_CODE + this._QUERY_PARAM_VALUE_AFFECTION_CHAR)
      && !query.startsWith(this._QUERY_PARAM_OAUTH_STATE + this._QUERY_PARAM_VALUE_AFFECTION_CHAR)
      && !query.startsWith(this._QUERY_PARAM_ORCID + this._QUERY_PARAM_VALUE_AFFECTION_CHAR),
    );
  }

  private static _getQueryParameters(): string {
    return window.location.href.split(this._QUERY_PARAM_START_CHAR)[1];
  }

  private static _getListQueryParameters(queryParam: string): string[] {
    return queryParam.split(this._QUERY_PARAM_SEPARATOR_CHAR);
  }

  private static _getQueryParametersString(listQueryParam: string[]): string {
    return this._QUERY_PARAM_START_CHAR + listQueryParam.join(this._QUERY_PARAM_SEPARATOR_CHAR);
  }
}

