export interface ArchiveMetadata {
  resId?: string;
  index: string;
  type: string;
  metadata: ArchiveMetadata.PublicMetadata;
  _links?: object;
}

export enum MetadataEnum {
  aipDispositionApproval = "aip-disposition-approval",
  aipOrganizationalUnit = "aip-organizational-unit",
  aipRetention = "aip-retention",
  aipRetentionEnd = "aip-retention-end",
  aipSize = "aip-size",
  aipFiles = "aip-file-number",
  dataciteXml = "datacite.xml",
  dlcmXml = "dlcm.xml",
  aipContainer = "aip-container",

  /* PRIVATE */
  premis = "premis",
  premisComplexType = "premisComplexType",

  premisAgent = "premis:agent",
  premisAgentIdentifier = "premis:agentIdentifier",
  premisAgentIdentifierType = "premis:agentIdentifierType",
  premisAgentIdentifierValue = "premis:agentIdentifierValue",
  premisAgentType = "premis:agentType",
  premisAgentVersion = "premis:agentVersion",

  premisEvent = "premis:event",
  premisEventDateTime = "premis:eventDateTime",
  premisEventIdentifier = "premis:eventIdentifier",
  premisEventIdentifierType = "premis:eventIdentifierType",
  premisEventIdentifierValue = "premis:eventIdentifierValue",
  premisEventType = "premis:eventType",
  premisLinkingObjectIdentifier = "premis:LinkingObjectIdentifier",
  premisLinkingObjectIdentifierType = "premis:LinkingObjectIdentifierType",
  premisLinkingObjectIdentifierValue = "premis:LinkingObjectIdentifierValue",

  premisObject = "premis:object",
  premisOriginalName = "premis:originalMail",
  premisObjectIdentifier = "premis:objectIdentifier",
  premisObjectIdentifierType = "premis:objectIdentifierType",
  premisObjectIdentifierValue = "premis:objectIdentifierValue",

  version = "version",

  /* PUBLIC */
  dataciteResource = "datacite:resource",
  datacite = "datacite",
  xmlnsDatacite = "xmlns:datacite",
  dataciteDates = "datacite:dates",
  dataciteDate = "datacite:date",
  dataciteTitles = "datacite:titles",
  dataciteTitle = "datacite:title",
  dataciteResourceType = "datacite:resourceType",
  resourceTypeGeneral = "resourceTypeGeneral",
  content = "content",
  dataciteRelatedIdentifiers = "datacite:relatedIdentifiers",
  dataciteRelatedIdentifier = "datacite:relatedIdentifier",
  relationType = "relationType",
  relatedIdentifierType = "relatedIdentifierType",
  xmlnsDlcm = "xmlns:dlcm",
  xmlnsMets = "xmlns:mets",
  dataciteFormats = "datacite:formats",
  dataciteFormat = "datacite:format",
  dataciteIdentifier = "datacite:identifier",
  identifierType = "identifierType",
  dataciteRightsList = "datacite:rightsList",
  dataciteRights = "datacite:rights",
  rightsURI = "rightsURI",
  xmlnsXlink = "xmlns:xlink",
  datacitePublicationYear = "datacite:publicationYear",
  datacitePublisher = "datacite:publisher",
  dataciteDescriptions = "datacite:descriptions",
  dataciteDescription = "datacite:description",
  descriptionType = "descriptionType",
  dataciteCreators = "datacite:creators",
  dataciteCreator = "datacite:creator",
  dataciteCreatorName = "datacite:creatorName",
  xmlnsFits = "xmlns:fits",
  xmlnsPremis = "xmlns:premis",
  aipUnit = "aip-unit",
  creation = "creation",
  dateType = "dateType",
  rightsUtiForAccessLevelFinal = "https://www.dlcm.ch/access-level/final",
  rightsUtiForAccessLevel = "https://www.dlcm.ch",
}

export namespace ArchiveMetadata {

  export abstract class Metadata {
    [MetadataEnum.aipContainer]: string;
    [MetadataEnum.aipDispositionApproval]: string;
    [MetadataEnum.aipOrganizationalUnit]: string;
    [MetadataEnum.aipRetention]: string;
    [MetadataEnum.aipRetentionEnd]: string;
    [MetadataEnum.aipUnit]: string;
    [MetadataEnum.aipSize]: number;
    [MetadataEnum.aipFiles]: number;
    [MetadataEnum.creation]: string;
  }

  export class PublicMetadata extends Metadata {
    [MetadataEnum.datacite]: Datacite;
    [MetadataEnum.dataciteXml]: string;
  }

  export class PrivateMetadata extends Metadata {
    [MetadataEnum.premis]: Premis;
    [MetadataEnum.dlcmXml]: string;
  }

  /* PRIVATE */
  export class Premis {
    [MetadataEnum.premisComplexType]: PremisComplexType;
  }

  export class PremisComplexType {
    [MetadataEnum.premisAgent]: PremisAgent[];
    [MetadataEnum.premisEvent]: PremisEvent[];
    [MetadataEnum.premisObject]: PremisObject[];
    [MetadataEnum.version]: number;
  }

  export class PremisAgent {
    [MetadataEnum.premisAgentIdentifier]: PremisAgentIdentifier;
    [MetadataEnum.premisAgentType]: string;
    [MetadataEnum.premisAgentVersion]: string;
  }

  export class PremisAgentIdentifier {
    [MetadataEnum.premisAgentIdentifierType]: string;
    [MetadataEnum.premisAgentIdentifierValue]: string;
  }

  export class PremisEvent {
    [MetadataEnum.premisEventIdentifier]: PremisEventIdentifier;
    [MetadataEnum.premisEventType]: string;
    [MetadataEnum.premisEventDateTime]: string;
    [MetadataEnum.premisLinkingObjectIdentifier]: PremisLinkingObjectIdentifier;
  }

  export class PremisEventIdentifier {
    [MetadataEnum.premisEventIdentifierType]: string;
    [MetadataEnum.premisEventIdentifierValue]: string;
  }

  export class PremisLinkingObjectIdentifier {
    [MetadataEnum.premisLinkingObjectIdentifierType]: string;
    [MetadataEnum.premisLinkingObjectIdentifierValue]: string;
  }

  export class PremisObject {
    [MetadataEnum.premisObjectIdentifier]: PremisObjectIdentifier;
    [MetadataEnum.premisOriginalName]: string;
  }

  export class PremisObjectIdentifier {
    [MetadataEnum.premisObjectIdentifierType]: string;
    [MetadataEnum.premisObjectIdentifierValue]: string;
  }

  /* PUBLIC */
  export class Datacite {
    [MetadataEnum.dataciteResource]: DataciteResource;
  }

  export class DataciteResource {
    [MetadataEnum.dataciteCreators]: DataciteCreators;
    [MetadataEnum.dataciteDates]: DataciteDates;
    [MetadataEnum.dataciteDescriptions]: DataciteDescriptions;
    [MetadataEnum.dataciteFormats]: DataciteFormats;
    [MetadataEnum.dataciteIdentifier]: DataciteIdentifier;
    [MetadataEnum.datacitePublicationYear]: number;
    [MetadataEnum.datacitePublisher]: string;
    [MetadataEnum.dataciteRelatedIdentifiers]: DataciteRelatedIdentifiers;
    [MetadataEnum.dataciteResourceType]: DataciteResourceType;
    [MetadataEnum.dataciteRightsList]: DataciteRightsList;
    [MetadataEnum.dataciteTitles]: DataciteTitles;
  }

  export class DataciteCreators {
    [MetadataEnum.dataciteCreator]: DataciteCreator;
  }

  export class DataciteCreator {
    [MetadataEnum.dataciteCreatorName]: string;
  }

  export class DataciteDates {
    [MetadataEnum.dataciteDate]: DataciteDate;
  }

  export class DataciteDate {
    [MetadataEnum.content]: string;
    [MetadataEnum.dateType]: string;
  }

  export class DataciteDescriptions {
    [MetadataEnum.dataciteDescription]: DataciteDescription[];
  }

  export class DataciteDescription {
    [MetadataEnum.descriptionType]: string;
    [MetadataEnum.content]?: string;
  }

  export class DataciteFormats {
    [MetadataEnum.dataciteFormat]: string[];
  }

  export class DataciteIdentifier {
    [MetadataEnum.content]: string;
    [MetadataEnum.identifierType]: string;
  }

  export class DataciteRelatedIdentifiers {
    [MetadataEnum.dataciteRelatedIdentifier]: DataciteRelatedIdentifier;
  }

  export class DataciteRelatedIdentifier {
    [MetadataEnum.content]: string;
    [MetadataEnum.relatedIdentifierType]: string;
    [MetadataEnum.relationType]: string;
  }

  export class DataciteResourceType {
    [MetadataEnum.content]: string;
    [MetadataEnum.resourceTypeGeneral]: string;
  }

  export class DataciteRightsList {
    [MetadataEnum.dataciteRights]: DataciteRights[];
  }

  export class DataciteRights {
    [MetadataEnum.content]: string;
    [MetadataEnum.rightsURI]: string;
  }

  export class DataciteTitles {
    [MetadataEnum.dataciteTitle]: string[];
  }
}

