import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {
  BaseResourceType,
  isNullOrUndefined,
  KeyValue,
  OrderEnum,
  ResourceNameSpace,
  ResourceState,
} from "solidify-frontend";

export interface DataTableColumns<TResource extends BaseResourceType = any> {
  field: keyof TResource & string;
  header: string;
  tooltip?: (value: string) => string | undefined;
  type?: FieldTypeEnum;
  order: OrderEnum;
  filterEnum?: KeyValue[];
  isMultipleFilterEnum?: boolean;
  translate?: boolean;
  isSortable?: boolean;
  isFilterable?: boolean;
  filterableField?: keyof TResource & string;
  sortableField?: keyof TResource & string;
  resourceNameSpace?: ResourceNameSpace;
  resourceState?: ResourceState<any, any>;
  component?: DataTableComponentEnum;
  width?: string | undefined;
}

export class DataTableColumnsUtil<TResource extends BaseResourceType = any> {
  static getFilterableField<TResource>(column: DataTableColumns<TResource>): keyof TResource & string {
    return isNullOrUndefined(column.filterableField) ? column.field : column.filterableField;
  }

  static getSortableField<TResource>(column: DataTableColumns<TResource>): keyof TResource & string {
    return isNullOrUndefined(column.sortableField) ? column.field : column.sortableField;
  }
}
