import {Injectable} from "@angular/core";
import {environment} from "@environments/environment";
import {AdminResourceApiEnum} from "@shared/enums/api.enum";
import {ApiService} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class OrcidService {
  constructor(private readonly _apiService: ApiService) {
  }

  public startOrcidAuth(): void {
    const startOrcidAuthUrl = AdminResourceApiEnum.orcid + "/start-orcid-auth";
    const originUrl: string = window.location.href;
    let orcidQueryParam = environment.orcidQueryParam + "=true";
    if (originUrl.includes("?")) {
      orcidQueryParam = "&" + orcidQueryParam;
    } else {
      orcidQueryParam = "?" + orcidQueryParam;
    }

    this._apiService.get<string>(startOrcidAuthUrl, {}, {"originUrl": originUrl + orcidQueryParam})
      .subscribe(val => this.getOrcid(val["authTransactionId"]));
  }

  private getOrcid(authTransactionId: string): void {
    const authorizeUrl: string = environment.orcidAuthorizeUrl;
    const clientId: string = environment.orcidClientId;
    const redirectUri: string = AdminResourceApiEnum.orcid + "/landing?authTransactionId=" + authTransactionId;
    window.location.href = authorizeUrl
      + "?client_id=" + clientId
      + "&response_type=code"
      + "&scope=/authenticate"
      + "&redirect_uri=" + redirectUri;
  }
}
