import {Injectable} from "@angular/core";
import {SharedInstitutionAction} from "@app/shared/stores/institution/shared-institution.action";
import {
  SharedInstitutionState,
  SharedInstitutionStateModel,
} from "@app/shared/stores/institution/shared-institution.state";
import {SharedLanguageAction} from "@app/shared/stores/language/shared-language.action";
import {
  SharedLanguageState,
  SharedLanguageStateModel,
} from "@app/shared/stores/language/shared-language.state";
import {SharedLicenseAction} from "@app/shared/stores/license/shared-license.action";
import {
  SharedLicenseState,
  SharedLicenseStateModel,
} from "@app/shared/stores/license/shared-license.state";
import {
  SharedOrganizationalUnitState,
  SharedOrganizationalUnitStateModel,
} from "@app/shared/stores/organizational-unit/shared-organizational-unit.state";
import {SharedPersonAction} from "@app/shared/stores/person/shared-person.action";
import {
  SharedPersonState,
  SharedPersonStateModel,
} from "@app/shared/stores/person/shared-person.state";
import {SharedPreservationPolicyAction} from "@app/shared/stores/preservation-policy/shared-preservation-policy.action";
import {
  SharedPreservationPolicyState,
  SharedPreservationPolicyStateModel,
} from "@app/shared/stores/preservation-policy/shared-preservation-policy.state";
import {SharedAction} from "@app/shared/stores/shared.action";
import {SharedSubmissionPolicyAction} from "@app/shared/stores/submission-policy/shared-submission-policy.action";
import {
  SharedSubmissionPolicyState,
  SharedSubmissionPolicyStateModel,
} from "@app/shared/stores/submission-policy/shared-submission-policy.state";
import {
  Action,
  Actions,
  ofActionCompleted,
  State,
  StateContext,
} from "@ngxs/store";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {SharedDisseminationPolicyAction} from "@shared/stores/dissemination-policy/shared-dissemination-policy.action";
import {
  SharedDisseminationPolicyState,
  SharedDisseminationPolicyStateModel,
} from "@shared/stores/dissemination-policy/shared-dissemination-policy.state";
import {SharedFundingAgencyAction} from "@shared/stores/funding-agency/shared-funding-agency.action";
import {
  SharedFundingAgencyState,
  SharedFundingAgencyStateModel,
} from "@shared/stores/funding-agency/shared-funding-agency.state";
import {
  SharedNotificationState,
  SharedNotificationStateModel,
} from "@shared/stores/notification/shared-notification.state";
import {SharedResearchDomainAction} from "@shared/stores/research-domain/shared-research-domain.action";
import {
  SharedResearchDomainState,
  SharedResearchDomainStateModel,
} from "@shared/stores/research-domain/shared-research-domain.state";
import {SharedRoleAction} from "@shared/stores/role/shared-role.action";
import {
  SharedRoleState,
  SharedRoleStateModel,
} from "@shared/stores/role/shared-role.state";
import {
  SharedVisualizationStateModel,
  VisualizationState,
} from "@shared/stores/visualization/visualization.state";
import {
  BaseStateModel,
  StoreUtil,
} from "solidify-frontend";

export interface SharedStateModel extends BaseStateModel {
  shared_organizationalUnit: SharedOrganizationalUnitStateModel;
  shared_language: SharedLanguageStateModel;
  shared_license: SharedLicenseStateModel;
  shared_submissionPolicy: SharedSubmissionPolicyStateModel;
  shared_preservationPolicy: SharedPreservationPolicyStateModel;
  shared_disseminationPolicy: SharedDisseminationPolicyStateModel;
  shared_person: SharedPersonStateModel;
  shared_institution: SharedInstitutionStateModel;
  shared_role: SharedRoleStateModel;
  shared_visualization: SharedVisualizationStateModel;
  shared_fundingAgency: SharedFundingAgencyStateModel;
  shared_researchDomain: SharedResearchDomainStateModel;
  shared_notification: SharedNotificationStateModel;
}

@Injectable()
@State<SharedStateModel>({
  name: LocalStateEnum.shared,
  defaults: {
    isLoadingCounter: 0,
    shared_organizationalUnit: null,
    shared_language: null,
    shared_license: null,
    shared_preservationPolicy: null,
    shared_submissionPolicy: null,
    shared_disseminationPolicy: null,
    shared_person: null,
    shared_institution: null,
    shared_role: null,
    shared_fundingAgency: null,
    shared_visualization: null,
    shared_researchDomain: null,
    shared_notification: null,
  },
  children: [
    SharedOrganizationalUnitState,
    SharedLanguageState,
    SharedLicenseState,
    SharedSubmissionPolicyState,
    SharedPreservationPolicyState,
    SharedDisseminationPolicyState,
    SharedPersonState,
    SharedInstitutionState,
    SharedRoleState,
    SharedFundingAgencyState,
    VisualizationState,
    SharedResearchDomainState,
    SharedNotificationState,
  ],
})
export class SharedState {

  constructor(private actions$: Actions) {
  }

  @Action(SharedAction.GetAllResources)
  getAllResources(ctx: StateContext<SharedStateModel>, action: SharedAction.GetAllResources): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(
      ctx,
      [
        {
          action: new SharedLanguageAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedLanguageAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedLanguageAction.GetAllFail)),
          ],
        },
        {
          action: new SharedRoleAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedRoleAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedRoleAction.GetAllFail)),
          ],
        },
        {
          action: new SharedLicenseAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedLicenseAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedLicenseAction.GetAllFail)),
          ],
        },
        {
          action: new SharedPreservationPolicyAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedPreservationPolicyAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedPreservationPolicyAction.GetAllFail)),
          ],
        },
        {
          action: new SharedSubmissionPolicyAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedSubmissionPolicyAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedSubmissionPolicyAction.GetAllFail)),
          ],
        },
        {
          action: new SharedDisseminationPolicyAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedDisseminationPolicyAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedDisseminationPolicyAction.GetAllFail)),
          ],
        },
        {
          action: new SharedPersonAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedPersonAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedPersonAction.GetAllFail)),
          ],
        },
        {
          action: new SharedInstitutionAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedInstitutionAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedInstitutionAction.GetAllFail)),
          ],
        },
        {
          action: new SharedFundingAgencyAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedFundingAgencyAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedFundingAgencyAction.GetAllFail)),
          ],
        },
        {
          action: new SharedResearchDomainAction.GetAll(),
          subActionCompletions: [
            this.actions$.pipe(ofActionCompleted(SharedResearchDomainAction.GetAllSuccess)),
            this.actions$.pipe(ofActionCompleted(SharedResearchDomainAction.GetAllFail)),
          ],
        },
      ],
    ).subscribe(isSuccess => {
      if (isSuccess) {
        ctx.dispatch(new SharedAction.GetAllResourcesSuccess(action));
      } else {
        ctx.dispatch(new SharedAction.GetAllResourcesFail(action));
      }
    });
  }

  @Action(SharedAction.GetAllResourcesSuccess)
  getAllResourcesSuccess(ctx: StateContext<SharedStateModel>, action: SharedAction.GetAllResourcesSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(SharedAction.GetAllResourcesFail)
  getAllResourcesFail(ctx: StateContext<SharedStateModel>): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }
}



