import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {SessionStorageEnum} from "@app/shared/enums/session-storage.enum";
import {OrganizationalUnit} from "@models";
import {LanguagesEnum} from "@shared/enums/languages.enum";
import {isNullOrUndefined} from "solidify-frontend";
import {environment} from "../../../environments/environment";

export class UserPreferencesUtil {
  static getPreferredLanguage(): LanguagesEnum {
    const savedLanguage = sessionStorage.getItem(SessionStorageEnum.language);
    if (savedLanguage === null) {
      return this.determineDefaultLanguageAndStoreIntoSessionStorage();
    }
    return savedLanguage as LanguagesEnum;
  }

  private static determineDefaultLanguageAndStoreIntoSessionStorage(): LanguagesEnum {
    const defaultLanguageDetermined = UserPreferencesUtil.determineDefaultLanguage();
    sessionStorage.setItem(SessionStorageEnum.language, defaultLanguageDetermined);
    return defaultLanguageDetermined;
  }

  private static determineDefaultLanguage(): LanguagesEnum {
    const languageFromLocalBrowser = UserPreferencesUtil.findCorrespondingLanguageToLocal();
    if (languageFromLocalBrowser === null) {
      return environment.defaultLanguage;
    }
    return languageFromLocalBrowser;
  }

  private static findCorrespondingLanguageToLocal(): LanguagesEnum {
    const browserLanguage: string = navigator.language;

    if (browserLanguage.startsWith(LanguagesEnum.en)) {
      return LanguagesEnum.en;
    }
    if (browserLanguage.startsWith(LanguagesEnum.fr)) {
      return LanguagesEnum.fr;
    }
    if (browserLanguage.startsWith(LanguagesEnum.de)) {
      return LanguagesEnum.de;
    }

    return null;
  }

  static getPreferredHomePageSearchScope(): SearchScopeEnum {
    const searchScope = sessionStorage.getItem(SessionStorageEnum.homePageSearchScope);
    if (searchScope === null) {
      return SearchScopeEnum.public;
    }
    return searchScope as SearchScopeEnum;
  }

  static getPreferredOrgUnitInDepositMenu(listAuthorizedOrgUnit: OrganizationalUnit[]): OrganizationalUnit | undefined {
    const orgUnitId = sessionStorage.getItem(SessionStorageEnum.orgUnitForDepositMenu);
    if (isNullOrUndefined(listAuthorizedOrgUnit) || listAuthorizedOrgUnit.length === 0) {
      return undefined;
    }
    const defaultOrgUnit = listAuthorizedOrgUnit[0];
    if (isNullOrUndefined(orgUnitId)) {
      return defaultOrgUnit;
    }
    let authorizedOrgUnit = listAuthorizedOrgUnit.find(o => o.resId === orgUnitId);
    if (isNullOrUndefined(authorizedOrgUnit)) {
      authorizedOrgUnit = defaultOrgUnit;
    }
    return authorizedOrgUnit;
  }

  static setPreferredOrgUnitInDepositMenu(orgUnitId: string): string | undefined {
    sessionStorage.setItem(SessionStorageEnum.orgUnitForDepositMenu, orgUnitId);
    if (isNullOrUndefined(orgUnitId)) {
      return undefined;
    }
    return orgUnitId;
  }
}
