import {Injectable} from "@angular/core";
import {
  AppCartArchiveAction,
  appCartArchiveNamespace,
} from "@app/stores/cart/archive/app-cart-archive.action";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {AccessResourceApiEnum} from "@shared/enums/api.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {ArchiveMetadata} from "@shared/models/business/archive-metadata.model";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  OverrideDefaultAction,
  ResourceState,
  ResourceStateModel,
} from "solidify-frontend";

export interface AppCartArchiveStateModel extends ResourceStateModel<ArchiveMetadata> {
}

@Injectable()
@State<AppCartArchiveStateModel>({
  name: LocalStateEnum.application_cart_archive,
  defaults: {
    ...defaultResourceStateInitValue(),
  },
})
export class AppCartArchiveState extends ResourceState<AppCartArchiveStateModel, ArchiveMetadata> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: appCartArchiveNamespace,
    });
  }

  protected get _urlResource(): string {
    return AccessResourceApiEnum.publicMetadata;
  }

  @OverrideDefaultAction()
  @Action(AppCartArchiveAction.GetByListId)
  getByListId(ctx: StateContext<AppCartArchiveStateModel>, action: AppCartArchiveAction.GetByListId): void {
    ctx.patchState({
      list: undefined,
    });
    super.getByListId(ctx, action);
  }

  @OverrideDefaultAction()
  @Action(AppCartArchiveAction.GetByListIdSuccess)
  getByListIdSuccess(ctx: StateContext<AppCartArchiveStateModel>, action: AppCartArchiveAction.GetByListIdSuccess): void {
    super.getByListIdSuccess(ctx, action);
    const list = ctx.getState().list;
    ctx.patchState({
      total: list ? list.length : 0,
    });
  }
}
