import {Injectable} from "@angular/core";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {AppPersonAction} from "@app/stores/person/app-person.action";
import {
  AppUserAction,
  appUserActionNameSpace,
} from "@app/stores/user/app-user.action";
import {User} from "@models";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiActionEnum} from "@shared/enums/api-action.enum";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  CollectionTyped,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  SolidifyStateError,
  urlSeparator,
} from "solidify-frontend";
import {environment} from "../../../environments/environment";

export interface AppUserStateModel extends ResourceStateModel<User> {
}

@Injectable()
@State<AppUserStateModel>({
  name: LocalStateEnum.application_user,
  defaults: {
    ...defaultResourceStateInitValue(),
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class AppUserState extends ResourceState<AppUserStateModel, User> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: appUserActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.users;
  }

  @Action(AppUserAction.GetCurrentUser)
  getCurrentUser(ctx: StateContext<AppUserStateModel>, action: AppUserAction.GetCurrentUser): Observable<CollectionTyped<User>> {
    return this.apiService.get<User>(AdminResourceApiEnum.users + urlSeparator + ApiActionEnum.AUTHENTICATED, null)
      .pipe(
        tap((user: User) => {
          ctx.dispatch(new AppUserAction.GetCurrentUserSuccess(action, user));
        }),
        catchError(error => {
          ctx.dispatch(new AppUserAction.GetCurrentUserFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(AppUserAction.GetCurrentUserSuccess)
  getCurrentUserSuccess(ctx: StateContext<AppUserStateModel>, action: AppUserAction.GetCurrentUserSuccess): void {
    ctx.patchState({
      current: action.user,
    });

    ctx.dispatch(new AppPersonAction.AddCurrentPerson(action.user.person));
  }
}
