import {LanguagesEnum} from "@app/shared/enums/languages.enum";
import {ThemeEnum} from "@app/shared/enums/theme.enum";
import {ChemicalMoleculeVisualizationEnum} from "@shared/enums/chemical-molecule-visualization.enum";
import {DefaultSolidifyEnvironment} from "solidify-frontend";

export interface DlcmEnvironment extends DefaultSolidifyEnvironment {
  appTitle: string;
  isDemoMode: boolean;
  theme: ThemeEnum;
  defaultLanguage: LanguagesEnum;
  defaultPageSize: number;
  defaultPageSizeHomePage: number;
  defaultEnumValuePageSizeOption: number;
  defaultEnumValuePageSizeLazyLoad: number;
  pageSizeOptions: number[];
  defaultHomeOrgUnitSize: number;
  minimalPageSizeToRetrievePaginationInfo: number;
  maximalPageSizeToRetrievePaginationInfo: number;
  useSelfHostedStreamSaverMitm: boolean; // If false, use fallback service worker on github. If true, use local service worker and allow to avoid message downloaded from https://jimmywarting.github.io/StreamSaver.js
  computeOAuthEndpoint: boolean;
  // Base URL For Module Applications (override by AppAction.LoadModules)
  preservation_planning?: string | undefined;
  authorization?: string | undefined;
  admin: string;
  ingest?: string | undefined;
  preingest?: string | undefined;
  access?: string | undefined;
  archivalStorage?: string[] | undefined;
  dataManagement?: string | undefined;

  // Documentation
  documentationTocApisGuide: string | undefined;
  documentationTocIntegrationGuide: string | undefined;
  documentationTocToolsGuide: string | undefined;
  documentationTocUserGuide: string | undefined;

  // OAuth2 properties
  tokenInMemoryStorage: boolean;
  reloadAfterLogout: boolean;

  // Orcid properties
  orcidQueryParam: string;
  orcidAuthorizeUrl: string;
  orcidClientId: string;

  // Breakpoints
  breakpointXs: string;
  breakpointSm: string;
  breakpointMd: string;
  breakpointLg: string;

  // Others
  modalHeight: string;
  modalWidth: string;
  showDebugInformation: boolean;
  institutionUrl: string;

  // Spinner
  defaultSpinnerStrokeWidth: number;
  defaultSpinnerDiameter: number;
  defaultButtonSpinnerStrokeWidth: number;
  defaultButtonSpinnerDiameter: number;

  // RSS
  rssUrl: string;
  rssMaxNumberOfNewsToDisplay: number;

  // Tweeter
  twitterAccount: string;
  twitterTweetToDisplay: number;

  // Carousel
  frequencyChangeCarouselTileInSecond: number;

  urlNationalArchivePronom: string;

  // Tooltip duration
  timeBeforeDisplayTooltipOnDataTable: number;
  timeBeforeDisplayTooltipOnInput: number;

  doiLink: string;

  // Polling
  refreshTabStatusCounterIntervalInSecond: number;
  refreshOrderAvailableIntervalInSecond: number;
  refreshNotificationInboxAvailableIntervalInSecond: number;
  refreshDepositSubmittedIntervalInSecond: number;

  cdkDropListIdPrefix: string;
  classInputIgnored: string;

  // visualization
  visualizationMaxSizeFile: number;

  visualizationChemicalMoleculeMode: ChemicalMoleculeVisualizationEnum;
  visualizationChemicalMolecule2dLibs: string[];
  visualizationChemicalMolecule3dLib: string;

  // image constant
  visualizationImageExtensions: string[];
  visualizationImageContentType: string[];
  visualizationImageMimeType: string[];
  visualizationImagePronomId: string[];

  // movie constant for all movie
  visualizationMovieExtensions: string[];
  visualizationMovieContentType: string[];
  visualizationMovieMimeType: string[];
  visualizationMoviePronomId: string[];

  // movie constant
  visualizationClassicalMovieExtensions: string[];
  visualizationClassicalMovieContentType: string[];
  visualizationClassicalMovieMimeType: string[];
  visualizationClassicalMoviePronomId: string[];

  // pdf constant
  visualizationPdfExtensions: string[];
  visualizationPdfContentType: string[];
  visualizationPdfMimeType: string[];
  visualizationPdfPronomId: string[];

  // office constant
  visualizationOfficeFileExtensions: string[];
  visualizationOfficeContentType: string[];
  visualizationOfficeMimeType: string[];
  visualizationOfficePronomId: string[];

  // ng2-xs- viewer
  visualizationNg2XsViewerExtensions: string[];
  visualizationNg2XsViewerContentType: string[];
  visualizationNg2XsViewerMimeType: string[];
  visualizationNg2XsViewerPronomId: string[];

  // sound constant
  visualizationSoundExtension: string[];
  visualizationSoundContentType: string[];
  visualizationSoundMimeType: string[];
  visualizationSoundPronomId: string[];

  // mol constant
  visualizationMolExtension: string[];
  visualizationMolContentType: string[];
  visualizationMolMimeType: string[];
  visualizationMolPronomId: string[];

  // sme constant
  visualizationSmeExtension: string[];
  visualizationSmeContentType: string[];
  visualizationSmeMimeType: string[];
  visualizationSmePronomId: string[];

  // text constant
  visualizationTextFileExtension: string[];
  visualizationTextFileContentType: string[];
  visualizationTextFileMimeType: string[];
  visualizationTextFilePronomId: string[];
}
