// This file should contains all the defaults settings shared in all environment files
// If you want to override properties for specific environment, do it in these files

import {ErrorBackendKeyEnum} from "@app/shared/enums/error-backend-key.enum";
import {LanguagesEnum} from "@app/shared/enums/languages.enum";
import {ThemeEnum} from "@app/shared/enums/theme.enum";
import {SharedSnackbarPresentational} from "@shared/components/presentationals/shared-snackbar/shared-snackbar.presentational";
import {ChemicalMoleculeVisualizationEnum} from "@shared/enums/chemical-molecule-visualization.enum";
import {MARK_AS_TRANSLATABLE} from "solidify-frontend";
import {DlcmEnvironment} from "./environment.defaults.model";

export const defaultEnvironment: DlcmEnvironment = {
  appTitle: "DLCM - Portal",
  production: false,
  forceRuntimeOverrideSetting: false,
  isDemoMode: false,

  theme: ThemeEnum.dlcm,
  defaultLanguage: LanguagesEnum.en,
  defaultPageSize: 10,
  defaultPageSizeHomePage: 10,
  defaultEnumValuePageSizeOption: 10000,
  defaultEnumValuePageSizeLazyLoad: 20,
  pageSizeOptions: [5, 10, 25, 50, 100],
  defaultHomeOrgUnitSize: 1000,
  minimalPageSizeToRetrievePaginationInfo: 1,
  maximalPageSizeToRetrievePaginationInfo: 2000,
  useSelfHostedStreamSaverMitm: false,
  computeOAuthEndpoint: false,

  // Base URL For Module Applications (override by AppAction.LoadModules)
  admin: "http://localhost:16115/dlcm/admin",

  // Documentation
  documentationTocApisGuide: "DLCM-APIs-toc.html",
  documentationTocIntegrationGuide: "DLCM-IntegrationGuide-toc.html",
  documentationTocToolsGuide: "DLCM-ToolsGuide-toc.html",
  documentationTocUserGuide: undefined,

  // OAuth2 properties
  oidc: false,
  requestAccessToken: true,
  tokenEndpoint: "http://localhost:4200/dlcm/oauth/token",
  dummyClientSecret: "123abc",
  clientId: "local-dev-angular",
  responseType: "token",
  scope: "READ",
  requireHttps: false,
  loginUrl: "http://localhost:4200/dlcm/oauth/authorize",
  allowedUrls: [],
  redirectUrl: window.location.origin,
  timeoutFactor: 0.75,
  postLogoutRedirectUri: "",
  logoutUrl: "",
  nonceStateSeparator: ";",
  disableNonceCheck: false,
  sendAccessToken: true,
  tokenInMemoryStorage: false,
  reloadAfterLogout: false,

  // Orcid
  orcidQueryParam: "orcid",
  orcidAuthorizeUrl: "https://orcid.org/oauth/authorize",
  orcidClientId: "APP-J5BJILONUJDONCIN",

  // Breakpoint
  breakpointXs: "0px",
  breakpointSm: "576px",
  breakpointMd: "992px",
  breakpointLg: "1360px",

  // Others
  defaultNotificationComponent: SharedSnackbarPresentational,
  defaultNotificationErrorDurationInSeconds: 5,
  defaultNotificationSuccessDurationInSeconds: 3,
  defaultNotificationWarningDurationInSeconds: 3,
  defaultNotificationInformationDurationInSeconds: 3,
  modalHeight: "400px",
  modalWidth: "90%",
  showDebugInformation: false,
  institutionUrl: "https://www.unige.ch/",

  // Spinner
  defaultSpinnerStrokeWidth: 3,
  defaultSpinnerDiameter: 75,
  defaultButtonSpinnerStrokeWidth: 3,
  defaultButtonSpinnerDiameter: 24,

  // RSS
  rssUrl: "https://www.unige.ch/feed/rss",
  rssMaxNumberOfNewsToDisplay: 8,

  // Tweeter
  twitterAccount: "dlcm_ch",
  twitterTweetToDisplay: 4,
  frequencyChangeCarouselTileInSecond: 5,

  urlNationalArchivePronom: "https://www.nationalarchives.gov.uk/PRONOM/",

  // Tooltip duration
  timeBeforeDisplayTooltipOnDataTable: 500,
  timeBeforeDisplayTooltipOnInput: 680,

  doiLink: "https://doi.org/",

  // Polling
  refreshTabStatusCounterIntervalInSecond: 60,
  refreshOrderAvailableIntervalInSecond: 5,
  refreshNotificationInboxAvailableIntervalInSecond: 60,
  refreshDepositSubmittedIntervalInSecond: 2,

  // Message
  notificationHttpUnauthorizedToTranslate: MARK_AS_TRANSLATABLE("error.http.unauthorized"),
  notificationHttpInvalidTokenToTranslate: MARK_AS_TRANSLATABLE("error.http.invalidToken"),
  notificationHttpForbiddenToTranslate: MARK_AS_TRANSLATABLE("error.http.forbidden"),
  notificationHttpNotFoundToTranslate: MARK_AS_TRANSLATABLE("error.http.notFound"),
  notificationHttpInternalErrorToTranslate: MARK_AS_TRANSLATABLE("error.http.internalError"),
  notificationHttpOfflineToTranslate: MARK_AS_TRANSLATABLE("error.http.offline"),
  validationErrorRequiredToTranslate: MARK_AS_TRANSLATABLE("validation.error.required"),
  validationErrorInvalidToTranslate: MARK_AS_TRANSLATABLE("validation.error.invalid"),

  // Errors
  httpErrorKeyToSkipInErrorHandler: [
    ErrorBackendKeyEnum.UPLOAD_DUPLICATE_DATA_FILES,
  ],
  errorToSkipInErrorHandler: new Error("ERROR TO SKIP IN ERROR HANDLER"),

  cdkDropListIdPrefix: "cdkDropList-",
  classInputIgnored: "ignored-input",

  // Visualization
  visualizationMaxSizeFile: 300000000,
  visualizationChemicalMoleculeMode: ChemicalMoleculeVisualizationEnum.threeAndTwoDimensional,
  visualizationChemicalMolecule2dLibs: ["JSmolJME.js", "jsme.nocache.js"],
  visualizationChemicalMolecule3dLib: "JSmol.min.js",

  // image constant
  visualizationImageExtensions: ["png", "gif", "jpeg", "tiff", "svg"],
  visualizationImageContentType: ["image/gif", "image/png", "image/jpeg", "image/tiff", "image/svg+xml"],
  visualizationImageMimeType: ["fmt/3", "fmt/4", "fmt/11", "fmt/12", "fmt/13", "fmt/42", "fmt/43", "fmt/44", "fmt/353", "x-fmt/399", "x-fmt/388", "xfmt/387", "fmt/155", "fmt/154", "fmt/153", "fmt/91", "fmt/92", "fmt/413", "x-fmt/109", "fmt/579", "fmt/580", "fmt/581", "fmt/582", "fmt/702"],
  visualizationImagePronomId: [],

  // movie constant for all movie
  visualizationMovieExtensions: ["webm", "flv", "gif", "gifv", "avi", "mov", "qt", "mpg", "mp2", "mpeg", "mpe", "mpv"],
  visualizationMovieContentType: ["video/mp4", "video/dv", "video/mpeg", "video/mj2", "video/xmatroska", "audio/xmatroska", "video/ogg", "audio/ogg", "application/ogg"],
  visualizationMovieMimeType: ["video/mp4", "video/dv", "video/mpeg", "video/mj2", "video/xmatroska", "audio/xmatroska", "video/ogg", "audio/ogg", "application/ogg"],
  visualizationMoviePronomId: ["fmt/199", "x-fmt/152", "fmt/649", "fmt/640", "fmt/337", "fmt/569", "fmt/569", "fmt/203", "fmt/945"],

  // movie constant
  visualizationClassicalMovieExtensions: ["mp4", "ogg", "webm"],
  visualizationClassicalMovieContentType: [],
  visualizationClassicalMovieMimeType: ["video/mp4", "video/ogg", "audio/ogg", "video/webm"],
  visualizationClassicalMoviePronomId: ["fmt/199", "fmt/203", "fmt/944", "fmt/945", "fmt/946", "fmt/947", "fmt/948"],

  // pdf constant
  visualizationPdfExtensions: ["pdf"],
  visualizationPdfContentType: [],
  visualizationPdfMimeType: ["application/pdf"],
  visualizationPdfPronomId: ["fmt/95", "fmt/3", "fmt/476", "fmt/477", "fmt/479", "fmt/480", "fmt/18", "fmt/19", "fmt/20", "fmt/276"],

  // office constant
  visualizationOfficeFileExtensions: ["pdf", "odt", "xls"],
  visualizationOfficeContentType: ["application/pdf", "application/vnd.oasis.opendocument.text", "application/msword", "application/vnd.openxmlformats-officedocument.wordprocessingml.document"],
  visualizationOfficeMimeType: [],
  visualizationOfficePronomId: ["fmt/95", "fmt/3", "fmt/476", "fmt/477", "fmt/479", "fmt/480", "fmt/18", "fmt/19", "fmt/20", "fmt/276", "fmt/290", "fmt/189"],

  // ng2-xs- viewer
  visualizationNg2XsViewerExtensions: ["pdf", "odt", "xls"],
  visualizationNg2XsViewerContentType: [],
  visualizationNg2XsViewerMimeType: ["application/pdf", "application/vnd.oasis.opendocument.text", "application/msword", "application/vnd.openxmlformats-officedocument.wordprocessingml.document"],
  visualizationNg2XsViewerPronomId: ["fmt/95", "fmt/3", "fmt/476", "fmt/477", "fmt/479", "fmt/480", "fmt/18", "fmt/19", "fmt/20", "fmt/276", "fmt/290", "fmt/189"],

  // sound constant
  visualizationSoundExtension: ["mp3", "mp4", "flac", "ogg", "wav", "acc"],
  visualizationSoundContentType: ["audio/mpeg"],
  visualizationSoundMimeType: ["audio/aac ",
    "audio/aacp"],
  visualizationSoundPronomId: [],

  // mol constant
  visualizationMolExtension: ["sdf", "log"],
  visualizationMolContentType: [],
  visualizationMolMimeType: ["text/plain", "chemical/x-jcamp-dx", "chemical/x-sketchel", "chemical/x-kinemage", "chemical/x-macmolecule", "chemical/x-macromodel-input", "chemical/x-daylight-smiles", "chemical/x-isostar", "chemical/x-genbank", "chemical/x-gcg8-sequence", "chemical/x-gaussian-input", "chemical/x-gaussian-cube", "chemical/x-gaussian-checkpoint", "chemical/x-gamess-input", "chemical/x-galactic-spc", "chemical/x-embl-dl-nucleotide", "chemical/x-cxf", "chemical/x-chem3d", "chemical/x-ctx", "chemical/x-csml", "chemical/x-crossfire", "chemical/x-compass", "chemical/x-cml", "chemical/x-cmdf", "chemical/x-cif", "chemical/x-chemdraw", "text/plain", "chemical/x-mdl-sdfile", "chemical/x-mdl-molfile", "chemical/x-alchemy", "chemical/x-cache-csf", "chemical/x-cerius", "chemical/x-cactvs-binary"],
  visualizationMolPronomId: [],

  // sme constant
  visualizationSmeExtension: ["mol"],
  visualizationSmeContentType: ["text/plain"],
  visualizationSmeMimeType: [],
  visualizationSmePronomId: [],

  // text constant
  visualizationTextFileExtension: ["txt", "json", "xml", "java", "yml", "ts", "css", "scss", "ads", "adb", "ada", "bash", "sh", "h", "cpp", "hpp", "cpp", "html", "htm", "shtml", "shtm", "xhtml", "xhtm", "jsp", "mak", "php", "php3", "php4", "php5", "phpml", "phps", "phps", "sql", "yml", "yaml"],
  visualizationTextFileContentType: [],
  visualizationTextFileMimeType: ["text/plain"],
  visualizationTextFilePronomId: [],
};

/*
 * For easier debugging in development mode, you can import the following file
 * to ignore zone related error stack frames such as `zone.run`, `zoneDelegate.invokeTask`.
 *
 * This import should be commented out in production mode because it will have a negative impact
 * on performance if an error is thrown.
 */
// import 'zone.js/dist/zone-error';  // Included with Angular CLI.
